// Bandpass Filter Indicator
// Last changed on 2 Mar 2010
// Part of Forex Strategy Builder v2.8.3.7+
// Website http://forexsb.com/
// This code or any part of it cannot be used in other applications without a permission.
// Copyright (c) 2010 Denny Imanuel - All rights reserved.

using System;
using System.Drawing;

namespace Forex_Strategy_Builder
{
    /// <summary>
    /// Bandpass Filter Indicator
    /// </summary>
    public class Bandpass_Filter : Indicator
    {
        /// <summary>
        /// Sets the default indicator parameters for the designated slot type
        /// </summary>
        public Bandpass_Filter(SlotTypes slotType)
        {
            // General properties
            IndicatorName  = "Bandpass Filter";
            PossibleSlots  = SlotTypes.OpenFilter | SlotTypes.CloseFilter;
            SeparatedChart = true;
			CustomIndicator = true;

            // Setting up the indicator parameters
            IndParam = new IndicatorParam();
            IndParam.IndicatorName = IndicatorName;
            IndParam.SlotType      = slotType;

            // The ComboBox parameters
            IndParam.ListParam[0].Caption  = "Logic";
            IndParam.ListParam[0].ItemList = new string[]
            {
                "The Bandpass Filter line rises",
                "The Bandpass Filter line falls",
                "The Bandpass Filter line is higher than zero",
                "The Bandpass Filter line is lower than zero",
                "The Bandpass Filter line crosses the zero line upward",
                "The Bandpass Filter line crosses the zero line downward",
                "The Bandpass Filter line changes its direction upward",
                "The Bandpass Filter line changes its direction downward",
                "The Bandpass Filter line crosses the Average Peak upward",
                "The Bandpass Filter line crosses the Average Valley downward",
                "The Bandpass Filter line is higher than the Average Peak",
                "The Bandpass Filter line is lower than the Average Valley"
            };
            IndParam.ListParam[0].Index    = 0;
            IndParam.ListParam[0].Text     = IndParam.ListParam[0].ItemList[IndParam.ListParam[0].Index];
            IndParam.ListParam[0].Enabled  = true;
            IndParam.ListParam[0].ToolTip  = "Logic of application of the indicator.";

             IndParam.ListParam[1].Caption  = "Smoothing method";
            IndParam.ListParam[1].ItemList = Enum.GetNames(typeof(MAMethod));
            IndParam.ListParam[1].Index    = (int)MAMethod.Simple;
            IndParam.ListParam[1].Text     = IndParam.ListParam[1].ItemList[IndParam.ListParam[1].Index];
            IndParam.ListParam[1].Enabled  = true;
            IndParam.ListParam[1].ToolTip  = "The smoothing method of Moving Average.";

            IndParam.ListParam[2].Caption  = "Base price";
            IndParam.ListParam[2].ItemList = Enum.GetNames(typeof(BasePrice));
            IndParam.ListParam[2].Index    = (int)BasePrice.Median;
            IndParam.ListParam[2].Text     = IndParam.ListParam[2].ItemList[IndParam.ListParam[2].Index];
            IndParam.ListParam[2].Enabled  = true;
            IndParam.ListParam[2].ToolTip  = "The price the Moving Average is based on.";


			// The NumericUpDown parameters
            IndParam.NumParam[0].Caption = "Bandpass Period";
            IndParam.NumParam[0].Value   = 10;
            IndParam.NumParam[0].Min     = 1;
            IndParam.NumParam[0].Max     = 200;
            IndParam.NumParam[0].Enabled = true;
            IndParam.NumParam[0].ToolTip = "The period of bandpass filter calculation.";
			
			IndParam.NumParam[1].Caption = "Smoothing Period";
            IndParam.NumParam[1].Value   = 20;
            IndParam.NumParam[1].Min     = 1;
            IndParam.NumParam[1].Max     = 200;
            IndParam.NumParam[1].Enabled = true;
            IndParam.NumParam[1].ToolTip = "The period of bandpass filter calculation.";

			IndParam.NumParam[2].Caption = "Bandpass Delta";
            IndParam.NumParam[2].Value   = 0.5;
            IndParam.NumParam[2].Min     = 0;
            IndParam.NumParam[2].Max     = 10;
			IndParam.NumParam[2].Point	 = 2;
            IndParam.NumParam[2].Enabled = true;
            IndParam.NumParam[2].ToolTip = "The delta value for bandpass filter.";
			
			IndParam.NumParam[3].Caption = "Bandwidth Multiplier";
            IndParam.NumParam[3].Value   = 0.1;
            IndParam.NumParam[3].Min     = 0;
            IndParam.NumParam[3].Max     = 10;
			IndParam.NumParam[3].Point	 = 2;
            IndParam.NumParam[3].Enabled = true;
            IndParam.NumParam[3].ToolTip = "The divisor to determine the upper and lower bandwidth.";
			
            // The CheckBox parameters
            IndParam.CheckParam[0].Caption = "Use previous bar value";
            IndParam.CheckParam[0].Checked = PrepareUsePrevBarValueCheckBox(slotType);
            IndParam.CheckParam[0].Enabled = true;
            IndParam.CheckParam[0].ToolTip = "Use the indicator value from the previous bar.";
			
			IndParam.CheckParam[1].Caption = "Use smoothed bandpass filter";
            IndParam.CheckParam[1].Checked = true;
            IndParam.CheckParam[1].Enabled = true;
            IndParam.CheckParam[1].ToolTip = "Use the smoothed value of bandpass filter.";

            return;
        }

        /// <summary>
        /// Calculates the indicator's components
        /// </summary>
        public override void Calculate(SlotTypes slotType)
        {
            // Reading the parameters
            MAMethod  maMethod 	= (MAMethod )IndParam.ListParam[1].Index;
            BasePrice basePrice = (BasePrice)IndParam.ListParam[2].Index;
			int 	iPeriod	= (int)IndParam.NumParam[0].Value;
			int 	iSmooth	= (int)IndParam.NumParam[1].Value;
			double 	Delta	= IndParam.NumParam[2].Value;
			double	dMpl	= IndParam.NumParam[3].Value;
            int 	iPrvs 	= IndParam.CheckParam[0].Checked ? 1 : 0;

            double[] adPrice 	= Price(basePrice);
			double[] adBPFilter = new double[Bars];
			double[] adBPSignal 	= new double[Bars];
			double[] adBPPeak 	= new double[Bars];
			double[] adBPValley	= new double[Bars];
			double[] adAvgPeak 	= new double[Bars];
			double[] adAvgValley 	= new double[Bars];
			
			// Calculation
            int iFirstBar = iPeriod + 2;

			double Beta = Math.Cos(2*Math.PI/iPeriod);
			double Gamma = 1/Math.Cos(4*Math.PI*Delta/iPeriod);
			double Alpha = Gamma - Math.Sqrt(Math.Pow(Gamma,2)-1);
			double Theta = 0.5*(1-Alpha);
			double Omega = Beta*(1+Alpha);
            
			for (int iBar = 2; iBar < Bars; iBar++)
            {
				adBPFilter[iBar] = Theta*(adPrice[iBar] - adPrice[iBar-2]) + Omega*adBPFilter[iBar-1] - Alpha*adBPFilter[iBar-2];
				if (adBPFilter[iBar]<adBPFilter[iBar-1] && adBPFilter[iBar-1]>adBPFilter[iBar-2])
					adBPPeak[iBar] = adBPFilter[iBar-1];
				else
					adBPPeak[iBar] = adBPPeak[iBar-1];
				if (adBPFilter[iBar]>adBPFilter[iBar-1] && adBPFilter[iBar-1]<adBPFilter[iBar-2])
					adBPValley[iBar] = adBPFilter[iBar-1];
				else
					adBPValley[iBar] = adBPValley[iBar-1];
			}

            if (IndParam.CheckParam[1].Checked)
				adBPSignal = MovingAverage(iSmooth, 0, maMethod, adBPFilter);
			else
				adBPSignal = adBPFilter;
			adAvgPeak = MovingAverage(iSmooth, 0, maMethod, adBPPeak);
			adAvgValley = MovingAverage(iSmooth, 0, maMethod, adBPValley);
			
			for (int iBar = 2; iBar < Bars; iBar++)
            {
				adAvgPeak[iBar] = dMpl*adAvgPeak[iBar];
				adAvgValley[iBar] = dMpl*adAvgValley[iBar];
			}
			
			
            // Saving the components
            Component = new IndicatorComp[6];

            Component[0] = new IndicatorComp();
            Component[0].CompName   = "Bandpass Filter";
            Component[0].DataType   = IndComponentType.IndicatorValue;
            Component[0].ChartType  = IndChartType.Line;
            Component[0].ChartColor = Color.Purple;
            Component[0].FirstBar   = iFirstBar;
            Component[0].Value      = adBPFilter;

            Component[1] = new IndicatorComp();
            Component[1].CompName   = "BandPass Signal";
            Component[1].DataType   = IndComponentType.IndicatorValue;
            Component[1].ChartType  = IndChartType.Line;
            Component[1].ChartColor = Color.Pink;
            Component[1].FirstBar   = iFirstBar;
            Component[1].Value      = adBPSignal;

            Component[2] = new IndicatorComp();
            Component[2].ChartType = IndChartType.NoChart;
            Component[2].FirstBar  = iFirstBar;
            Component[2].Value     = new double[Bars];

            Component[3] = new IndicatorComp();
            Component[3].ChartType = IndChartType.NoChart;
            Component[3].FirstBar  = iFirstBar;
            Component[3].Value     = new double[Bars];
			
			Component[4] = new IndicatorComp();
            Component[4].CompName   = "Average Peak";
            Component[4].DataType   = IndComponentType.IndicatorValue;
            Component[4].ChartType  = IndChartType.Line;
            Component[4].ChartColor = Color.Green;
            Component[4].FirstBar   = iFirstBar;
            Component[4].Value      = adAvgPeak;
			
			Component[5] = new IndicatorComp();
            Component[5].CompName   = "Average Valley";
            Component[5].DataType   = IndComponentType.IndicatorValue;
            Component[5].ChartType  = IndChartType.Line;
            Component[5].ChartColor = Color.Red;
            Component[5].FirstBar   = iFirstBar;
            Component[5].Value      = adAvgValley;

            // Sets the Component's type
            if (slotType == SlotTypes.OpenFilter)
            {
                Component[2].DataType = IndComponentType.AllowOpenLong;
                Component[2].CompName = "Is long entry allowed";
                Component[3].DataType = IndComponentType.AllowOpenShort;
                Component[3].CompName = "Is short entry allowed";
            }
            else if (slotType == SlotTypes.CloseFilter)
            {
                Component[2].DataType = IndComponentType.ForceCloseLong;
                Component[2].CompName = "Close out long position";
                Component[3].DataType = IndComponentType.ForceCloseShort;
                Component[3].CompName = "Close out short position";
            }

            switch (IndParam.ListParam[0].Text)
            {
                case "The Bandpass Filter line rises":
                    OscillatorLogic(iFirstBar, iPrvs, adBPSignal, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_rises);
                    break;

                case "The Bandpass Filter line falls":
                    OscillatorLogic(iFirstBar, iPrvs, adBPSignal, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_falls);
                    break;

                case "The Bandpass Filter line is higher than zero":
                    OscillatorLogic(iFirstBar, iPrvs, adBPSignal, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_is_higher_than_the_level_line);
                    break;

                case "The Bandpass Filter line is lower than zero":
                    OscillatorLogic(iFirstBar, iPrvs, adBPSignal, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_is_lower_than_the_level_line);
                    break;

                case "The Bandpass Filter line crosses the zero line upward":
                    OscillatorLogic(iFirstBar, iPrvs, adBPSignal, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_crosses_the_level_line_upward);
                    break;

                case "The Bandpass Filter line crosses the zero line downward":
                    OscillatorLogic(iFirstBar, iPrvs, adBPSignal, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_crosses_the_level_line_downward);
                    break;

                case "The Bandpass Filter line changes its direction upward":
                    OscillatorLogic(iFirstBar, iPrvs, adBPSignal, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_changes_its_direction_upward);
                    break;

                case "The Bandpass Filter line changes its direction downward":
                    OscillatorLogic(iFirstBar, iPrvs, adBPSignal, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_changes_its_direction_downward);
                    break;

                case "The Bandpass Filter line crosses the Average Peak upward":
                    IndicatorCrossesAnotherIndicatorUpwardLogic(iFirstBar, iPrvs, adBPSignal, adAvgPeak, ref Component[2], ref Component[3]);
                    break;

                case "The Bandpass Filter line crosses the Average Valley downward":
                    IndicatorCrossesAnotherIndicatorDownwardLogic(iFirstBar, iPrvs, adBPSignal, adAvgValley, ref Component[2], ref Component[3]);
                    break;

                case "The Bandpass Filter line is higher than the Average Peak":
                    IndicatorIsHigherThanAnotherIndicatorLogic(iFirstBar, iPrvs, adBPSignal, adAvgPeak, ref Component[2], ref Component[3]);
                    break;

                case "The Bandpass Filter line is lower than the Average Valley":
                    IndicatorIsLowerThanAnotherIndicatorLogic(iFirstBar, iPrvs, adBPSignal, adAvgValley, ref Component[2], ref Component[3]);
                    break;

                default:
                    break;
            }

            return;
        }

        /// <summary>
        /// Sets the indicator logic description
        /// </summary>
        public override void SetDescription(SlotTypes slotType)
        {
            EntryFilterLongDescription  = ToString() + "; the Bandpass Filter line ";
            EntryFilterShortDescription = ToString() + "; the Bandpass Filter line ";
            ExitFilterLongDescription   = ToString() + "; the Bandpass Filter line ";
            ExitFilterShortDescription  = ToString() + "; the Bandpass Filter line ";

            switch (IndParam.ListParam[0].Text)
            {
                case "The Bandpass Filter line rises":
                    EntryFilterLongDescription  += "rises";
                    EntryFilterShortDescription += "falls";
                    ExitFilterLongDescription   += "rises";
                    ExitFilterShortDescription  += "falls";
                    break;

                case "The Bandpass Filter line falls":
                    EntryFilterLongDescription  += "falls";
                    EntryFilterShortDescription += "rises";
                    ExitFilterLongDescription   += "falls";
                    ExitFilterShortDescription  += "rises";
                    break;

                case "The Bandpass Filter line is higher than zero":
                    EntryFilterLongDescription  += "is higher than the zero line";
                    EntryFilterShortDescription += "is lower than the zero line";
                    ExitFilterLongDescription   += "is higher than the zero line";
                    ExitFilterShortDescription  += "is lower than the zero line";
                    break;

                case "The Bandpass Filter line is lower than zero":
                    EntryFilterLongDescription  += "is lower than the zero line";
                    EntryFilterShortDescription += "is higher than the zero line";
                    ExitFilterLongDescription   += "is lower than the zero line";
                    ExitFilterShortDescription  += "is higher than the zero line";
                    break;

                case "The Bandpass Filter line crosses the zero line upward":
                    EntryFilterLongDescription  += "crosses the zero line upward";
                    EntryFilterShortDescription += "crosses the zero line downward";
                    ExitFilterLongDescription   += "crosses the zero line upward";
                    ExitFilterShortDescription  += "crosses the zero line downward";
                    break;

                case "The Bandpass Filter line crosses the zero line downward":
                    EntryFilterLongDescription  += "crosses the zero line downward";
                    EntryFilterShortDescription += "crosses the zero line upward";
                    ExitFilterLongDescription   += "crosses the zero line downward";
                    ExitFilterShortDescription  += "crosses the zero line upward";
                    break;

                case "The Bandpass Filter line changes its direction upward":
                    EntryFilterLongDescription  += "changes its direction upward";
                    EntryFilterShortDescription += "changes its direction downward";
                    ExitFilterLongDescription   += "changes its direction upward";
                    ExitFilterShortDescription  += "changes its direction downward";
                    break;

                case "The Bandpass Filter line changes its direction downward":
                    EntryFilterLongDescription  += "changes its direction downward";
                    EntryFilterShortDescription += "changes its direction upward";
                    ExitFilterLongDescription   += "changes its direction downward";
                    ExitFilterShortDescription  += "changes its direction upward";
                    break;

                case "The Bandpass Filter line is higher than the Average Peak":
                    EntryFilterLongDescription  += "is higher than the Average Peak";
                    EntryFilterShortDescription += "is lower than the Average Peak";
                    ExitFilterLongDescription   += "is higher than the Average Peak";
                    ExitFilterShortDescription  += "is lower than the Average Peak";
                    break;

                case "The Bandpass Filter line is lower than the Average Valley":
                    EntryFilterLongDescription  += "is lower than the Average Valley";
                    EntryFilterShortDescription += "is higher than the Average Valley";
                    ExitFilterLongDescription   += "is lower than the Average Valley";
                    ExitFilterShortDescription  += "is higher than the Average Valley";
                    break;

                case "The Bandpass Filter line crosses the Average Peak upward":
                    EntryFilterLongDescription  += "crosses the Average Peak upward";
                    EntryFilterShortDescription += "crosses the Average Peak downward";
                    ExitFilterLongDescription   += "crosses the Average Peak upward";
                    ExitFilterShortDescription  += "crosses the Average Peak downward";
                    break;

                case "The Bandpass Filter line crosses the Average Valley downward":
                    EntryFilterLongDescription  += "crosses the Average Valley downward";
                    EntryFilterShortDescription += "crosses the Average Valley upward";
                    ExitFilterLongDescription   += "crosses the Average Valley downward";
                    ExitFilterShortDescription  += "crosses the Average Valley upward";
                    break;

                default:
                    break;
            }

            return;
        }

        /// <summary>
        /// Indicator to string
        /// </summary>
        public override string ToString()
        {
            string sString = IndicatorName +
                (IndParam.CheckParam[0].Checked ? "* (" : " (") +
                IndParam.NumParam[0].ValueToString + ")";  // Bandpass Filter period

            return sString;
        }
    }
}
