// Butterworth Filter Indicator (Modified to HTF v 4.0.0 by Denny Imanuel)
// Last changed on 2009-05-05
// Part of Forex Strategy Builder v2.8.3.7+
// Website http://forexsb.com/
// This code or any part of it cannot be used in other applications without a permission.
// Copyright (c) 2006 - 2009 Miroslav Popov - All rights reserved.

using System;
using System.Drawing;

namespace Forex_Strategy_Builder
{
    /// <summary>
    /// Butterworth Filter Indicator 
    /// </summary>
    public class Butterworth_Filter : Indicator
    {
        /// <summary>
        /// Sets the default indicator parameters for the designated slot type
        /// </summary>
        public Butterworth_Filter(SlotTypes slotType)
        {
            // General properties
            IndicatorName = "Butterworth Filter";
            PossibleSlots = SlotTypes.Open | SlotTypes.OpenFilter | SlotTypes.Close | SlotTypes.CloseFilter;
			CustomIndicator = true;
			
            // Setting up the indicator parameters
            IndParam = new IndicatorParam();
            IndParam.IndicatorName = IndicatorName;
            IndParam.SlotType      = slotType;

            // The ComboBox parameters
            IndParam.ListParam[0].Caption = "Logic";
            if (slotType == SlotTypes.Open)
                IndParam.ListParam[0].ItemList = new string[]
                {
                    "Enter the market at the Butterworth Filter"
                };
            else if (slotType == SlotTypes.OpenFilter)
                IndParam.ListParam[0].ItemList = new string[]
                {
                    "The Butterworth Filter rises",
                    "The Butterworth Filter falls",
                    "The bar opens above the Butterworth Filter",
                    "The bar opens below the Butterworth Filter",
                    "The bar opens above the Butterworth Filter after opening below it",
                    "The bar opens below the Butterworth Filter after opening above it",
                    "The position opens above the Butterworth Filter",
                    "The position opens below the Butterworth Filter",
                };
            else if (slotType == SlotTypes.Close)
                IndParam.ListParam[0].ItemList = new string[]
                {
                    "Exit the market at the Butterworth Filter"
                };
            else if (slotType == SlotTypes.CloseFilter)
                IndParam.ListParam[0].ItemList = new string[]
                {
                    "The Butterworth Filter rises",
                    "The Butterworth Filter falls",
                    "The bar closes below the Butterworth Filter",
                    "The bar closes above the Butterworth Filter",
                };
            else
                IndParam.ListParam[0].ItemList = new string[]
                {
                    "Not Defined"
                };
            IndParam.ListParam[0].Index    = 0;
            IndParam.ListParam[0].Text     = IndParam.ListParam[0].ItemList[IndParam.ListParam[0].Index];
            IndParam.ListParam[0].Enabled  = true;
            IndParam.ListParam[0].ToolTip  = "Logic of application of the Butterworth Filter.";

            IndParam.ListParam[1].Caption  = "Smoothing method";
            IndParam.ListParam[1].ItemList = Enum.GetNames(typeof(MAMethod));
            IndParam.ListParam[1].Index    = (int)MAMethod.Simple;
            IndParam.ListParam[1].Text     = IndParam.ListParam[1].ItemList[IndParam.ListParam[1].Index];
            IndParam.ListParam[1].Enabled  = true;
            IndParam.ListParam[1].ToolTip  = "The smoothing method of Butterworth Filter.";

            IndParam.ListParam[2].Caption  = "Base price";
            IndParam.ListParam[2].ItemList = Enum.GetNames(typeof(BasePrice));
            IndParam.ListParam[2].Index    = (int)BasePrice.Median;
            IndParam.ListParam[2].Text     = IndParam.ListParam[2].ItemList[IndParam.ListParam[2].Index];
            IndParam.ListParam[2].Enabled  = true;
            IndParam.ListParam[2].ToolTip  = "The price the Butterworth Filter is based on.";
			
			IndParam.ListParam[3].Caption  = "Filter degree";
            IndParam.ListParam[3].ItemList = new string[]
				{
					"2 Poles",
					"3 Poles",
				};
            IndParam.ListParam[3].Index    = 0;
            IndParam.ListParam[3].Text     = IndParam.ListParam[3].ItemList[IndParam.ListParam[3].Index];
            IndParam.ListParam[3].Enabled  = true;
            IndParam.ListParam[3].ToolTip  = "The smoothness degree of the Butterworth Filter.";

            // The NumericUpDown parameters
            IndParam.NumParam[0].Caption   = "Period";
            IndParam.NumParam[0].Value     = 14;
            IndParam.NumParam[0].Min       = 1;
            IndParam.NumParam[0].Max       = 200;
            IndParam.NumParam[0].Enabled   = true;
            IndParam.NumParam[0].ToolTip   = "The Butterworth Filter period.";

            IndParam.NumParam[1].Caption   = "Shift";
            IndParam.NumParam[1].Value     = 0;
            IndParam.NumParam[1].Min       = 0;
            IndParam.NumParam[1].Max       = 200;
            IndParam.NumParam[1].Enabled   = true;
            IndParam.NumParam[1].ToolTip   = "How many bars to shift with.";

            // The CheckBox parameters
            IndParam.CheckParam[0].Caption = "Use previous bar value";
            IndParam.CheckParam[0].Checked = PrepareUsePrevBarValueCheckBox(slotType);
            IndParam.CheckParam[0].Enabled = true;
            IndParam.CheckParam[0].ToolTip = "Use the indicator value from the previous bar.";

            return;
        }

        /// <summary>
        /// Calculates the indicator's components
        /// </summary>
        public override void Calculate(SlotTypes slotType)
        {
            // Reading the parameters
            MAMethod  maMethod = (MAMethod )IndParam.ListParam[1].Index;
            BasePrice basePrice    = (BasePrice)IndParam.ListParam[2].Index;
            int       iPeriod  = (int)IndParam.NumParam[0].Value;
            int       iShift   = (int)IndParam.NumParam[1].Value;
            int       iPrvs    = IndParam.CheckParam[0].Checked ? 1 : 0;

            // TimeExecution
            if (basePrice == BasePrice.Open && iPeriod == 1 && iShift == 0)
                IndParam.ExecutionTime = ExecutionTime.AtBarOpening;

            // Calculation
			int iFirstBar = Math.Max(iPeriod,iShift) + 3 + iPrvs;
			
			double[] adPrice 	= Price(basePrice);
            double[] adButter	= new double[Bars];
			double a1;
			double b1;
			double c1;
			double coef1;
			double coef2;
			double coef3;
			double coef4;
            
			switch (IndParam.ListParam[3].Text)
			{
				case "2 Poles":
					
					a1 = Math.Exp(-Math.Sqrt(2)*Math.PI/iPeriod);
					b1 = 2*a1*Math.Cos(Math.Sqrt(2)*Math.PI/iPeriod);
					coef1=(1-b1+a1*a1)/4;
					coef2=b1;
					coef3=-a1*a1;
			
					for (int iBar=iFirstBar; iBar<Bars; iBar++)
					{
						adButter[iBar] = coef1*(adPrice[iBar]+2*adPrice[iBar-1]+adPrice[iBar-2]) + coef2*adButter[iBar-1] + coef3*adButter[iBar-2];
					}
					
					break;
				
				case "3 Poles":
					
					a1 = Math.Exp(-Math.PI/iPeriod);
					b1 = 2*a1*Math.Cos(Math.Sqrt(3)*Math.PI/iPeriod);
					c1 = a1*a1;
					coef1=(1-b1+c1)*(1-c1)/8;
					coef2=b1+c1;
					coef3=-(c1+b1*c1);
					coef4=c1*c1;
					
					for (int iBar=iFirstBar; iBar<Bars; iBar++)
					{
						adButter[iBar] = coef1*(adPrice[iBar]+3*adPrice[iBar-1]+3*adPrice[iBar-2]+adPrice[iBar-3]) + coef2*adButter[iBar-1] + coef3*adButter[iBar-2] + coef4*adButter[iBar-3];
					}
					
					break;
				
				default:
					break;
			}
			
			
            // Saving the components
            if (slotType == SlotTypes.Open || slotType == SlotTypes.Close)
            {
                Component = new IndicatorComp[2];

                Component[1] = new IndicatorComp();
                Component[1].Value = new double[Bars];
                
                for (int iBar = 2; iBar < Bars; iBar++)
                {   // Covers the cases when the price can pass through the BF without a signal
                    double dValue   = adButter[iBar - iPrvs];     // Current value
                    double dValue1  = adButter[iBar - iPrvs - 1]; // Previous value
                    double dTempVal = dValue;
                    if ((dValue1 > High[iBar - 1] && dValue < Low[iBar]) || // It jumps below the current bar
                        (dValue1 < Low[iBar - 1]  && dValue > High[iBar])|| // It jumps above the current bar
                        (Close[iBar - 1] < dValue && dValue < Open[iBar])|| // Positive gap
                        (Close[iBar - 1] > dValue && dValue > Open[iBar]))  // Negative gap
                        dTempVal = Open[iBar];
                    Component[1].Value[iBar] = dTempVal;
                }
            }
            else
            {
                Component = new IndicatorComp[3];

                Component[1] = new IndicatorComp();
                Component[1].ChartType = IndChartType.NoChart;
                Component[1].FirstBar  = iFirstBar;
                Component[1].Value     = new double[Bars];

                Component[2] = new IndicatorComp();
                Component[2].ChartType = IndChartType.NoChart;
                Component[2].FirstBar  = iFirstBar;
                Component[2].Value     = new double[Bars];
            }

            Component[0] = new IndicatorComp();
            Component[0].CompName   = "Butterworth Filter";
            Component[0].DataType   = IndComponentType.IndicatorValue;
            Component[0].ChartType  = IndChartType.Line;
            Component[0].ChartColor = Color.Gold;
            Component[0].FirstBar   = iFirstBar;
            Component[0].Value      = adButter;

            if (slotType == SlotTypes.Open)
            {
                Component[1].CompName = "Position opening price";
                Component[1].DataType = IndComponentType.OpenPrice;
            }
            else if (slotType == SlotTypes.OpenFilter)
            {
                Component[1].DataType = IndComponentType.AllowOpenLong;
                Component[1].CompName = "Is long entry allowed";
                Component[2].DataType = IndComponentType.AllowOpenShort;
                Component[2].CompName = "Is short entry allowed";
            }
            else if (slotType == SlotTypes.Close)
            {
                Component[1].CompName = "Position closing price";
                Component[1].DataType = IndComponentType.ClosePrice;
            }
            else if (slotType == SlotTypes.CloseFilter)
            {
                Component[1].DataType = IndComponentType.ForceCloseLong;
                Component[1].CompName = "Close out long position";
                Component[2].DataType = IndComponentType.ForceCloseShort;
                Component[2].CompName = "Close out short position";
            }

            if (slotType == SlotTypes.OpenFilter || slotType == SlotTypes.CloseFilter)
            {
                switch (IndParam.ListParam[0].Text)
                {
                    case "The Butterworth Filter rises":
                        IndicatorRisesLogic(iFirstBar, iPrvs, adButter, ref Component[1], ref Component[2]);
                        break;

                    case "The Butterworth Filter falls":
                        IndicatorFallsLogic(iFirstBar, iPrvs, adButter, ref Component[1], ref Component[2]);
                        break;

                    case "The bar opens above the Butterworth Filter":
                        BarOpensAboveIndicatorLogic(iFirstBar, iPrvs, adButter, ref Component[1], ref Component[2]);
                        break;

                    case "The bar opens below the Butterworth Filter":
                        BarOpensBelowIndicatorLogic(iFirstBar, iPrvs, adButter, ref Component[1], ref Component[2]);
                        break;

                    case "The bar opens above the Butterworth Filter after opening below it":
                        BarOpensAboveIndicatorAfterOpeningBelowLogic(iFirstBar, iPrvs, adButter, ref Component[1], ref Component[2]);
                        break;

                    case "The bar opens below the Butterworth Filter after opening above it":
                        BarOpensBelowIndicatorAfterOpeningAboveLogic(iFirstBar, iPrvs, adButter, ref Component[1], ref Component[2]);
                        break;

                    case "The position opens above the Butterworth Filter":
                        Component[0].PosPriceDependence = PositionPriceDependence.BuyHigherSellLower;
                        Component[0].UsePreviousBar     = iPrvs;
                        Component[1].DataType           = IndComponentType.Other;
                        Component[1].ShowInDynInfo      = false;
                        Component[2].DataType           = IndComponentType.Other;
                        Component[2].ShowInDynInfo      = false;
                        break;

                    case "The position opens below the Butterworth Filter":
                        Component[0].PosPriceDependence = PositionPriceDependence.BuyLowerSelHigher;
                        Component[0].UsePreviousBar     = iPrvs;
                        Component[1].DataType           = IndComponentType.Other;
                        Component[1].ShowInDynInfo      = false;
                        Component[2].DataType           = IndComponentType.Other;
                        Component[2].ShowInDynInfo      = false;
                        break;

                    case "The bar closes below the Butterworth Filter":
                        BarClosesBelowIndicatorLogic(iFirstBar, iPrvs, adButter, ref Component[1], ref Component[2]);
                        break;

                    case "The bar closes above the Butterworth Filter":
                        BarClosesAboveIndicatorLogic(iFirstBar, iPrvs, adButter, ref Component[1], ref Component[2]);
                        break;

                    default:
                        break;
                }
            }

            return;
        }

        /// <summary>
        /// Sets the indicator logic description
        /// </summary>
        public override void SetDescription(SlotTypes slotType)
        {
            EntryPointLongDescription  = "at the " + ToString();
            EntryPointShortDescription = "at the " + ToString();
            ExitPointLongDescription   = "at the " + ToString();
            ExitPointShortDescription  = "at the " + ToString();

            switch (IndParam.ListParam[0].Text)
            {
                case "The Butterworth Filter rises":
                    EntryFilterLongDescription  = "the " + ToString() + " rises";
                    EntryFilterShortDescription = "the " + ToString() + " falls";
                    ExitFilterLongDescription   = "the " + ToString() + " rises";
                    ExitFilterShortDescription  = "the " + ToString() + " falls";
                    break;

                case "The Butterworth Filter falls":
                    EntryFilterLongDescription  = "the " + ToString() + " falls";
                    EntryFilterShortDescription = "the " + ToString() + " rises";
                    ExitFilterLongDescription   = "the " + ToString() + " falls";
                    ExitFilterShortDescription  = "the " + ToString() + " rises";
                    break;

                case "The bar opens above the Butterworth Filter":
                    EntryFilterLongDescription  = "the bar opens above the " + ToString();
                    EntryFilterShortDescription = "the bar opens below the " + ToString();
                    break;

                case "The bar opens below the Butterworth Filter":
                    EntryFilterLongDescription  = "the bar opens below the " + ToString();
                    EntryFilterShortDescription = "the bar opens above the " + ToString();
                    break;

                case "The position opens above the Butterworth Filter":
                    EntryFilterLongDescription  = "the position opening price is higher than the " + ToString();
                    EntryFilterShortDescription = "the position opening price is lower than the "  + ToString();
                    break;

                case "The position opens below the Butterworth Filter":
                    EntryFilterLongDescription  = "the position opening price is lower than the "  + ToString();
                    EntryFilterShortDescription = "the position opening price is higher than the " + ToString();
                    break;

                case "The bar opens above the Butterworth Filter after opening below it":
                    EntryFilterLongDescription  = "the bar opens above the " + ToString() + " after opening below it";
                    EntryFilterShortDescription = "the bar opens below the " + ToString() + " after opening above it";
                    break;

                case "The bar opens below the Butterworth Filter after opening above it":
                    EntryFilterLongDescription  = "the bar opens below the " + ToString() + " after opening above it";
                    EntryFilterShortDescription = "the bar opens above the " + ToString() + " after opening below it";
                    break;

                case "The bar closes above the Butterworth Filter":
                    ExitFilterLongDescription  = "the bar closes above the " + ToString();
                    ExitFilterShortDescription = "the bar closes below the " + ToString();
                    break;

                case "The bar closes below the Butterworth Filter":
                    ExitFilterLongDescription  = "the bar closes below the " + ToString();
                    ExitFilterShortDescription = "the bar closes above the " + ToString();
                    break;

                default:
                    break;
            }

            return;
        }

        /// <summary>
        /// Indicator to string
        /// </summary>
        public override string ToString()
        {
            string sString = IndicatorName + 
                (IndParam.CheckParam[0].Checked ? "* (" : " (") +
                IndParam.ListParam[1].Text         + ", " + // Method
                IndParam.ListParam[2].Text         + ", " + // Price
				IndParam.ListParam[3].Text         + ", " + // Poles
                IndParam.NumParam[0].ValueToString + ", " + // BF period
                IndParam.NumParam[1].ValueToString + ")";   // BF shift

            return sString;
        }
    }
}