// Finite Volume Element Indicator (Modified to HTF v 4.0.0 by Denny Imanuel)
// Last changed on 2 Mar 2010
// Use with Forex Strategy Builder v2.8.3.7+
// Website http://autotradingstrategy.wordpress.com/
// This code or any part of it cannot be used in other applications without a permission.
// Copyright (c) 2010 Denny Imanuel - All rights reserved.

using System;
using System.Drawing;

namespace Forex_Strategy_Builder
{
    /// <summary>
    /// Finite Volume Element Indicator
    /// </summary>
    public class Finite_Volume_Element : Indicator
    {
        /// <summary>
        /// Sets the default indicator parameters for the designated slot type
        /// </summary>
        public Finite_Volume_Element(SlotTypes slotType)
        {
            // General properties
            IndicatorName  = "Finite Volume Element";
            PossibleSlots  = SlotTypes.OpenFilter | SlotTypes.CloseFilter;
            SeparatedChart = true;
			CustomIndicator = true;

            // Setting up the indicator parameters
            IndParam = new IndicatorParam();
            IndParam.IndicatorName = IndicatorName;
            IndParam.SlotType      = slotType;

            // The ComboBox parameters
            IndParam.ListParam[0].Caption  = "Logic";
            IndParam.ListParam[0].ItemList = new string[]
            {
                "The FVE Signal rises",
                "The FVE Signal falls",
                "The FVE Signal is higher than the Level line",
                "The FVE Signal is lower than the Level line",
                "The FVE Signal crosses the Level line upward",
                "The FVE Signal crosses the Level line downward",
                "The FVE Signal changes its direction upward",
                "The FVE Signal changes its direction downward"
            };
            IndParam.ListParam[0].Index    = 0;
            IndParam.ListParam[0].Text     = IndParam.ListParam[0].ItemList[IndParam.ListParam[0].Index];
            IndParam.ListParam[0].Enabled  = true;
            IndParam.ListParam[0].ToolTip  = "Logic of application of the indicator.";

            IndParam.ListParam[1].Caption  = "Chart's Signal";
            IndParam.ListParam[1].ItemList = new string[]
            {
				"Original FVE",
				"Moving Average of FVE",
                "Regression Slope of FVE",
			};
            IndParam.ListParam[1].Index    = 0;
            IndParam.ListParam[1].Text     = IndParam.ListParam[1].ItemList[IndParam.ListParam[1].Index];
            IndParam.ListParam[1].Enabled  = true;
            IndParam.ListParam[1].ToolTip  = "The method used for smoothing the both Moving Averages.";

            IndParam.ListParam[2].Caption  = "Smoothing Method";
            IndParam.ListParam[2].ItemList = Enum.GetNames(typeof(MAMethod));
            IndParam.ListParam[2].Index    = (int)MAMethod.Simple;
            IndParam.ListParam[2].Text     = IndParam.ListParam[2].ItemList[IndParam.ListParam[2].Index];
            IndParam.ListParam[2].Enabled  = true;
            IndParam.ListParam[2].ToolTip  = "The method used for smoothing the both Moving Averages.";

            // The NumericUpDown parameters
            IndParam.NumParam[0].Caption   = "FVE Samples";
            IndParam.NumParam[0].Value     = 22;
            IndParam.NumParam[0].Min       = 1;
            IndParam.NumParam[0].Max       = 200;
            IndParam.NumParam[0].Enabled   = true;
            IndParam.NumParam[0].ToolTip   = "The period of Finite Volume Element";

            IndParam.NumParam[1].Caption   = "Smoothing/Regression Period";
            IndParam.NumParam[1].Value     = 10;
            IndParam.NumParam[1].Min       = 1;
            IndParam.NumParam[1].Max       = 200;
            IndParam.NumParam[1].Enabled   = true;
            IndParam.NumParam[1].ToolTip   = "The period of smoothing or regression";

            IndParam.NumParam[2].Caption   = "CutOff";
            IndParam.NumParam[2].Value     = 0.003;
            IndParam.NumParam[2].Min       = 0;
            IndParam.NumParam[2].Max       = 1;
            IndParam.NumParam[2].Point     = 4;
            IndParam.NumParam[2].Enabled   = true;
            IndParam.NumParam[2].ToolTip   = "A cutoff level (for the appropriate logic).";

            IndParam.NumParam[3].Caption   = "Level";
            IndParam.NumParam[3].Value     = 0;
            IndParam.NumParam[3].Min       = -100;
            IndParam.NumParam[3].Max       = 100;
            IndParam.NumParam[3].Point     = 4;
            IndParam.NumParam[3].Enabled   = true;
            IndParam.NumParam[3].ToolTip   = "A critical level (for the appropriate logic).";

            // The CheckBox parameters
            IndParam.CheckParam[0].Caption = "Use previous bar value";
            IndParam.CheckParam[0].Checked = PrepareUsePrevBarValueCheckBox(slotType);
            IndParam.CheckParam[0].Enabled = true;
            IndParam.CheckParam[0].ToolTip = "Use the indicator value from the previous bar.";

            return;
        }

        /// <summary>
        /// Calculates the indicator's components
        /// </summary>
        public override void Calculate(SlotTypes slotType)
        {
            // Reading the parameters
            MAMethod  maMethod  = (MAMethod )IndParam.ListParam[2].Index;
            int       iSample   = (int)IndParam.NumParam[0].Value;
            int       iLength 	= (int)IndParam.NumParam[1].Value;
			double    dCutOff 	= IndParam.NumParam[2].Value;
            double    dLevel    = IndParam.NumParam[3].Value;
            int       iPrvs     = IndParam.CheckParam[0].Checked ? 1 : 0;

			double[] adTP 		= Price(BasePrice.Median);
			double[] adFVE 		= new double[Bars];
			double[] adFVESig 	= new double[Bars];
			double[] adVolPM 	= new double[Bars];
			double dMF;
			double dFVEfactor;
			double dFVESum;
			double dVOLAvg;
			
			int iFirstBar = Math.Max(iSample, iLength) + 2;
			
			for (int iBar=iFirstBar; iBar<Bars; iBar++)
			{	
				dMF = (Close[iBar]-(High[iBar]+Low[iBar])/2) + adTP[iBar] - adTP[iBar-1];
				if (dMF>dCutOff*Close[iBar])			dFVEfactor = 1;
				else if (dMF<-dCutOff*Close[iBar])		dFVEfactor = -1;
				else 									dFVEfactor = 0;				
				
				adVolPM[iBar] = Volume[iBar] * dFVEfactor;
				dFVESum = 0; dVOLAvg=0;
				for (int iBack=0; iBack<iSample; iBack++)
				{
					dFVESum = dFVESum + adVolPM[iBar-iBack];
					dVOLAvg = dVOLAvg + Volume[iBar-iBack]/iSample;
				}
				adFVE[iBar] = dFVESum/(dVOLAvg*iSample) * 100;	
			}	
			
			switch(IndParam.ListParam[1].Text)
			{
				case "Moving Average of FVE":
					adFVESig = MovingAverage(iLength, 0, maMethod, adFVE);
					break;
				case "Regression Slope of FVE":
					adFVESig = LinearRegressionSlope(adFVE, iLength);
					break;
				default:
					adFVESig = adFVE;
					break;
			}
			
            // Saving the components
            Component = new IndicatorComp[3];

            Component[0] = new IndicatorComp();
            Component[0].CompName  = "FVE Signal";
            Component[0].DataType  = IndComponentType.IndicatorValue;
            if 	 (IndParam.ListParam[1].Index==2) Component[0].ChartType  = IndChartType.Histogram;
			else Component[0].ChartType  = IndChartType.Line;
			Component[0].ChartColor = Color.Purple;
            Component[0].FirstBar  = iFirstBar;
            Component[0].Value     = adFVESig;

            Component[1] = new IndicatorComp();
            Component[1].ChartType = IndChartType.NoChart;
            Component[1].FirstBar  = iFirstBar;
            Component[1].Value     = new double[Bars];

            Component[2] = new IndicatorComp();
            Component[2].ChartType = IndChartType.NoChart;
            Component[2].FirstBar  = iFirstBar;
            Component[2].Value     = new double[Bars];

            // Sets the Component's type
            if (slotType == SlotTypes.OpenFilter)
            {
                Component[1].DataType = IndComponentType.AllowOpenLong;
                Component[1].CompName = "Is long entry allowed";
                Component[2].DataType = IndComponentType.AllowOpenShort;
                Component[2].CompName = "Is short entry allowed";
            }
            else if (slotType == SlotTypes.CloseFilter)
            {
                Component[1].DataType = IndComponentType.ForceCloseLong;
                Component[1].CompName = "Close out long position";
                Component[2].DataType = IndComponentType.ForceCloseShort;
                Component[2].CompName = "Close out short position";
            }

            // Calculation of the logic
            IndicatorLogic indLogic = IndicatorLogic.It_does_not_act_as_a_filter;

            switch (IndParam.ListParam[0].Text)
            {
                case "The FVE Signal rises":
                    indLogic = IndicatorLogic.The_indicator_rises;
                    SpecialValues = new double[1] { 0 };
                    break;

                case "The FVE Signal falls":
                    indLogic = IndicatorLogic.The_indicator_falls;
                    SpecialValues = new double[1] { 0 };
                    break;

                case "The FVE Signal is higher than the Level line":
                    indLogic = IndicatorLogic.The_indicator_is_higher_than_the_level_line;
                    SpecialValues = new double[2] { dLevel, -dLevel };
                    break;

                case "The FVE Signal is lower than the Level line":
                    indLogic = IndicatorLogic.The_indicator_is_lower_than_the_level_line;
                    SpecialValues = new double[2] { dLevel, -dLevel };
                    break;

                case "The FVE Signal crosses the Level line upward":
                    indLogic = IndicatorLogic.The_indicator_crosses_the_level_line_upward;
                    SpecialValues = new double[2] { dLevel, -dLevel };
                    break;

                case "The FVE Signal crosses the Level line downward":
                    indLogic = IndicatorLogic.The_indicator_crosses_the_level_line_downward;
                    SpecialValues = new double[2] { dLevel, -dLevel };
                    break;

                case "The FVE Signal changes its direction upward":
                    indLogic = IndicatorLogic.The_indicator_changes_its_direction_upward;
                    SpecialValues = new double[1] { 0 };
                    break;

                case "The FVE Signal changes its direction downward":
                    indLogic = IndicatorLogic.The_indicator_changes_its_direction_downward;
                    SpecialValues = new double[1] { 0 };
                    break;

                default:
                    break;
            }

            OscillatorLogic(iFirstBar, iPrvs, adFVESig, dLevel, -dLevel, ref Component[1], ref Component[2], indLogic);

            return;
        }

        /// <summary>
        /// Sets the indicator logic description
        /// </summary>
        public override void SetDescription(SlotTypes slotType)
        {
            string sLevelLong  = (IndParam.NumParam[2].Value == 0 ? "0" : IndParam.NumParam[2].ValueToString);
            string sLevelShort = (IndParam.NumParam[2].Value == 0 ? "0" : "-" + IndParam.NumParam[2].ValueToString);

            EntryFilterLongDescription  = "the " + ToString() + " ";
            EntryFilterShortDescription = "the " + ToString() + " ";
            ExitFilterLongDescription   = "the " + ToString() + " ";
            ExitFilterShortDescription  = "the " + ToString() + " ";

            switch (IndParam.ListParam[0].Text)
            {
                case "The FVE Signal rises":
                    EntryFilterLongDescription  += "rises";
                    EntryFilterShortDescription += "falls";
                    ExitFilterLongDescription   += "rises";
                    ExitFilterShortDescription  += "falls";
                    break;

                case "The FVE Signal falls":
                    EntryFilterLongDescription  += "falls";
                    EntryFilterShortDescription += "rises";
                    ExitFilterLongDescription   += "falls";
                    ExitFilterShortDescription  += "rises";
                    break;

                case "The FVE Signal is higher than the Level line":
                    EntryFilterLongDescription  += "is higher than the Level " + sLevelLong;
                    EntryFilterShortDescription += "is lower than the Level "  + sLevelShort;
                    ExitFilterLongDescription   += "is higher than the Level " + sLevelLong;
                    ExitFilterShortDescription  += "is lower than the Level "  + sLevelShort;
                    break;

                case "The FVE Signal is lower than the Level line":
                    EntryFilterLongDescription  += "is lower than the Level "  + sLevelLong;
                    EntryFilterShortDescription += "is higher than the Level " + sLevelShort;
                    ExitFilterLongDescription   += "is lower than the Level "  + sLevelLong;
                    ExitFilterShortDescription  += "is higher than the Level " + sLevelShort;
                    break;

                case "The FVE Signal crosses the Level line upward":
                    EntryFilterLongDescription  += "crosses the Level " + sLevelLong  + " upward";
                    EntryFilterShortDescription += "crosses the Level " + sLevelShort + " downward";
                    ExitFilterLongDescription   += "crosses the Level " + sLevelLong  + " upward";
                    ExitFilterShortDescription  += "crosses the Level " + sLevelShort + " downward";
                    break;

                case "The FVE Signal crosses the Level line downward":
                    EntryFilterLongDescription  += "crosses the Level " + sLevelLong  + " downward";
                    EntryFilterShortDescription += "crosses the Level " + sLevelShort + " upward";
                    ExitFilterLongDescription   += "crosses the Level " + sLevelLong  + " downward";
                    ExitFilterShortDescription  += "crosses the Level " + sLevelShort + " upward";
                    break;

                case "The FVE Signal changes its direction upward":
                    EntryFilterLongDescription  += "changes its direction upward";
                    EntryFilterShortDescription += "changes its direction downward";
                    ExitFilterLongDescription   += "changes its direction upward";
                    ExitFilterShortDescription  += "changes its direction downward";
                    break;

                case "The FVE Signal changes its direction downward":
                    EntryFilterLongDescription  += "changes its direction downward";
                    EntryFilterShortDescription += "changes its direction upward";
                    ExitFilterLongDescription   += "changes its direction downward";
                    ExitFilterShortDescription  += "changes its direction upward";
                    break;

                default:
                    break;
            }

            return;
        }

        /// <summary>
        /// Indicator to string
        /// </summary>
        public override string ToString()
        {
            string sString = IndicatorName +
                (IndParam.CheckParam[0].Checked ? "* (" : " (") +
                IndParam.ListParam[1].Text         + ", " + // Method
                IndParam.ListParam[2].Text         + ", " + // Price
                IndParam.NumParam[0].ValueToString + ", " + // MA Period
                IndParam.NumParam[1].ValueToString + ")";   // LR Length

            return sString;
        }
		
		protected static double[] LinearRegressionSlope(double[] adInput, int iLength) 
		{
		
			double    dSigX;
			double    dSigY;
			double	  dSigXY;
			double    dSigXX;
			double[]  adSlope = new double[Bars];
			
            int iFirstBar = iLength + 2;

            for (int iBar = iFirstBar; iBar < Bars; iBar++)
			{
				dSigX = 0;
				dSigY = 0;
				dSigXX = 0;
				dSigXY = 0;
				for (int index = 0; index < iLength; index++)
                {
					dSigX  = dSigX  + index;
					dSigY  = dSigY  + adInput[iBar-index];
					dSigXY = dSigXY + index * adInput[iBar-index];
					dSigXX = dSigXX + index * index;
				}
				adSlope[iBar] = - (iLength*dSigXY - dSigX*dSigY)/(iLength*dSigXX - dSigX*dSigX);				
			}
			
			return adSlope;
		}
		
		
    }
}
