// Adaptive Stochastic Indicator (Modified to HTF v 4.0.0 by Denny Imanuel)
// Last changed on 2009-11-17
// Part of Forex Strategy Builder
// Website http://forexsb.com/
// Copyright (c) 2006 - 2009 Miroslav Popov - All rights reserved.
// This code or any part of it cannot be used in other applications without a permission.

using System;
using System.Drawing;

namespace Forex_Strategy_Builder
{
    /// <summary>
    /// Stochastics Indicator
    /// </summary>
    public class Adaptive_Stochastics_HTF : Indicator
    {
        /// <summary>
        /// Sets the default indicator parameters for the designated slot type
        /// </summary>
        public Adaptive_Stochastics_HTF(SlotTypes slotType)
        {
            // General properties
            IndicatorName  = "Adaptive Stochastics (HTF v4)";
            PossibleSlots  = SlotTypes.OpenFilter | SlotTypes.CloseFilter;
            SeparatedChart = true;
            SeparatedChartMinValue = 0;
            SeparatedChartMaxValue = 100;
			CustomIndicator = true;

            // Setting up the indicator parameters
            IndParam = new IndicatorParam();
            IndParam.IndicatorName = IndicatorName;
            IndParam.SlotType      = slotType;

            // The ComboBox parameters
            IndParam.ListParam[0].Caption  = "Logic";
            IndParam.ListParam[0].ItemList = new string[]
            {
                "The %D rises",
                "The %D falls",
                "The %D is higher than the Level line",
                "The %D is lower than the Level line",
                "The %D crosses the Level line upward",
                "The %D crosses the Level line downward",
                "The %D changes its direction upward",
                "The %D changes its direction downward",
                "The %K is higher than the %D",
                "The %K is lower than the %D",
                "The %K crosses the %D upward",
                "The %K crosses the %D downward",
            };
            IndParam.ListParam[0].Index    = 0;
            IndParam.ListParam[0].Text     = IndParam.ListParam[0].ItemList[IndParam.ListParam[0].Index];
            IndParam.ListParam[0].Enabled  = true;
            IndParam.ListParam[0].ToolTip  = "Logic of application of the indicator.";

            IndParam.ListParam[1].Caption  = "Smoothing method";
            IndParam.ListParam[1].ItemList = Enum.GetNames(typeof(MAMethod));
            IndParam.ListParam[1].Index    = (int)MAMethod.Simple;
            IndParam.ListParam[1].Text     = IndParam.ListParam[1].ItemList[IndParam.ListParam[1].Index];
            IndParam.ListParam[1].Enabled  = true;
            IndParam.ListParam[1].ToolTip  = "The Moving Average method used for smoothing.";

            IndParam.ListParam[4].Caption = "Higher Time Frame Reference";
			IndParam.ListParam[4].ItemList = new string[] { "1 Minute", "5 Minutes", "15 Minutes", "30 Minutes", "1 Hour", "4 Hours", "1 Day", "1 Week"};
            if (Period == DataPeriods.min1) 		IndParam.ListParam[4].Index    = 1;
            else if (Period == DataPeriods.min5) 	IndParam.ListParam[4].Index    = 2;
            else if (Period == DataPeriods.min15) 	IndParam.ListParam[4].Index    = 3;
            else if (Period == DataPeriods.min30)	IndParam.ListParam[4].Index    = 4;
            else if (Period == DataPeriods.hour1)	IndParam.ListParam[4].Index    = 5;
            else if (Period == DataPeriods.hour4)	IndParam.ListParam[4].Index    = 6;
            else if (Period == DataPeriods.day)		IndParam.ListParam[4].Index    = 7;
            IndParam.ListParam[4].Text     = IndParam.ListParam[4].ItemList[IndParam.ListParam[4].Index];
            IndParam.ListParam[4].Enabled  = true;
            IndParam.ListParam[4].ToolTip  = "Choose higher time frame as compared to current chart period for this indicator.";		

			IndParam.NumParam[0].Caption = "Min %K period";
            IndParam.NumParam[0].Value   = 3;
            IndParam.NumParam[0].Min     = 1;
            IndParam.NumParam[0].Max     = 50;
            IndParam.NumParam[0].Enabled = true;
            IndParam.NumParam[0].ToolTip = "The minimum value for smoothing period of %K.";

			IndParam.NumParam[1].Caption = "Max %K period";
            IndParam.NumParam[1].Value   = 28;
            IndParam.NumParam[1].Min     = 10;
            IndParam.NumParam[1].Max     = 200;
            IndParam.NumParam[1].Enabled = true;
            IndParam.NumParam[1].ToolTip = "The maximum value for smoothing period of %K.";
			
            IndParam.NumParam[2].Caption = "%D period";
            IndParam.NumParam[2].Value   = 3;
            IndParam.NumParam[2].Min     = 1;
            IndParam.NumParam[2].Max     = 100;
            IndParam.NumParam[2].Enabled = true;
            IndParam.NumParam[2].ToolTip = "The fixed value for smoothing period of Fast %D.";

            IndParam.NumParam[4].Caption = "Level";
            IndParam.NumParam[4].Value   = 20;
            IndParam.NumParam[4].Min     = 0;
            IndParam.NumParam[4].Max     = 100;
            IndParam.NumParam[4].Enabled = true;
            IndParam.NumParam[4].ToolTip = "A critical level (for the appropriate logic).";

            // The CheckBox parameters
            IndParam.CheckParam[0].Caption = "Use previous bar value";
            IndParam.CheckParam[0].Checked = PrepareUsePrevBarValueCheckBox(slotType);
            IndParam.CheckParam[0].Enabled = true;
            IndParam.CheckParam[0].ToolTip = "Use the indicator value from the previous bar.";

            return;
        }

        /// <summary>
        /// Calculates the indicator's components
        /// </summary>
        public override void Calculate(SlotTypes slotType)
        {
            // Reading the parameters
            MAMethod maMethod = (MAMethod)IndParam.ListParam[1].Index;
            int iKMin  = (int)IndParam.NumParam[0].Value;
			int iKMax  = (int)IndParam.NumParam[0].Value;
            int iD	   = (int)IndParam.NumParam[2].Value;
            int iLevel = (int)IndParam.NumParam[4].Value;
            int iPrvs  = IndParam.CheckParam[0].Checked ? 1 : 0;

            // Convert to Higher Time Frame ---------------------------------------------
			DataPeriods htfPeriod = DataPeriods.week;
			double[] hfOpen = new double[Bars];
			double[] hfClose = new double[Bars];
			double[] hfHigh = new double[Bars];
			double[] hfLow = new double[Bars];
			double[] hfVolume = new double[Bars];
			double[] hfPrice = new double[Bars];
			int[] 	 hIndex = new int[Bars];
			int      iFrame;
			int      hBars;
			
			switch (IndParam.ListParam[4].Index)
			{
				case 1: htfPeriod = DataPeriods.min5; break;
				case 2: htfPeriod = DataPeriods.min15; break;
				case 3: htfPeriod = DataPeriods.min30; break;
				case 4: htfPeriod = DataPeriods.hour1; break;
				case 5: htfPeriod = DataPeriods.hour4; break;
				case 6: htfPeriod = DataPeriods.day; break;
				case 7: htfPeriod = DataPeriods.week; break;
			}
			
			int err1 = HigherTimeFrame(Period, htfPeriod, out hIndex, out hBars, out iFrame, out hfHigh, out hfLow, out hfOpen, out hfClose, out hfVolume);
			//int err2 = HigherBasePrice(basePrice, hBars, hfHigh, hfLow, hfOpen, hfClose, out hfPrice);	
			if (err1==1) return;
			//------------------------------------------------------------------------

			// Calculation
            int iFirstBar = iKMax + iD + 3;
			
			double[] adHighs 		= new double[Bars];
            double[] adLows 		= new double[Bars];
			double[] adPriceStdDev 	= new double[Bars];
            double[] adMaxStdDev 	= new double[Bars];
			double[] adMinStdDev 	= new double[Bars];
			double[] adK = new double[Bars];
			
			double dRatio;
			int iKAvg = (int) (iKMax+iKMin)/2;
			int iK;
			
			adPriceStdDev = StdDev(hfClose, iKAvg);
			adMaxStdDev   = Highest(adPriceStdDev, iKAvg);
			adMinStdDev	  = Lowest (adPriceStdDev, iKAvg);		
            
            for (int iBar = iKMax; iBar < hBars; iBar++)
            {
                // Calculate Effective Length for %K Period
				if (adMaxStdDev[iBar]-adMinStdDev[iBar]!=0)
				{
					dRatio = (adPriceStdDev[iBar]-adMinStdDev[iBar])/(adMaxStdDev[iBar]-adMinStdDev[iBar]);
					iK = (int) (iKMin + (iKMax-iKMin)*(1-dRatio));
				}
				else
					iK = (int) iKMax;
				
				// Calculate Highest High and Lowest Low
				double dMax = double.MinValue;
				double dMin = double.MaxValue;
                for (int iBack = 0; iBack < iK; iBack++)
                {
                    if (hfHigh[iBar-iBack] > dMax) dMax = hfHigh[iBar-iBack];
					if (hfLow[iBar-iBack] < dMin) dMin = hfLow[iBar-iBack];
                }
                adHighs[iBar] = dMax;
				adLows[iBar] = dMin;
				
				// Calculate %K Value
				if (adHighs[iBar] == adLows[iBar])
                    adK[iBar] = 50;
                else
                    adK[iBar] = 100 * (hfClose[iBar] - adLows[iBar]) / (adHighs[iBar] - adLows[iBar]);
            }

			// Calculate %D Value
            double[] adD = MovingAverage(iD, 0, maMethod, adK);

			// Convert to Current Time Frame ----------------------------------------------
			iPrvs = iPrvs*iFrame;
			iFirstBar = iFirstBar*iFrame;
			if (iFirstBar>0.25*Bars) return;
			
			int err3 = CurrentTimeFrame(hIndex, hBars, ref adK);
			int err4 = CurrentTimeFrame(hIndex, hBars, ref adD);		
			//-----------------------------------------------------------------------------	

            // Saving the components
            Component = new IndicatorComp[4];

            Component[0] = new IndicatorComp();
            Component[0].CompName   = "%K";
            Component[0].DataType   = IndComponentType.IndicatorValue;
            Component[0].ChartType  = IndChartType.Line;
            Component[0].ChartColor = Color.Brown;
            Component[0].FirstBar   = iFirstBar;
            Component[0].Value      = adK;

            Component[1] = new IndicatorComp();
            Component[1].CompName   = "%D";
            Component[1].DataType   = IndComponentType.IndicatorValue;
            Component[1].ChartType  = IndChartType.Line;
            Component[1].ChartColor = Color.Blue;
            Component[1].FirstBar   = iFirstBar;
            Component[1].Value      = adD;

            Component[2] = new IndicatorComp();
            Component[2].ChartType = IndChartType.NoChart;
            Component[2].FirstBar  = iFirstBar;
            Component[2].Value     = new double[Bars];

            Component[3] = new IndicatorComp();
            Component[3].ChartType = IndChartType.NoChart;
            Component[3].FirstBar  = iFirstBar;
            Component[3].Value     = new double[Bars];

            // Sets the Component's type
            if (slotType == SlotTypes.OpenFilter)
            {
                Component[2].DataType = IndComponentType.AllowOpenLong;
                Component[2].CompName = "Is long entry allowed";
                Component[3].DataType = IndComponentType.AllowOpenShort;
                Component[3].CompName = "Is short entry allowed";
            }
            else if (slotType == SlotTypes.CloseFilter)
            {
                Component[2].DataType = IndComponentType.ForceCloseLong;
                Component[2].CompName = "Close out long position";
                Component[3].DataType = IndComponentType.ForceCloseShort;
                Component[3].CompName = "Close out short position";
            }

            // Calculation of the logic
            IndicatorLogic indLogic = IndicatorLogic.It_does_not_act_as_a_filter;

            if (IndParam.ListParam[0].Text == "The %K crosses the %D upward")
            {
                SpecialValues = new double[1] { 50 };
                IndicatorCrossesAnotherIndicatorUpwardLogic(iFirstBar, iPrvs,adK, adD, ref Component[2], ref Component[3]);
                return;
            }
            else if (IndParam.ListParam[0].Text == "The %K crosses the %D downward")
            {
                SpecialValues = new double[1] { 50 };
                IndicatorCrossesAnotherIndicatorDownwardLogic(iFirstBar, iPrvs, adK, adD, ref Component[2], ref Component[3]);
                return;
            }
            else if (IndParam.ListParam[0].Text == "The %K is higher than the %D")
            {
                SpecialValues = new double[1] { 50 };
                IndicatorIsHigherThanAnotherIndicatorLogic(iFirstBar, iPrvs, adK, adD, ref Component[2], ref Component[3]);
                return;
            }
            else if (IndParam.ListParam[0].Text == "The %K is lower than the %D")
            {
                SpecialValues = new double[1] { 50 };
                IndicatorIsLowerThanAnotherIndicatorLogic(iFirstBar, iPrvs, adK, adD, ref Component[2], ref Component[3]);
                return;
            }
            else
            {
                switch (IndParam.ListParam[0].Text)
                {
                    case "The %D rises":
                        indLogic = IndicatorLogic.The_indicator_rises;
                        SpecialValues = new double[1] { 50 };
                        break;

                    case "The %D falls":
                        indLogic = IndicatorLogic.The_indicator_falls;
                        SpecialValues = new double[1] { 50 };
                        break;

                    case "The %D is higher than the Level line":
                        indLogic = IndicatorLogic.The_indicator_is_higher_than_the_level_line;
                        SpecialValues = new double[2] { iLevel, 100 - iLevel };
                        break;

                    case "The %D is lower than the Level line":
                        indLogic = IndicatorLogic.The_indicator_is_lower_than_the_level_line;
                        SpecialValues = new double[2] { iLevel, 100 - iLevel };
                        break;

                    case "The %D crosses the Level line upward":
                        indLogic = IndicatorLogic.The_indicator_crosses_the_level_line_upward;
                        SpecialValues = new double[2] { iLevel, 100 - iLevel };
                        break;

                    case "The %D crosses the Level line downward":
                        indLogic = IndicatorLogic.The_indicator_crosses_the_level_line_downward;
                        SpecialValues = new double[2] { iLevel, 100 - iLevel };
                        break;

                    case "The %D changes its direction upward":
                        indLogic = IndicatorLogic.The_indicator_changes_its_direction_upward;
                        SpecialValues = new double[1] { 50 };
                        break;

                    case "The %D changes its direction downward":
                        indLogic = IndicatorLogic.The_indicator_changes_its_direction_downward;
                        SpecialValues = new double[1] { 50 };
                        break;

                    default:
                        break;
                }

                OscillatorLogic(iFirstBar, iPrvs, adD, iLevel, 100 - iLevel, ref Component[2], ref Component[3], indLogic);
            }

            return;
        }

        /// <summary>
        /// Sets the indicator logic description
        /// </summary>
        public override void SetDescription(SlotTypes slotType)
        {
            string sLevelLong  = IndParam.NumParam[3].ValueToString;
            string sLevelShort = IndParam.NumParam[3].AnotherValueToString(100 - IndParam.NumParam[3].Value);

            EntryFilterLongDescription  = ToString() + " - ";
            EntryFilterShortDescription = ToString() + " - ";
            ExitFilterLongDescription   = ToString() + " - ";
            ExitFilterShortDescription  = ToString() + " - ";

            switch (IndParam.ListParam[0].Text)
            {
                case "The %D rises":
                    EntryFilterLongDescription  += "the %D rises";
                    EntryFilterShortDescription += "the %D falls";
                    ExitFilterLongDescription   += "the %D rises";
                    ExitFilterShortDescription  += "the %D falls";
                    break;

                case "The %D falls":
                    EntryFilterLongDescription  += "the %D falls";
                    EntryFilterShortDescription += "the %D rises";
                    ExitFilterLongDescription   += "the %D falls";
                    ExitFilterShortDescription  += "the %D rises";
                    break;

                case "The %D is higher than the Level line":
                    EntryFilterLongDescription  += "the %D is higher than the Level " + sLevelLong;
                    EntryFilterShortDescription += "the %D is lower than the Level "  + sLevelShort;
                    ExitFilterLongDescription   += "the %D is higher than the Level " + sLevelLong;
                    ExitFilterShortDescription  += "the %D is lower than the Level "  + sLevelShort;
                    break;

                case "The %D is lower than the Level line":
                    EntryFilterLongDescription  += "the %D is lower than the Level "  + sLevelLong;
                    EntryFilterShortDescription += "the %D is higher than the Level " + sLevelShort;
                    ExitFilterLongDescription   += "the %D is lower than the Level "  + sLevelLong;
                    ExitFilterShortDescription  += "the %D is higher than the Level " + sLevelShort;
                    break;

                case "The %D crosses the Level line upward":
                    EntryFilterLongDescription  += "the %D crosses the Level " + sLevelLong  + " upward";
                    EntryFilterShortDescription += "the %D crosses the Level " + sLevelShort + " downward";
                    ExitFilterLongDescription   += "the %D crosses the Level " + sLevelLong  + " upward";
                    ExitFilterShortDescription  += "the %D crosses the Level " + sLevelShort + " downward";
                    break;

                case "The %D crosses the Level line downward":
                    EntryFilterLongDescription  += "the %D crosses the Level " + sLevelLong  + " downward";
                    EntryFilterShortDescription += "the %D crosses the Level " + sLevelShort + " upward";
                    ExitFilterLongDescription   += "the %D crosses the Level " + sLevelLong  + " downward";
                    ExitFilterShortDescription  += "the %D crosses the Level " + sLevelShort + " upward";
                    break;

                case "The %K crosses the %D upward":
                    EntryFilterLongDescription  += "the %K crosses the %D upward";
                    EntryFilterShortDescription += "the %K crosses the %D downward";
                    ExitFilterLongDescription   += "the %K crosses the %D upward";
                    ExitFilterShortDescription  += "the %K crosses the %D downward";
                    break;

                case "The %K crosses the %D downward":
                    EntryFilterLongDescription  += "the %K crosses the %D downward";
                    EntryFilterShortDescription += "the %K crosses the %D upward";
                    ExitFilterLongDescription   += "the %K crosses the %D downward";
                    ExitFilterShortDescription  += "the %K crosses the %D upward";
                    break;

                case "The %K is higher than the %D":
                    EntryFilterLongDescription  += "the %K is higher than the %D";
                    EntryFilterShortDescription += "the %K is lower than the %D";
                    ExitFilterLongDescription   += "the %K is higher than the %D";
                    ExitFilterShortDescription  += "the %K is lower than the %D";
                    break;

                case "The %K is lower than  the %D":
                    EntryFilterLongDescription  += "the %K is lower than the %D";
                    EntryFilterShortDescription += "the %K is higher than than the %D";
                    ExitFilterLongDescription   += "the %K is lower than the %D";
                    ExitFilterShortDescription  += "the %K is higher than than the %D";
                    break;

                case "The %D changes its direction upward":
                    EntryFilterLongDescription  += "the %D changes its direction upward";
                    EntryFilterShortDescription += "the %D changes its direction downward";
                    ExitFilterLongDescription   += "the %D changes its direction upward";
                    ExitFilterShortDescription  += "the %D changes its direction downward";
                    break;

                case "The %D changes its direction downward":
                    EntryFilterLongDescription  += "the %D changes its direction downward";
                    EntryFilterShortDescription += "the %D changes its direction upward";
                    ExitFilterLongDescription   += "the %D changes its direction downward";
                    ExitFilterShortDescription  += "the %D changes its direction upward";
                    break;

                default:
                    break;
            }

            return;
        }

        /// <summary>
        /// Indicator to string
        /// </summary>
        public override string ToString()
        {
            string sString = IndicatorName + 
                (IndParam.CheckParam[0].Checked ? "* (" : " (") +
                IndParam.ListParam[1].Text         + ", " + // Smoothing method
                IndParam.NumParam[0].ValueToString + ", " + // %K Min period
                IndParam.NumParam[1].ValueToString + ", " + // %K Max period
                IndParam.NumParam[2].ValueToString + ")";   // %D period

            return sString;
        }
    
		protected static double[] Highest(double[] adHigh, int iLen)
		{
			double[] adHighest = new double[Bars];

            for (int iBar = iLen; iBar < Bars; iBar++)
            {
                double dMax = double.MinValue;
                for (int iBack = 0; iBack < iLen; iBack++)
                {
                    if (adHigh[iBar-iBack] > dMax) dMax = adHigh[iBar-iBack];
                }
                adHighest[iBar] = dMax;
            }			
			return adHighest;
		}
		
		protected static double[] Lowest(double[] adLow, int iLen)
		{
            double[] adLowest  = new double[Bars];

            for (int iBar = iLen; iBar < Bars; iBar++)
            {
                double dMin = double.MaxValue;
                for (int iBack = 0; iBack < iLen; iBack++)
                {
                    if (adLow[iBar-iBack] < dMin) dMin = adLow[iBar-iBack];
                }
                adLowest[iBar]  = dMin;
            }			
			return adLowest;
		}
		
		protected static double[] StdDev(double[] adPrice, int iLen)
		{
			double[] adStdDev = new double[Bars];
			double[] adVar = new double[Bars];
			
			adVar = Var(adPrice, iLen);
			
			for (int iBar = iLen; iBar<Bars; iBar++)
			{
				adStdDev[iBar] = Math.Sqrt(adVar[iBar]);
			}
			
			return adStdDev;
		}
		
		protected static double[] Var(double[] adPrice, int iLen)
		{
			double[] adVar = new double[Bars];
			double[] adMean = new double[Bars];
			double dSum;
			
			adMean = Avg(adPrice, iLen);
			
			for (int iBar = iLen; iBar<Bars; iBar++)
			{
				dSum = 0;
				for (int iBack = 0; iBack<iLen; iBack++)
				{
					dSum = dSum + Math.Pow(adPrice[iBar-iBack]-adMean[iBar-iBack],2);
				}
				adVar[iBar] = dSum/iLen;
			}			
			return adVar;	
		}
	
		protected static double[] Avg(double[] adPrice, int iLen)
		{
			double[] adAvg = new double[Bars];
			double dSum;
			
			for (int iBar = iLen; iBar<Bars; iBar++)
			{
				dSum = 0;
				for (int iBack = 0; iBack<iLen; iBack++)
				{
					dSum = dSum + adPrice[iBar-iBack];
				}
				adAvg[iBar] = dSum/iLen;
			}			
			return adAvg;
		}
	
				/// <summary>
        /// Convert current time frame to higher time frame
        /// </summary>
		protected static int HigherTimeFrame(DataPeriods currentTF, DataPeriods higherTF, out int[] hIndex, out int hBars, out int iFrame, out double[] hfHigh, out double[] hfLow, out double[] hfOpen, out double[] hfClose, out double[] hfVolume)
		{
			hfOpen = new double[Bars];
			hfClose = new double[Bars];
			hfHigh = new double[Bars];
			hfLow = new double[Bars];
			hfVolume = new double[Bars];
			hIndex = new int[Bars];
			hBars =0;
			int iRem =10;
			
			
			// Frame Calculation
			if (higherTF>currentTF)
				iFrame = (int)((int)higherTF/(int)currentTF);
			else
				iFrame = 1;
			if (higherTF==DataPeriods.week && currentTF==DataPeriods.day) iFrame=5;
			if (iFrame>60) return 1;
			
			int hBar=0;
			for (int iBar=1; iBar<Bars; iBar++)
			{			
					
				// Time Synchronization
				int iMin  = (int) Time[iBar].Minute;
				int iHour = (int) Time[iBar].Hour;
				int iDay  = (int) Time[iBar].DayOfWeek;
				if ((int)higherTF<=60 && (int)currentTF<=30)	iRem=(int)((iMin%(int)higherTF)/(int)currentTF);
				else if (higherTF==DataPeriods.hour4 && currentTF==DataPeriods.min15) 	iRem=(int)((iHour%4)*4+(iMin/15));
				else if (higherTF==DataPeriods.hour4 && currentTF==DataPeriods.min30) 	iRem=(int)((iHour%4)*2+(iMin/30));
				else if (higherTF==DataPeriods.hour4 && currentTF==DataPeriods.hour1) 	iRem=(int)(iHour%4);
				else if (higherTF==DataPeriods.day && currentTF==DataPeriods.min30)		iRem=(int)((iHour%24)*2+(iMin/30));
				else if (higherTF==DataPeriods.day && currentTF==DataPeriods.hour1)		iRem=(int)(iHour%24);
				else if (higherTF==DataPeriods.day && currentTF==DataPeriods.hour4)		iRem=(int)((iHour%24)/4);
				else if (higherTF==DataPeriods.week && currentTF==DataPeriods.day)		iRem=(int)((iDay-1)%7);
				else	iRem = iBar % iFrame;
				
				// Higher Time Frame Conversion
				if (iRem == 0)
				{
					hBar++;
					hfOpen[hBar]   = Open[iBar];
					hfClose[hBar]  = Close[iBar];
					hfHigh[hBar]   = High[iBar];
					hfLow[hBar]    = Low[iBar];
					hfVolume[hBar] = Volume[iBar];
				}
				else
				{
					hfClose[hBar]  = Close[iBar];
					if (High[iBar] > hfHigh[hBar])	hfHigh[hBar] = High[iBar];
					if (Low[iBar]  < hfLow[hBar])	hfLow[hBar] = Low[iBar];
					hfVolume[hBar] += Volume[iBar];
				}
				hIndex[iBar] = hBar;
			}
			hBars = hBar+1;
			
			return 0;
		}

		/// <summary>
        /// Convert higher time frame to current time frame
        /// </summary>
		protected static int CurrentTimeFrame(int[] hIndex, int hBars, ref double[] hIndicator)
		{
			double[] hBuffer = new double[Bars];
			int hBar;
			
			for (int iBar=0; iBar<Bars; iBar++)
			{	
				hBar = hIndex[iBar];	
				hBuffer[iBar] = hIndicator[hBar];
			}
			
			for (int iBar=0; iBar<Bars; iBar++)
			{
				hIndicator[iBar] = hBuffer[iBar];
			}
			
			return 0;
		}
				
		/// <summary>
        /// Calculate base price for higher time frame data
        /// </summary>
		protected static int HigherBasePrice(BasePrice basePrice, int hBars, double[] hfHigh, double[] hfLow, double[] hfOpen, double[] hfClose, out double[] hfPrice)
		{
			hfPrice = new double[Bars];
			
			switch (basePrice)
			{
				case BasePrice.Open:	hfPrice = hfOpen;	break;
				case BasePrice.Close:	hfPrice = hfClose;	break;
				case BasePrice.High:	hfPrice = hfHigh;	break;
				case BasePrice.Low:		hfPrice = hfLow;	break;
				case BasePrice.Median:
					for (int iBar=0; iBar<hBars; iBar++)
					{
						hfPrice[iBar] = (hfLow[iBar] + hfHigh[iBar]) / 2;
					}
					break;
				case BasePrice.Typical:
					for (int iBar=0; iBar<hBars; iBar++)
					{
						hfPrice[iBar] = (hfLow[iBar] + hfHigh[iBar] + hfClose[iBar]) / 3;
					}
					break;
				case BasePrice.Weighted:
					for (int iBar=0; iBar<hBars; iBar++)
					{
						hfPrice[iBar] = (hfOpen[iBar] + hfLow[iBar] + hfHigh[iBar] + hfClose[iBar]) / 4;
					}
					break;
				default:
					break;
			}
			return 0;
		}


	}
}
