// Bandpass Filter Indicator (Modified to HTF v 4.0.0 by Denny Imanuel)
// Last changed on 2 Mar 2010
// Part of Forex Strategy Builder v2.8.3.7+
// Website http://forexsb.com/
// This code or any part of it cannot be used in other applications without a permission.
// Copyright (c) 2010 Denny Imanuel - All rights reserved.

using System;
using System.Drawing;

namespace Forex_Strategy_Builder
{
    /// <summary>
    /// Bandpass Filter Indicator
    /// </summary>
    public class Bandpass_Filter_HTF : Indicator
    {
        /// <summary>
        /// Sets the default indicator parameters for the designated slot type
        /// </summary>
        public Bandpass_Filter_HTF(SlotTypes slotType)
        {
            // General properties
            IndicatorName  = "Bandpass Filter (HTF v4)";
            PossibleSlots  = SlotTypes.OpenFilter | SlotTypes.CloseFilter;
            SeparatedChart = true;
			CustomIndicator = true;

            // Setting up the indicator parameters
            IndParam = new IndicatorParam();
            IndParam.IndicatorName = IndicatorName;
            IndParam.SlotType      = slotType;

            // The ComboBox parameters
            IndParam.ListParam[0].Caption  = "Logic";
            IndParam.ListParam[0].ItemList = new string[]
            {
                "The Bandpass Filter line rises",
                "The Bandpass Filter line falls",
                "The Bandpass Filter line is higher than zero",
                "The Bandpass Filter line is lower than zero",
                "The Bandpass Filter line crosses the zero line upward",
                "The Bandpass Filter line crosses the zero line downward",
                "The Bandpass Filter line changes its direction upward",
                "The Bandpass Filter line changes its direction downward",
                "The Bandpass Filter line crosses the Average Peak upward",
                "The Bandpass Filter line crosses the Average Valley downward",
                "The Bandpass Filter line is higher than the Average Peak",
                "The Bandpass Filter line is lower than the Average Valley"
            };
            IndParam.ListParam[0].Index    = 0;
            IndParam.ListParam[0].Text     = IndParam.ListParam[0].ItemList[IndParam.ListParam[0].Index];
            IndParam.ListParam[0].Enabled  = true;
            IndParam.ListParam[0].ToolTip  = "Logic of application of the indicator.";

             IndParam.ListParam[1].Caption  = "Smoothing method";
            IndParam.ListParam[1].ItemList = Enum.GetNames(typeof(MAMethod));
            IndParam.ListParam[1].Index    = (int)MAMethod.Simple;
            IndParam.ListParam[1].Text     = IndParam.ListParam[1].ItemList[IndParam.ListParam[1].Index];
            IndParam.ListParam[1].Enabled  = true;
            IndParam.ListParam[1].ToolTip  = "The smoothing method of Moving Average.";

            IndParam.ListParam[2].Caption  = "Base price";
            IndParam.ListParam[2].ItemList = Enum.GetNames(typeof(BasePrice));
            IndParam.ListParam[2].Index    = (int)BasePrice.Median;
            IndParam.ListParam[2].Text     = IndParam.ListParam[2].ItemList[IndParam.ListParam[2].Index];
            IndParam.ListParam[2].Enabled  = true;
            IndParam.ListParam[2].ToolTip  = "The price the Moving Average is based on.";

			IndParam.ListParam[4].Caption = "Higher Time Frame Reference";
			IndParam.ListParam[4].ItemList = new string[] { "1 Minute", "5 Minutes", "15 Minutes", "30 Minutes", "1 Hour", "4 Hours", "1 Day", "1 Week"};
            if (Period == DataPeriods.min1) 		IndParam.ListParam[4].Index    = 1;
            else if (Period == DataPeriods.min5) 	IndParam.ListParam[4].Index    = 2;
            else if (Period == DataPeriods.min15) 	IndParam.ListParam[4].Index    = 3;
            else if (Period == DataPeriods.min30)	IndParam.ListParam[4].Index    = 4;
            else if (Period == DataPeriods.hour1)	IndParam.ListParam[4].Index    = 5;
            else if (Period == DataPeriods.hour4)	IndParam.ListParam[4].Index    = 6;
            else if (Period == DataPeriods.day)		IndParam.ListParam[4].Index    = 7;
            IndParam.ListParam[4].Text     = IndParam.ListParam[4].ItemList[IndParam.ListParam[4].Index];
            IndParam.ListParam[4].Enabled  = true;
            IndParam.ListParam[4].ToolTip  = "Choose higher time frame as compared to current chart period for this indicator.";		

			
			// The NumericUpDown parameters
            IndParam.NumParam[0].Caption = "Bandpass Period";
            IndParam.NumParam[0].Value   = 10;
            IndParam.NumParam[0].Min     = 1;
            IndParam.NumParam[0].Max     = 200;
            IndParam.NumParam[0].Enabled = true;
            IndParam.NumParam[0].ToolTip = "The period of bandpass filter calculation.";
			
			IndParam.NumParam[1].Caption = "Smoothing Period";
            IndParam.NumParam[1].Value   = 20;
            IndParam.NumParam[1].Min     = 1;
            IndParam.NumParam[1].Max     = 200;
            IndParam.NumParam[1].Enabled = true;
            IndParam.NumParam[1].ToolTip = "The period of bandpass filter calculation.";

			IndParam.NumParam[2].Caption = "Bandpass Delta";
            IndParam.NumParam[2].Value   = 0.5;
            IndParam.NumParam[2].Min     = 0;
            IndParam.NumParam[2].Max     = 10;
			IndParam.NumParam[2].Point	 = 2;
            IndParam.NumParam[2].Enabled = true;
            IndParam.NumParam[2].ToolTip = "The delta value for bandpass filter.";
			
			IndParam.NumParam[3].Caption = "Bandwidth Multiplier";
            IndParam.NumParam[3].Value   = 0.1;
            IndParam.NumParam[3].Min     = 0;
            IndParam.NumParam[3].Max     = 10;
			IndParam.NumParam[3].Point	 = 2;
            IndParam.NumParam[3].Enabled = true;
            IndParam.NumParam[3].ToolTip = "The divisor to determine the upper and lower bandwidth.";
			
            // The CheckBox parameters
            IndParam.CheckParam[0].Caption = "Use previous bar value";
            IndParam.CheckParam[0].Checked = PrepareUsePrevBarValueCheckBox(slotType);
            IndParam.CheckParam[0].Enabled = true;
            IndParam.CheckParam[0].ToolTip = "Use the indicator value from the previous bar.";
			
			IndParam.CheckParam[1].Caption = "Use smoothed bandpass filter";
            IndParam.CheckParam[1].Checked = true;
            IndParam.CheckParam[1].Enabled = true;
            IndParam.CheckParam[1].ToolTip = "Use the smoothed value of bandpass filter.";

            return;
        }

        /// <summary>
        /// Calculates the indicator's components
        /// </summary>
        public override void Calculate(SlotTypes slotType)
        {
            // Reading the parameters
            MAMethod  maMethod 	= (MAMethod )IndParam.ListParam[1].Index;
            BasePrice basePrice = (BasePrice)IndParam.ListParam[2].Index;
			int 	iPeriod	= (int)IndParam.NumParam[0].Value;
			int 	iSmooth	= (int)IndParam.NumParam[1].Value;
			double 	Delta	= IndParam.NumParam[2].Value;
			double	dMpl	= IndParam.NumParam[3].Value;
            int 	iPrvs 	= IndParam.CheckParam[0].Checked ? 1 : 0;

            // Convert to Higher Time Frame ---------------------------------------------
			DataPeriods htfPeriod = DataPeriods.week;
			double[] hfOpen = new double[Bars];
			double[] hfClose = new double[Bars];
			double[] hfHigh = new double[Bars];
			double[] hfLow = new double[Bars];
			double[] hfVolume = new double[Bars];
			double[] hfPrice = new double[Bars];
			int[] 	 hIndex = new int[Bars];
			int      iFrame;
			int      hBars;
			
			switch (IndParam.ListParam[4].Index)
			{
				case 1: htfPeriod = DataPeriods.min5; break;
				case 2: htfPeriod = DataPeriods.min15; break;
				case 3: htfPeriod = DataPeriods.min30; break;
				case 4: htfPeriod = DataPeriods.hour1; break;
				case 5: htfPeriod = DataPeriods.hour4; break;
				case 6: htfPeriod = DataPeriods.day; break;
				case 7: htfPeriod = DataPeriods.week; break;
			}
			
			int err1 = HigherTimeFrame(Period, htfPeriod, out hIndex, out hBars, out iFrame, out hfHigh, out hfLow, out hfOpen, out hfClose, out hfVolume);
			int err2 = HigherBasePrice(basePrice, hBars, hfHigh, hfLow, hfOpen, hfClose, out hfPrice);	
			if (err1==1) return;
			//------------------------------------------------------------------------

			//double[] adPrice 	= Price(basePrice);
			double[] adBPFilter = new double[Bars];
			double[] adBPSignal 	= new double[Bars];
			double[] adBPPeak 	= new double[Bars];
			double[] adBPValley	= new double[Bars];
			double[] adAvgPeak 	= new double[Bars];
			double[] adAvgValley 	= new double[Bars];
			
			// Calculation
            int iFirstBar = iPeriod + 2;

			double Beta = Math.Cos(2*Math.PI/iPeriod);
			double Gamma = 1/Math.Cos(4*Math.PI*Delta/iPeriod);
			double Alpha = Gamma - Math.Sqrt(Math.Pow(Gamma,2)-1);
			double Theta = 0.5*(1-Alpha);
			double Omega = Beta*(1+Alpha);
            
			for (int iBar = 2; iBar < hBars; iBar++)
            {
				adBPFilter[iBar] = Theta*(hfPrice[iBar] - hfPrice[iBar-2]) + Omega*adBPFilter[iBar-1] - Alpha*adBPFilter[iBar-2];
				if (adBPFilter[iBar]<adBPFilter[iBar-1] && adBPFilter[iBar-1]>adBPFilter[iBar-2])
					adBPPeak[iBar] = adBPFilter[iBar-1];
				else
					adBPPeak[iBar] = adBPPeak[iBar-1];
				if (adBPFilter[iBar]>adBPFilter[iBar-1] && adBPFilter[iBar-1]<adBPFilter[iBar-2])
					adBPValley[iBar] = adBPFilter[iBar-1];
				else
					adBPValley[iBar] = adBPValley[iBar-1];
			}

            if (IndParam.CheckParam[1].Checked)
				adBPSignal = MovingAverage(iSmooth, 0, maMethod, adBPFilter);
			else
				adBPSignal = adBPFilter;
			adAvgPeak = MovingAverage(iSmooth, 0, maMethod, adBPPeak);
			adAvgValley = MovingAverage(iSmooth, 0, maMethod, adBPValley);
			
			for (int iBar = 2; iBar < hBars; iBar++)
            {
				adAvgPeak[iBar] = dMpl*adAvgPeak[iBar];
				adAvgValley[iBar] = dMpl*adAvgValley[iBar];
			}

			// Convert to Current Time Frame ----------------------------------------------
			iPrvs = iPrvs*iFrame;
			iFirstBar = iFirstBar*iFrame;
			if (iFirstBar>0.25*Bars) return;
			
			int err3 = CurrentTimeFrame(hIndex, hBars, ref adBPFilter);
			int err4 = CurrentTimeFrame(hIndex, hBars, ref adBPSignal);
			int err5 = CurrentTimeFrame(hIndex, hBars, ref adAvgPeak);
			int err6 = CurrentTimeFrame(hIndex, hBars, ref adAvgValley);			
			//-----------------------------------------------------------------------------				
			
            // Saving the components
            Component = new IndicatorComp[6];

            Component[0] = new IndicatorComp();
            Component[0].CompName   = "Bandpass Filter";
            Component[0].DataType   = IndComponentType.IndicatorValue;
            Component[0].ChartType  = IndChartType.Line;
            Component[0].ChartColor = Color.Purple;
            Component[0].FirstBar   = iFirstBar;
            Component[0].Value      = adBPFilter;

            Component[1] = new IndicatorComp();
            Component[1].CompName   = "BandPass Signal";
            Component[1].DataType   = IndComponentType.IndicatorValue;
            Component[1].ChartType  = IndChartType.Line;
            Component[1].ChartColor = Color.Pink;
            Component[1].FirstBar   = iFirstBar;
            Component[1].Value      = adBPSignal;

            Component[2] = new IndicatorComp();
            Component[2].ChartType = IndChartType.NoChart;
            Component[2].FirstBar  = iFirstBar;
            Component[2].Value     = new double[Bars];

            Component[3] = new IndicatorComp();
            Component[3].ChartType = IndChartType.NoChart;
            Component[3].FirstBar  = iFirstBar;
            Component[3].Value     = new double[Bars];
			
			Component[4] = new IndicatorComp();
            Component[4].CompName   = "Average Peak";
            Component[4].DataType   = IndComponentType.IndicatorValue;
            Component[4].ChartType  = IndChartType.Line;
            Component[4].ChartColor = Color.Green;
            Component[4].FirstBar   = iFirstBar;
            Component[4].Value      = adAvgPeak;
			
			Component[5] = new IndicatorComp();
            Component[5].CompName   = "Average Valley";
            Component[5].DataType   = IndComponentType.IndicatorValue;
            Component[5].ChartType  = IndChartType.Line;
            Component[5].ChartColor = Color.Red;
            Component[5].FirstBar   = iFirstBar;
            Component[5].Value      = adAvgValley;

            // Sets the Component's type
            if (slotType == SlotTypes.OpenFilter)
            {
                Component[2].DataType = IndComponentType.AllowOpenLong;
                Component[2].CompName = "Is long entry allowed";
                Component[3].DataType = IndComponentType.AllowOpenShort;
                Component[3].CompName = "Is short entry allowed";
            }
            else if (slotType == SlotTypes.CloseFilter)
            {
                Component[2].DataType = IndComponentType.ForceCloseLong;
                Component[2].CompName = "Close out long position";
                Component[3].DataType = IndComponentType.ForceCloseShort;
                Component[3].CompName = "Close out short position";
            }

            switch (IndParam.ListParam[0].Text)
            {
                case "The Bandpass Filter line rises":
                    OscillatorLogic(iFirstBar, iPrvs, adBPSignal, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_rises);
                    break;

                case "The Bandpass Filter line falls":
                    OscillatorLogic(iFirstBar, iPrvs, adBPSignal, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_falls);
                    break;

                case "The Bandpass Filter line is higher than zero":
                    OscillatorLogic(iFirstBar, iPrvs, adBPSignal, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_is_higher_than_the_level_line);
                    break;

                case "The Bandpass Filter line is lower than zero":
                    OscillatorLogic(iFirstBar, iPrvs, adBPSignal, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_is_lower_than_the_level_line);
                    break;

                case "The Bandpass Filter line crosses the zero line upward":
                    OscillatorLogic(iFirstBar, iPrvs, adBPSignal, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_crosses_the_level_line_upward);
                    break;

                case "The Bandpass Filter line crosses the zero line downward":
                    OscillatorLogic(iFirstBar, iPrvs, adBPSignal, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_crosses_the_level_line_downward);
                    break;

                case "The Bandpass Filter line changes its direction upward":
                    OscillatorLogic(iFirstBar, iPrvs, adBPSignal, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_changes_its_direction_upward);
                    break;

                case "The Bandpass Filter line changes its direction downward":
                    OscillatorLogic(iFirstBar, iPrvs, adBPSignal, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_changes_its_direction_downward);
                    break;

                case "The Bandpass Filter line crosses the Average Peak upward":
                    IndicatorCrossesAnotherIndicatorUpwardLogic(iFirstBar, iPrvs, adBPSignal, adAvgPeak, ref Component[2], ref Component[3]);
                    break;

                case "The Bandpass Filter line crosses the Average Valley downward":
                    IndicatorCrossesAnotherIndicatorDownwardLogic(iFirstBar, iPrvs, adBPSignal, adAvgValley, ref Component[2], ref Component[3]);
                    break;

                case "The Bandpass Filter line is higher than the Average Peak":
                    IndicatorIsHigherThanAnotherIndicatorLogic(iFirstBar, iPrvs, adBPSignal, adAvgPeak, ref Component[2], ref Component[3]);
                    break;

                case "The Bandpass Filter line is lower than the Average Valley":
                    IndicatorIsLowerThanAnotherIndicatorLogic(iFirstBar, iPrvs, adBPSignal, adAvgValley, ref Component[2], ref Component[3]);
                    break;

                default:
                    break;
            }

            return;
        }

        /// <summary>
        /// Sets the indicator logic description
        /// </summary>
        public override void SetDescription(SlotTypes slotType)
        {
            EntryFilterLongDescription  = ToString() + "; the Bandpass Filter line ";
            EntryFilterShortDescription = ToString() + "; the Bandpass Filter line ";
            ExitFilterLongDescription   = ToString() + "; the Bandpass Filter line ";
            ExitFilterShortDescription  = ToString() + "; the Bandpass Filter line ";

            switch (IndParam.ListParam[0].Text)
            {
                case "The Bandpass Filter line rises":
                    EntryFilterLongDescription  += "rises";
                    EntryFilterShortDescription += "falls";
                    ExitFilterLongDescription   += "rises";
                    ExitFilterShortDescription  += "falls";
                    break;

                case "The Bandpass Filter line falls":
                    EntryFilterLongDescription  += "falls";
                    EntryFilterShortDescription += "rises";
                    ExitFilterLongDescription   += "falls";
                    ExitFilterShortDescription  += "rises";
                    break;

                case "The Bandpass Filter line is higher than zero":
                    EntryFilterLongDescription  += "is higher than the zero line";
                    EntryFilterShortDescription += "is lower than the zero line";
                    ExitFilterLongDescription   += "is higher than the zero line";
                    ExitFilterShortDescription  += "is lower than the zero line";
                    break;

                case "The Bandpass Filter line is lower than zero":
                    EntryFilterLongDescription  += "is lower than the zero line";
                    EntryFilterShortDescription += "is higher than the zero line";
                    ExitFilterLongDescription   += "is lower than the zero line";
                    ExitFilterShortDescription  += "is higher than the zero line";
                    break;

                case "The Bandpass Filter line crosses the zero line upward":
                    EntryFilterLongDescription  += "crosses the zero line upward";
                    EntryFilterShortDescription += "crosses the zero line downward";
                    ExitFilterLongDescription   += "crosses the zero line upward";
                    ExitFilterShortDescription  += "crosses the zero line downward";
                    break;

                case "The Bandpass Filter line crosses the zero line downward":
                    EntryFilterLongDescription  += "crosses the zero line downward";
                    EntryFilterShortDescription += "crosses the zero line upward";
                    ExitFilterLongDescription   += "crosses the zero line downward";
                    ExitFilterShortDescription  += "crosses the zero line upward";
                    break;

                case "The Bandpass Filter line changes its direction upward":
                    EntryFilterLongDescription  += "changes its direction upward";
                    EntryFilterShortDescription += "changes its direction downward";
                    ExitFilterLongDescription   += "changes its direction upward";
                    ExitFilterShortDescription  += "changes its direction downward";
                    break;

                case "The Bandpass Filter line changes its direction downward":
                    EntryFilterLongDescription  += "changes its direction downward";
                    EntryFilterShortDescription += "changes its direction upward";
                    ExitFilterLongDescription   += "changes its direction downward";
                    ExitFilterShortDescription  += "changes its direction upward";
                    break;

                case "The Bandpass Filter line is higher than the Average Peak":
                    EntryFilterLongDescription  += "is higher than the Average Peak";
                    EntryFilterShortDescription += "is lower than the Average Peak";
                    ExitFilterLongDescription   += "is higher than the Average Peak";
                    ExitFilterShortDescription  += "is lower than the Average Peak";
                    break;

                case "The Bandpass Filter line is lower than the Average Valley":
                    EntryFilterLongDescription  += "is lower than the Average Valley";
                    EntryFilterShortDescription += "is higher than the Average Valley";
                    ExitFilterLongDescription   += "is lower than the Average Valley";
                    ExitFilterShortDescription  += "is higher than the Average Valley";
                    break;

                case "The Bandpass Filter line crosses the Average Peak upward":
                    EntryFilterLongDescription  += "crosses the Average Peak upward";
                    EntryFilterShortDescription += "crosses the Average Peak downward";
                    ExitFilterLongDescription   += "crosses the Average Peak upward";
                    ExitFilterShortDescription  += "crosses the Average Peak downward";
                    break;

                case "The Bandpass Filter line crosses the Average Valley downward":
                    EntryFilterLongDescription  += "crosses the Average Valley downward";
                    EntryFilterShortDescription += "crosses the Average Valley upward";
                    ExitFilterLongDescription   += "crosses the Average Valley downward";
                    ExitFilterShortDescription  += "crosses the Average Valley upward";
                    break;

                default:
                    break;
            }

            return;
        }

        /// <summary>
        /// Indicator to string
        /// </summary>
        public override string ToString()
        {
            string sString = IndicatorName +
                (IndParam.CheckParam[0].Checked ? "* (" : " (") +
                IndParam.NumParam[0].ValueToString + ")";  // Bandpass Filter period

            return sString;
        }
		
				/// <summary>
        /// Convert current time frame to higher time frame
        /// </summary>
		protected static int HigherTimeFrame(DataPeriods currentTF, DataPeriods higherTF, out int[] hIndex, out int hBars, out int iFrame, out double[] hfHigh, out double[] hfLow, out double[] hfOpen, out double[] hfClose, out double[] hfVolume)
		{
			hfOpen = new double[Bars];
			hfClose = new double[Bars];
			hfHigh = new double[Bars];
			hfLow = new double[Bars];
			hfVolume = new double[Bars];
			hIndex = new int[Bars];
			hBars =0;
			int iRem =10;
			
			
			// Frame Calculation
			if (higherTF>currentTF)
				iFrame = (int)((int)higherTF/(int)currentTF);
			else
				iFrame = 1;
			if (higherTF==DataPeriods.week && currentTF==DataPeriods.day) iFrame=5;
			if (iFrame>60) return 1;
			
			int hBar=0;
			for (int iBar=1; iBar<Bars; iBar++)
			{			
					
				// Time Synchronization
				int iMin  = (int) Time[iBar].Minute;
				int iHour = (int) Time[iBar].Hour;
				int iDay  = (int) Time[iBar].DayOfWeek;
				if ((int)higherTF<=60 && (int)currentTF<=30)	iRem=(int)((iMin%(int)higherTF)/(int)currentTF);
				else if (higherTF==DataPeriods.hour4 && currentTF==DataPeriods.min15) 	iRem=(int)((iHour%4)*4+(iMin/15));
				else if (higherTF==DataPeriods.hour4 && currentTF==DataPeriods.min30) 	iRem=(int)((iHour%4)*2+(iMin/30));
				else if (higherTF==DataPeriods.hour4 && currentTF==DataPeriods.hour1) 	iRem=(int)(iHour%4);
				else if (higherTF==DataPeriods.day && currentTF==DataPeriods.min30)		iRem=(int)((iHour%24)*2+(iMin/30));
				else if (higherTF==DataPeriods.day && currentTF==DataPeriods.hour1)		iRem=(int)(iHour%24);
				else if (higherTF==DataPeriods.day && currentTF==DataPeriods.hour4)		iRem=(int)((iHour%24)/4);
				else if (higherTF==DataPeriods.week && currentTF==DataPeriods.day)		iRem=(int)((iDay-1)%7);
				else	iRem = iBar % iFrame;
				
				// Higher Time Frame Conversion
				if (iRem == 0)
				{
					hBar++;
					hfOpen[hBar]   = Open[iBar];
					hfClose[hBar]  = Close[iBar];
					hfHigh[hBar]   = High[iBar];
					hfLow[hBar]    = Low[iBar];
					hfVolume[hBar] = Volume[iBar];
				}
				else
				{
					hfClose[hBar]  = Close[iBar];
					if (High[iBar] > hfHigh[hBar])	hfHigh[hBar] = High[iBar];
					if (Low[iBar]  < hfLow[hBar])	hfLow[hBar] = Low[iBar];
					hfVolume[hBar] += Volume[iBar];
				}
				hIndex[iBar] = hBar;
			}
			hBars = hBar+1;
			
			return 0;
		}

		/// <summary>
        /// Convert higher time frame to current time frame
        /// </summary>
		protected static int CurrentTimeFrame(int[] hIndex, int hBars, ref double[] hIndicator)
		{
			double[] hBuffer = new double[Bars];
			int hBar;
			
			for (int iBar=0; iBar<Bars; iBar++)
			{	
				hBar = hIndex[iBar];	
				hBuffer[iBar] = hIndicator[hBar];
			}
			
			for (int iBar=0; iBar<Bars; iBar++)
			{
				hIndicator[iBar] = hBuffer[iBar];
			}
			
			return 0;
		}
				
		/// <summary>
        /// Calculate base price for higher time frame data
        /// </summary>
		protected static int HigherBasePrice(BasePrice basePrice, int hBars, double[] hfHigh, double[] hfLow, double[] hfOpen, double[] hfClose, out double[] hfPrice)
		{
			hfPrice = new double[Bars];
			
			switch (basePrice)
			{
				case BasePrice.Open:	hfPrice = hfOpen;	break;
				case BasePrice.Close:	hfPrice = hfClose;	break;
				case BasePrice.High:	hfPrice = hfHigh;	break;
				case BasePrice.Low:		hfPrice = hfLow;	break;
				case BasePrice.Median:
					for (int iBar=0; iBar<hBars; iBar++)
					{
						hfPrice[iBar] = (hfLow[iBar] + hfHigh[iBar]) / 2;
					}
					break;
				case BasePrice.Typical:
					for (int iBar=0; iBar<hBars; iBar++)
					{
						hfPrice[iBar] = (hfLow[iBar] + hfHigh[iBar] + hfClose[iBar]) / 3;
					}
					break;
				case BasePrice.Weighted:
					for (int iBar=0; iBar<hBars; iBar++)
					{
						hfPrice[iBar] = (hfOpen[iBar] + hfLow[iBar] + hfHigh[iBar] + hfClose[iBar]) / 4;
					}
					break;
				default:
					break;
			}
			return 0;
		}


    }
}
