// Bar Closing Indicator (Modified to HTF v 4.0.0 by Denny Imanuel)
// Last changed on 2009-04-15
// Part of Forex Strategy Builder v2.8.3.5+
// Website http://forexsb.com/
// Copyright (c) 2006 - 2009 Miroslav Popov - All rights reserved.
// This code or any part of it cannot be used in other applications without a permission.

namespace Forex_Strategy_Builder
{
    /// <summary>
    /// Bar Closing Indicator
    /// </summary>
    public class Bar_Closing_HTF : Indicator
	{
        /// <summary>
        /// Sets the default indicator parameters for the designated slot type
        /// </summary>
        public Bar_Closing_HTF(SlotTypes slotType)
		{
            // General properties
            IndicatorName = "Bar Closing (HTF v4)";
            PossibleSlots = SlotTypes.Open | SlotTypes.Close;
			CustomIndicator = true;

            // Setting up the indicator parameters
            IndParam = new IndicatorParam();
            IndParam.IndicatorName = IndicatorName;
            IndParam.SlotType      = slotType;
            IndParam.IndicatorType = TypeOfIndicator.Additional;
            IndParam.ExecutionTime = ExecutionTime.AtBarClosing;

            // The ComboBox parameters
            IndParam.ListParam[0].Caption = "Logic";
            if (slotType == SlotTypes.Open)
            {
                IndParam.ListParam[0].ItemList = new string [] { "Enter the market at the end of the bar" };
                IndParam.ListParam[1].ToolTip  = "The execution price of all entry orders.";
            }
            else
            {
                IndParam.ListParam[0].ItemList = new string[] { "Exit the market at the end of the bar" };
                IndParam.ListParam[1].ToolTip  = "The execution price of all exit orders.";
            }
            IndParam.ListParam[0].Index   = 0;
            IndParam.ListParam[0].Text    = IndParam.ListParam[0].ItemList[IndParam.ListParam[0].Index];
            IndParam.ListParam[0].Enabled = true;
            IndParam.ListParam[0].ToolTip = "Logic of application of the indicator.";

            IndParam.ListParam[1].Caption  = "Base price";
            IndParam.ListParam[1].ItemList = new string[] { "Close" };
            IndParam.ListParam[1].Index    = 0;
            IndParam.ListParam[1].Text     = IndParam.ListParam[1].ItemList[IndParam.ListParam[1].Index];
            IndParam.ListParam[1].Enabled  = true;
			
			IndParam.ListParam[4].Caption = "Higher Time Frame Reference";
			IndParam.ListParam[4].ItemList = new string[] { "Automatic", "5 Minutes", "15 Minutes", "30 Minutes", "1 Hour", "4 Hours", "1 Day", "1 Week"};
            if (Period == DataPeriods.min1) 		IndParam.ListParam[4].Index    = 1;
            else if (Period == DataPeriods.min5) 	IndParam.ListParam[4].Index    = 2;
            else if (Period == DataPeriods.min15) 	IndParam.ListParam[4].Index    = 3;
            else if (Period == DataPeriods.min30)	IndParam.ListParam[4].Index    = 4;
            else if (Period == DataPeriods.hour1)	IndParam.ListParam[4].Index    = 5;
            else if (Period == DataPeriods.hour4)	IndParam.ListParam[4].Index    = 6;
            else if (Period == DataPeriods.day)		IndParam.ListParam[4].Index    = 7;
            IndParam.ListParam[4].Text     = IndParam.ListParam[4].ItemList[IndParam.ListParam[4].Index];
            IndParam.ListParam[4].Enabled  = true;
            IndParam.ListParam[4].ToolTip  = "Choose higher time frame as compared to current chart period for this indicator.";		


            return;
        }

        /// <summary>
        /// Calculates the indicator's components
        /// </summary>
        public override void Calculate(SlotTypes slotType)
        {
			DataPeriods htfPeriod = DataPeriods.week;
			double[] fltClose = new double[Bars];
			
			switch (IndParam.ListParam[4].Index)
			{
				case 1: htfPeriod = DataPeriods.min5; break;
				case 2: htfPeriod = DataPeriods.min15; break;
				case 3: htfPeriod = DataPeriods.min30; break;
				case 4: htfPeriod = DataPeriods.hour1; break;
				case 5: htfPeriod = DataPeriods.hour4; break;
				case 6: htfPeriod = DataPeriods.day; break;
				case 7: htfPeriod = DataPeriods.week; break;
			}
			
			// Filtering the Closing Price Based on Bar;
			int err = HigherPriceFilter(Period, htfPeriod, "Price Closing", Close, out fltClose);
			//if (err==1) return;

            // Saving the components
			Component = new IndicatorComp[1];

			Component[0] = new IndicatorComp();
			Component[0].CompName  = "Closing price of the bar";
			Component[0].DataType  = (IndParam.SlotType == SlotTypes.Open) ? IndComponentType.OpenPrice : IndComponentType.ClosePrice;
			Component[0].ChartType = IndChartType.NoChart;
			Component[0].FirstBar  = 2;
			Component[0].Value     = fltClose;

            return;
        }

        /// <summary>
        /// Sets the indicator logic description
        /// </summary>
        public override void SetDescription(SlotTypes slotType)
        {
            EntryPointLongDescription  = "at the end of the bar";
            EntryPointShortDescription = "at the end of the bar";
            ExitPointLongDescription   = "at the end of the bar";
            ExitPointShortDescription  = "at the end of the bar";

            return;
        }

        /// <summary>
        /// Indicator to string
        /// </summary>
        public override string ToString()
        {
            string sString = IndicatorName;

            return sString;
        }
		

		protected static int HigherPriceFilter(DataPeriods currentTF, DataPeriods higherTF, string FilterType,double[] InputPrice, out double[] FilteredPrice)
		{
			FilteredPrice = new double[Bars];
			int iFrame;
			int iRem;
			
			// Frame Calculation
			if (higherTF>currentTF)
				iFrame = (int)((int)higherTF/(int)currentTF);
			else
				iFrame = 1;
			if (higherTF==DataPeriods.week && currentTF==DataPeriods.day) iFrame=5;
			//if (iFrame>60) return 1;	
					
			// Bar Counter
            for (int iBar=1; iBar<Bars; iBar++)
			{
				int iMin  = (int) Time[iBar].Minute;
				int iHour = (int) Time[iBar].Hour;
				int iDay  = (int) Time[iBar].DayOfWeek;
				if ((int)higherTF<=60 && (int)currentTF<=30)	iRem=(int)((iMin%(int)higherTF)/(int)currentTF);
				else if (higherTF==DataPeriods.hour4 && currentTF==DataPeriods.min15) 	iRem=(int)((iHour%4)*4+(iMin/15));
				else if (higherTF==DataPeriods.hour4 && currentTF==DataPeriods.min30) 	iRem=(int)((iHour%4)*2+(iMin/30));
				else if (higherTF==DataPeriods.hour4 && currentTF==DataPeriods.hour1) 	iRem=(int)(iHour%4);
				else if (higherTF==DataPeriods.day && currentTF==DataPeriods.min30)		iRem=(int)((iHour%24)*2+(iMin/30));
				else if (higherTF==DataPeriods.day && currentTF==DataPeriods.hour1)		iRem=(int)(iHour%24);
				else if (higherTF==DataPeriods.day && currentTF==DataPeriods.hour4)		iRem=(int)((iHour%24)/4);
				else if (higherTF==DataPeriods.week && currentTF==DataPeriods.day)		iRem=(int)((iDay-1)%7);
				else	iRem = iBar % iFrame;
						
				switch (FilterType)
				{
				
					case "Price Opening":
						if (iRem == 0) FilteredPrice[iBar] = InputPrice[iBar];
						else FilteredPrice[iBar] = 0;
						break;
					case "Price Closing":
						if (iRem == iFrame-1) FilteredPrice[iBar] = InputPrice[iBar];
						else FilteredPrice[iBar] = 0;
						break;
				};
				
				if (iFrame>60) FilteredPrice[iBar]=0;
			}			
			
			return 0;
		}


    }
}