// Heiken Ashi Indicator (Modified to HTF v 4.0.0 by Denny Imanuel)
// Last changed on 2009-05-05
// Part of Forex Strategy Builder v2.8.3.7+
// Website http://forexsb.com/
// Copyright (c) 2006 - 2009 Miroslav Popov - All rights reserved.
// This code or any part of it cannot be used in other applications without a permission.

using System.Drawing;

namespace Forex_Strategy_Builder
{
    /// <summary>
    /// Heiken Ashi Indicator
    /// </summary>
    public class Heiken_Ashi_HTF : Indicator
    {
        /// <summary>
        /// Sets the default indicator parameters for the designated slot type
        /// </summary>
        public Heiken_Ashi_HTF(SlotTypes slotType)
        {
            // General properties
            IndicatorName = "Heiken Ashi (HTF v4)";
            PossibleSlots = SlotTypes.Open | SlotTypes.OpenFilter | SlotTypes.Close | SlotTypes.CloseFilter;
			CustomIndicator = true;
            
			// Setting up the indicator parameters
            IndParam = new IndicatorParam();
            IndParam.IndicatorName = IndicatorName;
            IndParam.SlotType      = slotType;
            IndParam.IndicatorType = TypeOfIndicator.Indicator;

            // The ComboBox parameters
            IndParam.ListParam[0].Caption = "Logic";
            if (slotType == SlotTypes.Open)
                IndParam.ListParam[0].ItemList = new string[]
                {
                    "Enter long at the H.A. High",
                    "Enter long at the H.A. Low",
                };
            else if (slotType == SlotTypes.OpenFilter)
                IndParam.ListParam[0].ItemList = new string[]
                {
                    "White H.A. bar without lower shadow",
                    "White H.A. bar",
                    "Black H.A. bar",
                    "Black H.A. bar without upper shadow"
                };
            else if (slotType == SlotTypes.Close)
                IndParam.ListParam[0].ItemList = new string[]
                {
                    "Exit long at the H.A. High",
                    "Exit long at the H.A. Low",
                };
            else if (slotType == SlotTypes.CloseFilter)
                IndParam.ListParam[0].ItemList = new string[]
                {
                    "Black H.A. bar without upper shadow",
                    "Black H.A. bar",
                    "White H.A. bar",
                    "White H.A. bar without lower shadow"
                };
            else
                IndParam.ListParam[0].ItemList = new string[]
                {
                    "Not Defined"
                };
            IndParam.ListParam[0].Index    = 0;
            IndParam.ListParam[0].Text     = IndParam.ListParam[0].ItemList[IndParam.ListParam[0].Index];
            IndParam.ListParam[0].Enabled  = true;
            IndParam.ListParam[0].ToolTip  = "Logic of application of the Donchian Channel.";

            IndParam.ListParam[1].Caption  = "Base price";
            IndParam.ListParam[1].ItemList = new string[] { "High, Low, Open, Close" };
            IndParam.ListParam[1].Index    = 0;
            IndParam.ListParam[1].Text     = IndParam.ListParam[1].ItemList[IndParam.ListParam[1].Index];
            IndParam.ListParam[1].Enabled  = true;
			IndParam.ListParam[1].ToolTip = "The price the Heiken Aishi is based on.";

			IndParam.ListParam[4].Caption = "Higher Time Frame Reference";
			IndParam.ListParam[4].ItemList = new string[] { "Automatic", "5 Minutes", "15 Minutes", "30 Minutes", "1 Hour", "4 Hours", "1 Day", "1 Week"};
            if (Period == DataPeriods.min1) 		IndParam.ListParam[4].Index    = 1;
            else if (Period == DataPeriods.min5) 	IndParam.ListParam[4].Index    = 2;
            else if (Period == DataPeriods.min15) 	IndParam.ListParam[4].Index    = 3;
            else if (Period == DataPeriods.min30)	IndParam.ListParam[4].Index    = 4;
            else if (Period == DataPeriods.hour1)	IndParam.ListParam[4].Index    = 5;
            else if (Period == DataPeriods.hour4)	IndParam.ListParam[4].Index    = 6;
            else if (Period == DataPeriods.day)		IndParam.ListParam[4].Index    = 7;
            IndParam.ListParam[4].Text     = IndParam.ListParam[4].ItemList[IndParam.ListParam[4].Index];
            IndParam.ListParam[4].Enabled  = true;
            IndParam.ListParam[4].ToolTip  = "Choose higher time frame as compared to current chart period for this indicator.";		

			
            // The CheckBox parameters
            IndParam.CheckParam[0].Caption = "Use previous bar value";
            IndParam.CheckParam[0].Checked = PrepareUsePrevBarValueCheckBox(slotType);
            IndParam.CheckParam[0].Enabled = true;
            IndParam.CheckParam[0].ToolTip = "Use the indicator value from the previous bar.";

            return;
        }

        /// <summary>
        /// Calculates the indicator's components
        /// </summary>
        public override void Calculate(SlotTypes slotType)
        {
            // Reading the parameters
            int iPrvs = IndParam.CheckParam[0].Checked ? 1 : 0;

			// Convert to Higher Time Frame ---------------------------------------------
			DataPeriods htfPeriod = DataPeriods.week;
			double[] hfOpen = new double[Bars];
			double[] hfClose = new double[Bars];
			double[] hfHigh = new double[Bars];
			double[] hfLow = new double[Bars];
			double[] hfVolume = new double[Bars];
			double[] hfPrice = new double[Bars];
			int[] 	 hIndex = new int[Bars];
			int      iFrame;
			int      hBars;
			
			switch (IndParam.ListParam[4].Index)
			{
				case 1: htfPeriod = DataPeriods.min5; break;
				case 2: htfPeriod = DataPeriods.min15; break;
				case 3: htfPeriod = DataPeriods.min30; break;
				case 4: htfPeriod = DataPeriods.hour1; break;
				case 5: htfPeriod = DataPeriods.hour4; break;
				case 6: htfPeriod = DataPeriods.day; break;
				case 7: htfPeriod = DataPeriods.week; break;
			}
			
			int err1 = HigherTimeFrame(Period, htfPeriod, out hIndex, out hBars, out iFrame, out hfHigh, out hfLow, out hfOpen, out hfClose, out hfVolume);
			//int err2 = HigherBasePrice(basePrice, hBars, hfHigh, hfLow, hfOpen, hfClose, out hfPrice);	
			if (err1 ==1) return;
			//------------------------------------------------------------------------
			
			
            // Calculation
            double[] adHAOpen  = new double[Bars];
            double[] adHAHigh  = new double[Bars];
            double[] adHALow   = new double[Bars];
            double[] adHAClose = new double[Bars];

            adHAOpen[0]  = hfOpen[0];
            adHAHigh[0]  = hfHigh[0];
            adHALow[0]   = hfLow[0];
            adHAClose[0] = hfClose[0];

            int iFirstBar = 1 + iPrvs;

            for (int iBar = 1; iBar < hBars; iBar++)
            {
                adHAClose[iBar] = (hfOpen[iBar] + hfHigh[iBar] + hfLow[iBar] + hfClose[iBar]) / 4;
                adHAOpen[iBar]  = (adHAOpen[iBar - 1] + adHAClose[iBar - 1]) / 2;
                adHAHigh[iBar]  = High[iBar] > adHAOpen[iBar] ? High[iBar] : adHAOpen[iBar];
                adHAHigh[iBar]  = adHAClose[iBar] > adHAHigh[iBar] ? adHAClose[iBar] : adHAHigh[iBar];
                adHALow[iBar]   = Low[iBar] < adHAOpen[iBar] ? Low[iBar] : adHAOpen[iBar];
                adHALow[iBar]   = adHAClose[iBar] < adHALow[iBar] ? adHAClose[iBar] : adHALow[iBar];
            }

			// Convert to Current Time Frame ----------------------------------------------
			iPrvs = iPrvs*iFrame;
			iFirstBar = iFirstBar*iFrame;
			if (iFirstBar>0.25*Bars) return;
			
			int err3 = CurrentTimeFrame(hIndex, hBars, ref adHAOpen);
			int err4 = CurrentTimeFrame(hIndex, hBars, ref adHAHigh);
			int err5 = CurrentTimeFrame(hIndex, hBars, ref adHALow);
			int err6 = CurrentTimeFrame(hIndex, hBars, ref adHAClose);
			int err7 = CurrentTimeFrame(hIndex, hBars, ref hfOpen);
			int err8 = CurrentTimeFrame(hIndex, hBars, ref hfClose);
			int err9 = CurrentTimeFrame(hIndex, hBars, ref hfHigh);
			int err10 = CurrentTimeFrame(hIndex, hBars, ref hfLow);				
			//-----------------------------------------------------------------------------	


            // Saving the components
            Component = new IndicatorComp[10];

            Component[0] = new IndicatorComp();
            Component[0].CompName   = "H.A. Open";
            Component[0].DataType   = IndComponentType.IndicatorValue;
            Component[0].ChartType  = IndChartType.Dot;
            Component[0].ChartColor = Color.Green;
            Component[0].FirstBar   = iFirstBar;
            Component[0].Value      = adHAOpen;

            Component[1] = new IndicatorComp();
            Component[1].CompName   = "H.A. High";
            Component[1].DataType   = IndComponentType.IndicatorValue;
            Component[1].ChartType  = IndChartType.Dot;
            Component[1].ChartColor = Color.Blue;
            Component[1].FirstBar   = iFirstBar;
            Component[1].Value      = adHAHigh;

            Component[2] = new IndicatorComp();
            Component[2].CompName   = "H.A. Low";
            Component[2].DataType   = IndComponentType.IndicatorValue;
            Component[2].ChartType  = IndChartType.Dot;
            Component[2].ChartColor = Color.Blue;
            Component[2].FirstBar   = iFirstBar;
            Component[2].Value      = adHALow;

            Component[3] = new IndicatorComp();
            Component[3].CompName   = "H.A. Close";
            Component[3].DataType   = IndComponentType.IndicatorValue;
            Component[3].ChartType  = IndChartType.Dot;
            Component[3].ChartColor = Color.Red;
            Component[3].FirstBar   = iFirstBar;
            Component[3].Value      = adHAClose;

            Component[4] = new IndicatorComp();
            Component[4].ChartType = IndChartType.NoChart;
            Component[4].FirstBar  = iFirstBar;
            Component[4].Value     = new double[Bars];

            Component[5] = new IndicatorComp();
            Component[5].ChartType = IndChartType.NoChart;
            Component[5].FirstBar  = iFirstBar;
            Component[5].Value     = new double[Bars];
			
			Component[6] = new IndicatorComp();
            Component[6].CompName   = "High";
            Component[6].DataType   = IndComponentType.IndicatorValue;
			Component[6].ChartType	= IndChartType.Level;
			Component[6].ChartColor	= Color.Blue;
			Component[6].FirstBar	= iFirstBar;
			Component[6].Value	    = hfHigh;

			Component[7] = new IndicatorComp();
		    Component[7].CompName   = "Low";
            Component[7].DataType   = IndComponentType.IndicatorValue;
			Component[7].ChartType	= IndChartType.Level;
            Component[7].ChartColor = Color.Blue;
			Component[7].FirstBar	= iFirstBar;
			Component[7].Value	    = hfLow;
		
			Component[8] = new IndicatorComp();
            Component[8].CompName   = "Open";
            Component[8].DataType   = IndComponentType.IndicatorValue;
			Component[8].ChartType	= IndChartType.CloudUp;
			Component[8].ChartColor	= Color.DarkRed;
			Component[8].FirstBar	= iFirstBar;
			Component[8].Value	    = hfOpen;

			Component[9] = new IndicatorComp();
		    Component[9].CompName   = "Close";
            Component[9].DataType   = IndComponentType.IndicatorValue;
			Component[9].ChartType	= IndChartType.CloudDown;
            Component[9].ChartColor = Color.DarkGreen;
			Component[9].FirstBar	= iFirstBar;
			Component[9].Value	    = hfClose;


            // Sets the Component's type
            if (slotType == SlotTypes.Open)
            {
                Component[4].DataType = IndComponentType.OpenLongPrice;
                Component[4].CompName = "Long position entry price";
                Component[5].DataType = IndComponentType.OpenShortPrice;
                Component[5].CompName = "Short position entry price";
            }
            else if (slotType == SlotTypes.OpenFilter)
            {
                Component[4].DataType = IndComponentType.AllowOpenLong;
                Component[4].CompName = "Is long entry allowed";
                Component[5].DataType = IndComponentType.AllowOpenShort;
                Component[5].CompName = "Is short entry allowed";
            }
            else if (slotType == SlotTypes.Close)
            {
                Component[4].DataType = IndComponentType.CloseLongPrice;
                Component[4].CompName = "Long position closing price";
                Component[5].DataType = IndComponentType.CloseShortPrice;
                Component[5].CompName = "Short position closing price";
            }
            else if (slotType == SlotTypes.CloseFilter)
            {
                Component[4].DataType = IndComponentType.ForceCloseLong;
                Component[4].CompName = "Close out long position";
                Component[5].DataType = IndComponentType.ForceCloseShort;
                Component[5].CompName = "Close out short position";
            }

            if (slotType == SlotTypes.Open || slotType == SlotTypes.Close)
            {
                for (int iBar = 2*iFrame; iBar < Bars; iBar++)
                {
                    if (IndParam.ListParam[0].Text == "Enter long at the H.A. High" ||
                        IndParam.ListParam[0].Text == "Exit long at the H.A. High")
                    {
                        Component[4].Value[iBar] = adHAHigh[iBar - iPrvs];
                        Component[5].Value[iBar] = adHALow[iBar - iPrvs];
                    }
                    else
                    {
                        Component[4].Value[iBar] = adHALow[iBar - iPrvs];
                        Component[5].Value[iBar] = adHAHigh[iBar - iPrvs];
                    }
                }
            }
            else
            {
                switch (IndParam.ListParam[0].Text)
                {
                    case "White H.A. bar without lower shadow":
                        for (int iBar = iFirstBar; iBar < Bars; iBar++)
                        {
                            Component[4].Value[iBar] = adHAClose[iBar - iPrvs] > adHAOpen[iBar - iPrvs] &&
                                adHALow[iBar - iPrvs] == adHAOpen[iBar - iPrvs] ? 1 : 0;
                            Component[5].Value[iBar] = adHAClose[iBar - iPrvs] < adHAOpen[iBar - iPrvs] &&
                                adHAHigh[iBar - iPrvs] == adHAOpen[iBar - iPrvs] ? 1 : 0;
                        }
                        break;

                    case "White H.A. bar":
                        for (int iBar = iFirstBar; iBar < Bars; iBar++)
                        {
                            Component[4].Value[iBar] = adHAClose[iBar - iPrvs] > adHAOpen[iBar - iPrvs] ? 1 : 0;
                            Component[5].Value[iBar] = adHAClose[iBar - iPrvs] < adHAOpen[iBar - iPrvs] ? 1 : 0;
                        }
                        break;

                    case "Black H.A. bar":
                        for (int iBar = iFirstBar; iBar < Bars; iBar++)
                        {
                            Component[4].Value[iBar] = adHAClose[iBar - iPrvs] < adHAOpen[iBar - iPrvs] ? 1 : 0;
                            Component[5].Value[iBar] = adHAClose[iBar - iPrvs] > adHAOpen[iBar - iPrvs] ? 1 : 0;
                        }
                        break;

                    case "Black H.A. bar without upper shadow":
                        for (int iBar = iFirstBar; iBar < Bars; iBar++)
                        {
                            Component[4].Value[iBar] = adHAClose[iBar - iPrvs] < adHAOpen[iBar - iPrvs] &&
                                adHAHigh[iBar - iPrvs] == adHAOpen[iBar - iPrvs] ? 1 : 0;
                            Component[5].Value[iBar] = adHAClose[iBar - iPrvs] > adHAOpen[iBar - iPrvs] &&
                                adHALow[iBar - iPrvs] == adHAOpen[iBar - iPrvs] ? 1 : 0;
                        }
                        break;

                    default:
                        break;
                }
            }

            return;
        }

        /// <summary>
        /// Sets the indicator logic description
        /// </summary>
        public override void SetDescription(SlotTypes slotType)
        {
            switch (IndParam.ListParam[0].Text)
            {
                case "Enter long at the H.A. High":
                    EntryPointLongDescription  = "at the " + ToString() + " High";
                    EntryPointShortDescription = "at the " + ToString() + " Low";
                    break;

                case "Enter long at the H.A. Low":
                    EntryPointLongDescription  = "at the " + ToString() + " Low";
                    EntryPointShortDescription = "at the " + ToString() + " High";
                    break;

                case "Exit long at the H.A. High":
                    ExitPointLongDescription  = "at the " + ToString() + " High";
                    ExitPointShortDescription = "at the " + ToString() + " Low";
                    break;

                case "Exit long at the H.A. Low":
                    ExitPointLongDescription  = "at the " + ToString() + " Low";
                    ExitPointShortDescription = "at the " + ToString() + " High";
                    break;

                case "White H.A. bar without lower shadow":
                    EntryFilterLongDescription  = "the " + ToString() + " bar is white and without lower shadow";
                    EntryFilterShortDescription = "the " + ToString() + " bar is black and without upper shadow";
                    ExitFilterLongDescription   = "the " + ToString() + " bar is white and without lower shadow";
                    ExitFilterShortDescription  = "the " + ToString() + " bar is black and without upper shadow";
                    break;

                case "Black H.A. bar without upper shadow":
                    EntryFilterLongDescription  = "the " + ToString() + " bar is black and without upper shadow";
                    EntryFilterShortDescription = "the " + ToString() + " bar is white and without lower shadow";
                    ExitFilterLongDescription   = "the " + ToString() + " bar is black and without upper shadow";
                    ExitFilterShortDescription  = "the " + ToString() + " bar is white and without lower shadow";
                    break;

                case "White H.A. bar":
                    EntryFilterLongDescription  = "the " + ToString() + " bar is white";
                    EntryFilterShortDescription = "the " + ToString() + " bar is black";
                    ExitFilterLongDescription   = "the " + ToString() + " bar is white";
                    ExitFilterShortDescription  = "the " + ToString() + " bar is black";
                    break;

                case "Black H.A. bar":
                    EntryFilterLongDescription  = "the " + ToString() + " bar is black";
                    EntryFilterShortDescription = "the " + ToString() + " bar is white";
                    ExitFilterLongDescription   = "the " + ToString() + " bar is black";
                    ExitFilterShortDescription  = "the " + ToString() + " bar is white";
                    break;

                default:
                    break;
            }

            return;
        }

        /// <summary>
        /// Indicator to string
        /// </summary>
        public override string ToString()
        {
            string sString = IndicatorName + (IndParam.CheckParam[0].Checked ? "*" : "");

            return sString;
        }

		/// <summary>
        /// Convert current time frame to higher time frame
        /// </summary>
		protected static int HigherTimeFrame(DataPeriods currentTF, DataPeriods higherTF, out int[] hIndex, out int hBars, out int iFrame, out double[] hfHigh, out double[] hfLow, out double[] hfOpen, out double[] hfClose, out double[] hfVolume)
		{
			hfOpen = new double[Bars];
			hfClose = new double[Bars];
			hfHigh = new double[Bars];
			hfLow = new double[Bars];
			hfVolume = new double[Bars];
			hIndex = new int[Bars];
			hBars =0;
			int iRem =0;
			
			
			// Frame Calculation
			if (higherTF>currentTF)
				iFrame = (int)((int)higherTF/(int)currentTF);
			else
				iFrame = 1;
			if (higherTF==DataPeriods.week && currentTF==DataPeriods.day) iFrame=5;
			if (iFrame>60) return 1;
			
			int hBar=0;
			for (int iBar=1; iBar<Bars; iBar++)
			{			
				
				
				// Time Synchronization
				int iMin  = (int) Time[iBar].Minute;
				int iHour = (int) Time[iBar].Hour;
				int iDay  = (int) Time[iBar].DayOfWeek;
				if ((int)higherTF<=60 && (int)currentTF<=30)	iRem=(int)((iMin%(int)higherTF)/(int)currentTF);
				else if (higherTF==DataPeriods.hour4 && currentTF==DataPeriods.min15) 	iRem=(int)((iHour%4)*4+(iMin/15));
				else if (higherTF==DataPeriods.hour4 && currentTF==DataPeriods.min30) 	iRem=(int)((iHour%4)*2+(iMin/30));
				else if (higherTF==DataPeriods.hour4 && currentTF==DataPeriods.hour1) 	iRem=(int)(iHour%4);
				else if (higherTF==DataPeriods.day && currentTF==DataPeriods.min30)		iRem=(int)((iHour%24)*2+(iMin/30));
				else if (higherTF==DataPeriods.day && currentTF==DataPeriods.hour1)		iRem=(int)(iHour%24);
				else if (higherTF==DataPeriods.day && currentTF==DataPeriods.hour4)		iRem=(int)((iHour%24)/4);
				else if (higherTF==DataPeriods.week && currentTF==DataPeriods.day)		iRem=(int)((iDay-1)%7);
				else	iRem = iBar % iFrame;
				
				// Higher Time Frame Conversion
				if (iRem == 0)
				{
					hBar++;
					hfOpen[hBar]   = Open[iBar];
					hfClose[hBar]  = Close[iBar];
					hfHigh[hBar]   = High[iBar];
					hfLow[hBar]    = Low[iBar];
					hfVolume[hBar] = Volume[iBar];
				}
				else
				{
					hfClose[hBar]  = Close[iBar];
					if (High[iBar] > hfHigh[hBar])	hfHigh[hBar] = High[iBar];
					if (Low[iBar]  < hfLow[hBar])	hfLow[hBar] = Low[iBar];
					hfVolume[hBar] += Volume[iBar];
				}
				hIndex[iBar] = hBar;
			}
			hBars = hBar+1;
			
			return 0;
		}

		/// <summary>
        /// Convert higher time frame to current time frame
        /// </summary>
		protected static int CurrentTimeFrame(int[] hIndex, int hBars, ref double[] hIndicator)
		{
			double[] hBuffer = new double[Bars];
			int hBar;
			
			for (int iBar=0; iBar<Bars; iBar++)
			{	
				hBar = hIndex[iBar];	
				hBuffer[iBar] = hIndicator[hBar];
			}
			
			for (int iBar=0; iBar<Bars; iBar++)
			{
				hIndicator[iBar] = hBuffer[iBar];
			}
			
			return 0;
		}
				
		/// <summary>
        /// Calculate base price for higher time frame data
        /// </summary>
		protected static int HigherBasePrice(BasePrice basePrice, int hBars, double[] hfHigh, double[] hfLow, double[] hfOpen, double[] hfClose, out double[] hfPrice)
		{
			hfPrice = new double[Bars];
			
			switch (basePrice)
			{
				case BasePrice.Open:	hfPrice = hfOpen;	break;
				case BasePrice.Close:	hfPrice = hfClose;	break;
				case BasePrice.High:	hfPrice = hfHigh;	break;
				case BasePrice.Low:		hfPrice = hfLow;	break;
				case BasePrice.Median:
					for (int iBar=0; iBar<hBars; iBar++)
					{
						hfPrice[iBar] = (hfLow[iBar] + hfHigh[iBar]) / 2;
					}
					break;
				case BasePrice.Typical:
					for (int iBar=0; iBar<hBars; iBar++)
					{
						hfPrice[iBar] = (hfLow[iBar] + hfHigh[iBar] + hfClose[iBar]) / 3;
					}
					break;
				case BasePrice.Weighted:
					for (int iBar=0; iBar<hBars; iBar++)
					{
						hfPrice[iBar] = (hfOpen[iBar] + hfLow[iBar] + hfHigh[iBar] + hfClose[iBar]) / 4;
					}
					break;
				default:
					break;
			}
			return 0;
		}



	}
}
