// Moving Averages Crossover Indicator (Modified to HTF v 4.0.0 by Denny Imanuel)
// Last changed on 2009-05-05
// Part of Forex Strategy Builder v2.8.3.7+
// Website http://forexsb.com/
// Copyright (c) 2006 - 2009 Miroslav Popov - All rights reserved.
// This code or any part of it cannot be used in other applications without a permission.

using System;
using System.Drawing;

namespace Forex_Strategy_Builder
{
    /// <summary>
    /// Moving Averages Crossover Indicator
    /// </summary>
    public class Moving_Averages_Crossover : Indicator
    {
        /// <summary>
        /// Sets the default indicator parameters for the designated slot type
        /// </summary>
        public Moving_Averages_Crossover(SlotTypes slotType)
        {
            // General properties
            IndicatorName = "Moving Averages Crossover (HTF v4)";
            PossibleSlots = SlotTypes.OpenFilter | SlotTypes.CloseFilter;
			CustomIndicator = true;

            // Setting up the indicator parameters
            IndParam = new IndicatorParam();
            IndParam.IndicatorName = IndicatorName;
            IndParam.SlotType      = slotType;

            // The ComboBox parameters
            IndParam.ListParam[0].Caption  = "Logic";
            IndParam.ListParam[0].ItemList = new string[]
            {
                "The Fast MA crosses the Slow MA upward",
                "The Fast MA crosses the Slow MA downward",
                "The Fast MA is higher than the Slow MA",
                "The Fast MA is lower than the Slow MA",
            };
            IndParam.ListParam[0].Index    = 0;
            IndParam.ListParam[0].Text     = IndParam.ListParam[0].ItemList[IndParam.ListParam[0].Index];
            IndParam.ListParam[0].Enabled  = true;
            IndParam.ListParam[0].ToolTip  = "Logic of application of the indicator.";

            IndParam.ListParam[1].Caption  = "Base price";
            IndParam.ListParam[1].ItemList = Enum.GetNames(typeof(BasePrice));
            IndParam.ListParam[1].Index    = (int)BasePrice.Close;
            IndParam.ListParam[1].Text     = IndParam.ListParam[1].ItemList[IndParam.ListParam[1].Index];
            IndParam.ListParam[1].Enabled  = true;
            IndParam.ListParam[1].ToolTip  = "The price both Moving Averages are based on.";

            IndParam.ListParam[3].Caption  = "MA method";
            IndParam.ListParam[3].ItemList = Enum.GetNames(typeof(MAMethod));
            IndParam.ListParam[3].Index    = (int)MAMethod.Simple;
            IndParam.ListParam[3].Text     = IndParam.ListParam[3].ItemList[IndParam.ListParam[3].Index];
            IndParam.ListParam[3].Enabled  = true;
            IndParam.ListParam[3].ToolTip  = "The method used for smoothing the Fast Moving Averages.";

			IndParam.ListParam[4].Caption = "Higher Time Frame Reference";
			IndParam.ListParam[4].ItemList = new string[] { "Automatic", "5 Minutes", "15 Minutes", "30 Minutes", "1 Hour", "4 Hours", "1 Day", "1 Week"};
            if (Period == DataPeriods.min1) 		IndParam.ListParam[4].Index    = 1;
            else if (Period == DataPeriods.min5) 	IndParam.ListParam[4].Index    = 2;
            else if (Period == DataPeriods.min15) 	IndParam.ListParam[4].Index    = 3;
            else if (Period == DataPeriods.min30)	IndParam.ListParam[4].Index    = 4;
            else if (Period == DataPeriods.hour1)	IndParam.ListParam[4].Index    = 5;
            else if (Period == DataPeriods.hour4)	IndParam.ListParam[4].Index    = 6;
            else if (Period == DataPeriods.day)		IndParam.ListParam[4].Index    = 7;
            IndParam.ListParam[4].Text     = IndParam.ListParam[4].ItemList[IndParam.ListParam[4].Index];
            IndParam.ListParam[4].Enabled  = true;
            IndParam.ListParam[4].ToolTip  = "Choose higher time frame as compared to current chart period for this indicator.";		

            // The NumericUpDown parameters
            IndParam.NumParam[0].Caption   = "Fast MA period";
            IndParam.NumParam[0].Value     = 13;
            IndParam.NumParam[0].Min       = 1;
            IndParam.NumParam[0].Max       = 200;
            IndParam.NumParam[0].Enabled   = true;
            IndParam.NumParam[0].ToolTip   = "The period of Fast MA.";

            IndParam.NumParam[1].Caption   = "Slow MA period";
            IndParam.NumParam[1].Value     = 21;
            IndParam.NumParam[1].Min       = 1;
            IndParam.NumParam[1].Max       = 200;
            IndParam.NumParam[1].Enabled   = true;
            IndParam.NumParam[1].ToolTip   = "The period of Slow MA.";

            IndParam.NumParam[2].Caption   = "Fast MA shift";
            IndParam.NumParam[2].Value     = 0;
            IndParam.NumParam[2].Min       = 0;
            IndParam.NumParam[2].Max       = 100;
            IndParam.NumParam[2].Point     = 0;
            IndParam.NumParam[2].Enabled   = true;
            IndParam.NumParam[2].ToolTip   = "The shifting value of Fast MA.";

            IndParam.NumParam[3].Caption   = "Slow MA shift";
            IndParam.NumParam[3].Value     = 0;
            IndParam.NumParam[3].Min       = 0;
            IndParam.NumParam[3].Max       = 100;
            IndParam.NumParam[3].Point     = 0;
            IndParam.NumParam[3].Enabled   = true;
            IndParam.NumParam[3].ToolTip   = "The shifting value of Slow MA.";

            // The CheckBox parameters
            IndParam.CheckParam[0].Caption = "Use previous bar value";
            IndParam.CheckParam[0].Checked = PrepareUsePrevBarValueCheckBox(slotType);
            IndParam.CheckParam[0].Enabled = true;
            IndParam.CheckParam[0].ToolTip = "Use the indicator value from the previous bar.";

            return;
        }

        /// <summary>
        /// Calculates the indicator's components
        /// </summary>
        public override void Calculate(SlotTypes slotType)
        {
            // Reading the parameters
            BasePrice basePrice    = (BasePrice)IndParam.ListParam[1].Index;
            MAMethod  fastMAMethod = (MAMethod )IndParam.ListParam[3].Index;
            MAMethod  slowMAMethod = (MAMethod )IndParam.ListParam[3].Index;
            int       iNFastMA  = (int)IndParam.NumParam[0].Value;
            int       iNSlowMA  = (int)IndParam.NumParam[1].Value;
            int       iSFastMA  = (int)IndParam.NumParam[2].Value;
            int       iSSlowMA  = (int)IndParam.NumParam[3].Value;
            int       iPrvs     = IndParam.CheckParam[0].Checked ? 1 : 0;

			// Convert to Higher Time Frame ---------------------------------------------
			DataPeriods htfPeriod = DataPeriods.week;
			double[] hfOpen = new double[Bars];
			double[] hfClose = new double[Bars];
			double[] hfHigh = new double[Bars];
			double[] hfLow = new double[Bars];
			double[] hfVolume = new double[Bars];
			double[] hfPrice = new double[Bars];
			int[] 	 hIndex = new int[Bars];
			int      iFrame;
			int      hBars;
			
			switch (IndParam.ListParam[4].Index)
			{
				case 1: htfPeriod = DataPeriods.min5; break;
				case 2: htfPeriod = DataPeriods.min15; break;
				case 3: htfPeriod = DataPeriods.min30; break;
				case 4: htfPeriod = DataPeriods.hour1; break;
				case 5: htfPeriod = DataPeriods.hour4; break;
				case 6: htfPeriod = DataPeriods.day; break;
				case 7: htfPeriod = DataPeriods.week; break;
			}
			
			int err1 = HigherTimeFrame(Period, htfPeriod, out hIndex, out hBars, out iFrame, out hfHigh, out hfLow, out hfOpen, out hfClose, out hfVolume);
			int err2 = HigherBasePrice(basePrice, hBars, hfHigh, hfLow, hfOpen, hfClose, out hfPrice);	
			if (err1 ==1) return;
			//------------------------------------------------------------------------

            int     iFirstBar = (int)Math.Max(iNFastMA + iSFastMA, iNSlowMA + iSSlowMA) + 2;
            double[] adMAFast = MovingAverage(iNFastMA, iSFastMA, fastMAMethod, hfPrice);
            double[] adMASlow = MovingAverage(iNSlowMA, iSSlowMA, slowMAMethod, hfPrice);
            double[] adMAOscillator = new double[Bars];

            for (int iBar = iFirstBar; iBar < hBars; iBar++)
                adMAOscillator[iBar] = adMAFast[iBar] - adMASlow[iBar];

			// Convert to Current Time Frame ----------------------------------------------
			iPrvs = iPrvs*iFrame;
			iFirstBar = iFirstBar*iFrame;
			if (iFirstBar>0.25*Bars) return;
			
			int err3 = CurrentTimeFrame(hIndex, hBars, ref adMAOscillator);
			int err4 = CurrentTimeFrame(hIndex, hBars, ref adMAFast);
			int err5 = CurrentTimeFrame(hIndex, hBars, ref adMASlow);		
			//-----------------------------------------------------------------------------	

			// Saving the components
            Component = new IndicatorComp[4];

            Component[0] = new IndicatorComp();
            Component[0].CompName   = "Fast Moving Average";
            Component[0].ChartColor = Color.Goldenrod;
            Component[0].DataType   = IndComponentType.IndicatorValue;
            Component[0].ChartType  = IndChartType.Line;
            Component[0].FirstBar   = iFirstBar;
            Component[0].Value      = adMAFast;

            Component[1] = new IndicatorComp();
            Component[1].CompName   = "Slow Moving Average";
            Component[1].ChartColor = Color.IndianRed;
            Component[1].DataType   = IndComponentType.IndicatorValue;
            Component[1].ChartType  = IndChartType.Line;
            Component[1].FirstBar   = iFirstBar;
            Component[1].Value      = adMASlow;

            Component[2] = new IndicatorComp();
            Component[2].ChartType = IndChartType.NoChart;
            Component[2].FirstBar  = iFirstBar;
            Component[2].Value     = new double[Bars];

            Component[3] = new IndicatorComp();
            Component[3].ChartType = IndChartType.NoChart;
            Component[3].FirstBar  = iFirstBar;
            Component[3].Value     = new double[Bars];

            // Sets the Component's type
            if (slotType == SlotTypes.OpenFilter)
            {
                Component[2].DataType = IndComponentType.AllowOpenLong;
                Component[2].CompName = "Is long entry allowed";
                Component[3].DataType = IndComponentType.AllowOpenShort;
                Component[3].CompName = "Is short entry allowed";
            }
            else if (slotType == SlotTypes.CloseFilter)
            {
                Component[2].DataType = IndComponentType.ForceCloseLong;
                Component[2].CompName = "Close out long position";
                Component[3].DataType = IndComponentType.ForceCloseShort;
                Component[3].CompName = "Close out short position";
            }

            // Calculation of the logic
            IndicatorLogic indLogic = IndicatorLogic.It_does_not_act_as_a_filter;

            switch (IndParam.ListParam[0].Text)
            {
                case "The Fast MA crosses the Slow MA upward":
                    indLogic = IndicatorLogic.The_indicator_crosses_the_level_line_upward;
                    break;
                case "The Fast MA crosses the Slow MA downward":
                    indLogic = IndicatorLogic.The_indicator_crosses_the_level_line_downward;
                    break;
                case "The Fast MA is higher than the Slow MA":
                    indLogic = IndicatorLogic.The_indicator_is_higher_than_the_level_line;
                    break;
                case "The Fast MA is lower than the Slow MA":
                    indLogic = IndicatorLogic.The_indicator_is_lower_than_the_level_line;
                    break;
                default:
                    break;
            }

            OscillatorLogic(iFirstBar, iPrvs, adMAOscillator, 0, 0, ref Component[2], ref Component[3], indLogic);

            return;
        }

        /// <summary>
        /// Sets the indicator logic description
        /// </summary>
        public override void SetDescription(SlotTypes slotType)
        {
            EntryFilterLongDescription  = ToString() + "; the Fast MA ";
            EntryFilterShortDescription = ToString() + "; the Fast MA ";
            ExitFilterLongDescription   = ToString() + "; the Fast MA ";
            ExitFilterShortDescription  = ToString() + "; the Fast MA ";

            switch (IndParam.ListParam[0].Text)
            {
                case "The Fast MA crosses the Slow MA upward":
                    EntryFilterLongDescription  += "crosses the Slow MA upward";
                    EntryFilterShortDescription += "crosses the Slow MA downward";
                    ExitFilterLongDescription   += "crosses the Slow MA upward";
                    ExitFilterShortDescription  += "crosses the Slow MA downward";
                    break;

                case "The Fast MA crosses the Slow MA downward":
                    EntryFilterLongDescription  += "crosses the Slow MA downward";
                    EntryFilterShortDescription += "crosses the Slow MA upward";
                    ExitFilterLongDescription   += "crosses the Slow MA downward";
                    ExitFilterShortDescription  += "crosses the Slow MA upward";
                    break;

                case "The Fast MA is higher than the Slow MA":
                    EntryFilterLongDescription  += "is higher than the Slow MA";
                    EntryFilterShortDescription += "is lower than the Slow MA";
                    ExitFilterLongDescription   += "is higher than the Slow MA";
                    ExitFilterShortDescription  += "is lower than the Slow MA";
                    break;

                case "The Fast MA is lower than the Slow MA":
                    EntryFilterLongDescription  += "is lower than the Slow MA";
                    EntryFilterShortDescription += "is higher than the Slow MA";
                    ExitFilterLongDescription   += "is lower than the Slow MA";
                    ExitFilterShortDescription  += "is higher than the Slow MA";
                    break;

                default:
                    break;
            }

            return;
        }

        /// <summary>
        /// Indicator to string
        /// </summary>
        public override string ToString()
        {
            string sString = IndicatorName +
                (IndParam.CheckParam[0].Checked ? "* (" : " (") +
                IndParam.ListParam[1].Text         + ", " + // Price
                IndParam.ListParam[3].Text         + ", " + // Fast MA Method
                IndParam.ListParam[4].Text         + ", " + // Slow MA Method
                IndParam.NumParam[0].ValueToString + ", " + // Fast MA period
                IndParam.NumParam[1].ValueToString + ", " + // Slow MA period
                IndParam.NumParam[2].ValueToString + ", " + // Fast MA shift
                IndParam.NumParam[3].ValueToString + ")";   // Slow MA shift

            return sString;
        }
		
				/// <summary>
        /// Convert current time frame to higher time frame
        /// </summary>
		protected static int HigherTimeFrame(DataPeriods currentTF, DataPeriods higherTF, out int[] hIndex, out int hBars, out int iFrame, out double[] hfHigh, out double[] hfLow, out double[] hfOpen, out double[] hfClose, out double[] hfVolume)
		{
			hfOpen = new double[Bars];
			hfClose = new double[Bars];
			hfHigh = new double[Bars];
			hfLow = new double[Bars];
			hfVolume = new double[Bars];
			hIndex = new int[Bars];
			hBars =0;
			int iRem =0;
			
			
			// Frame Calculation
			if (higherTF>currentTF)
				iFrame = (int)((int)higherTF/(int)currentTF);
			else
				iFrame = 1;
			if (higherTF==DataPeriods.week && currentTF==DataPeriods.day) iFrame=5;
			if (iFrame>60) return 1;
			
			int hBar=0;
			for (int iBar=1; iBar<Bars; iBar++)
			{			
				
				
				// Time Synchronization
				int iMin  = (int) Time[iBar].Minute;
				int iHour = (int) Time[iBar].Hour;
				int iDay  = (int) Time[iBar].DayOfWeek;
				if ((int)higherTF<=60 && (int)currentTF<=30)	iRem=(int)((iMin%(int)higherTF)/(int)currentTF);
				else if (higherTF==DataPeriods.hour4 && currentTF==DataPeriods.min15) 	iRem=(int)((iHour%4)*4+(iMin/15));
				else if (higherTF==DataPeriods.hour4 && currentTF==DataPeriods.min30) 	iRem=(int)((iHour%4)*2+(iMin/30));
				else if (higherTF==DataPeriods.hour4 && currentTF==DataPeriods.hour1) 	iRem=(int)(iHour%4);
				else if (higherTF==DataPeriods.day && currentTF==DataPeriods.min30)		iRem=(int)((iHour%24)*2+(iMin/30));
				else if (higherTF==DataPeriods.day && currentTF==DataPeriods.hour1)		iRem=(int)(iHour%24);
				else if (higherTF==DataPeriods.day && currentTF==DataPeriods.hour4)		iRem=(int)((iHour%24)/4);
				else if (higherTF==DataPeriods.week && currentTF==DataPeriods.day)		iRem=(int)((iDay-1)%7);
				else	iRem = iBar % iFrame;
				
				// Higher Time Frame Conversion
				if (iRem == 0)
				{
					hBar++;
					hfOpen[hBar]   = Open[iBar];
					hfClose[hBar]  = Close[iBar];
					hfHigh[hBar]   = High[iBar];
					hfLow[hBar]    = Low[iBar];
					hfVolume[hBar] = Volume[iBar];
				}
				else
				{
					hfClose[hBar]  = Close[iBar];
					if (High[iBar] > hfHigh[hBar])	hfHigh[hBar] = High[iBar];
					if (Low[iBar]  < hfLow[hBar])	hfLow[hBar] = Low[iBar];
					hfVolume[hBar] += Volume[iBar];
				}
				hIndex[iBar] = hBar;
			}
			hBars = hBar+1;
			
			return 0;
		}

		/// <summary>
        /// Convert higher time frame to current time frame
        /// </summary>
		protected static int CurrentTimeFrame(int[] hIndex, int hBars, ref double[] hIndicator)
		{
			double[] hBuffer = new double[Bars];
			int hBar;
			
			for (int iBar=0; iBar<Bars; iBar++)
			{	
				hBar = hIndex[iBar];	
				hBuffer[iBar] = hIndicator[hBar];
			}
			
			for (int iBar=0; iBar<Bars; iBar++)
			{
				hIndicator[iBar] = hBuffer[iBar];
			}
			
			return 0;
		}
				
		/// <summary>
        /// Calculate base price for higher time frame data
        /// </summary>
		protected static int HigherBasePrice(BasePrice basePrice, int hBars, double[] hfHigh, double[] hfLow, double[] hfOpen, double[] hfClose, out double[] hfPrice)
		{
			hfPrice = new double[Bars];
			
			switch (basePrice)
			{
				case BasePrice.Open:	hfPrice = hfOpen;	break;
				case BasePrice.Close:	hfPrice = hfClose;	break;
				case BasePrice.High:	hfPrice = hfHigh;	break;
				case BasePrice.Low:		hfPrice = hfLow;	break;
				case BasePrice.Median:
					for (int iBar=0; iBar<hBars; iBar++)
					{
						hfPrice[iBar] = (hfLow[iBar] + hfHigh[iBar]) / 2;
					}
					break;
				case BasePrice.Typical:
					for (int iBar=0; iBar<hBars; iBar++)
					{
						hfPrice[iBar] = (hfLow[iBar] + hfHigh[iBar] + hfClose[iBar]) / 3;
					}
					break;
				case BasePrice.Weighted:
					for (int iBar=0; iBar<hBars; iBar++)
					{
						hfPrice[iBar] = (hfOpen[iBar] + hfLow[iBar] + hfHigh[iBar] + hfClose[iBar]) / 4;
					}
					break;
				default:
					break;
			}
			return 0;
		}


    }
}
