// Parabolic SAR Indicator (Modified to HTF v 4.0.0 by Denny Imanuel)
// Last changed on 2009-05-05
// Part of Forex Strategy Builder v2.8.3.7+
// Website http://forexsb.com/
// This code or any part of it cannot be used in other applications without a permission.
// Copyright (c) 2006 - 2009 Miroslav Popov - All rights reserved.

using System;
using System.Drawing;

namespace Forex_Strategy_Builder
{
    /// <summary>
    /// Parabolic SAR Indicator
    /// </summary>
    public class Parabolic_SAR_HTF : Indicator
    {
        /// <summary>
        /// Sets the default indicator parameters for the designated slot type
        /// </summary>
        public Parabolic_SAR_HTF(SlotTypes slotType)
        {
            // General properties
            IndicatorName = "Parabolic SAR (HTF v4)";
            PossibleSlots = SlotTypes.OpenFilter | SlotTypes.Close;
			CustomIndicator = true;

            // Setting up the indicator parameters
            IndParam = new IndicatorParam();
            IndParam.IndicatorName = IndicatorName;
            IndParam.SlotType      = slotType;

            // The ComboBox parameters
            IndParam.ListParam[0].Caption  = "Logic";
            if (slotType == SlotTypes.OpenFilter)
                IndParam.ListParam[0].ItemList = new string[] 
                {
                    "The price is higher than the PSAR value" 
                };
            else if (slotType == SlotTypes.Close)
                IndParam.ListParam[0].ItemList = new string[]
                { 
                    "Exit the market at PSAR" 
                };
            else
                IndParam.ListParam[0].ItemList = new string[]
                {
                    "Not Defined"
                };
            IndParam.ListParam[0].Index    = 0;
            IndParam.ListParam[0].Text     = IndParam.ListParam[0].ItemList[IndParam.ListParam[0].Index];
            IndParam.ListParam[0].Enabled  = true;
            IndParam.ListParam[0].ToolTip  = "Logic of application of the indicator.";

			IndParam.ListParam[4].Caption = "Higher Time Frame Reference";
			IndParam.ListParam[4].ItemList = new string[] { "Automatic", "5 Minutes", "15 Minutes", "30 Minutes", "1 Hour", "4 Hours", "1 Day", "1 Week"};
            if (Period == DataPeriods.min1) 		IndParam.ListParam[4].Index    = 1;
            else if (Period == DataPeriods.min5) 	IndParam.ListParam[4].Index    = 2;
            else if (Period == DataPeriods.min15) 	IndParam.ListParam[4].Index    = 3;
            else if (Period == DataPeriods.min30)	IndParam.ListParam[4].Index    = 4;
            else if (Period == DataPeriods.hour1)	IndParam.ListParam[4].Index    = 5;
            else if (Period == DataPeriods.hour4)	IndParam.ListParam[4].Index    = 6;
            else if (Period == DataPeriods.day)		IndParam.ListParam[4].Index    = 7;
            IndParam.ListParam[4].Text     = IndParam.ListParam[4].ItemList[IndParam.ListParam[4].Index];
            IndParam.ListParam[4].Enabled  = true;
            IndParam.ListParam[4].ToolTip  = "Choose higher time frame as compared to current chart period for this indicator.";		

            // The NumericUpDown parameters
            IndParam.NumParam[0].Caption = "Starting AF";
            IndParam.NumParam[0].Value   = 0.02;
            IndParam.NumParam[0].Min     = 0.00;
            IndParam.NumParam[0].Max     = 5.00;
            IndParam.NumParam[0].Point   = 2;
            IndParam.NumParam[0].Enabled = true;
            IndParam.NumParam[0].ToolTip = "The starting value of Acceleration Factor.";

            IndParam.NumParam[1].Caption = "Increment";
            IndParam.NumParam[1].Value   = 0.02;
            IndParam.NumParam[1].Min     = 0.01;
            IndParam.NumParam[1].Max     = 5.00;
            IndParam.NumParam[1].Point   = 2;
            IndParam.NumParam[1].Enabled = true;
            IndParam.NumParam[1].ToolTip = "Increment value.";

            IndParam.NumParam[2].Caption = "Maximum AF";
            IndParam.NumParam[2].Value   = 2.00;
            IndParam.NumParam[2].Min     = 0.01;
            IndParam.NumParam[2].Max     = 9.00;
            IndParam.NumParam[2].Point   = 2;
            IndParam.NumParam[2].Enabled = true;
            IndParam.NumParam[2].ToolTip = "The maximum value of the Acceleration Factor.";

            return;
        }

        /// <summary>
        /// Calculates the indicator's components
        /// </summary>
        public override void Calculate(SlotTypes slotType)
        {
            double dAFMin = IndParam.NumParam[0].Value;
            double dAFInc = IndParam.NumParam[1].Value;
            double dAFMax = IndParam.NumParam[2].Value;

			// Convert to Higher Time Frame ---------------------------------------------
			DataPeriods htfPeriod = DataPeriods.week;
			double[] hfOpen = new double[Bars];
			double[] hfClose = new double[Bars];
			double[] hfHigh = new double[Bars];
			double[] hfLow = new double[Bars];
			double[] hfVolume = new double[Bars];
			double[] hfPrice = new double[Bars];
			int[] 	 hIndex = new int[Bars];
			int      iFrame;
			int      hBars;
			
			switch (IndParam.ListParam[4].Index)
			{
				case 1: htfPeriod = DataPeriods.min5; break;
				case 2: htfPeriod = DataPeriods.min15; break;
				case 3: htfPeriod = DataPeriods.min30; break;
				case 4: htfPeriod = DataPeriods.hour1; break;
				case 5: htfPeriod = DataPeriods.hour4; break;
				case 6: htfPeriod = DataPeriods.day; break;
				case 7: htfPeriod = DataPeriods.week; break;
			}
			
			int err1 = HigherTimeFrame(Period, htfPeriod, out hIndex, out hBars, out iFrame, out hfHigh, out hfLow, out hfOpen, out hfClose, out hfVolume);
			//int err2 = HigherBasePrice(basePrice, hBars, hfHigh, hfLow, hfOpen, hfClose, out hfPrice);	
			if (err1 ==1) return;
			//------------------------------------------------------------------------

            // Reading the parameters
            int intDirNew;
            double dAF;
            double dPExtr;
            double dPSARNew = 0;
            int[]    aiDir  = new int[Bars];
            double[] adPSAR = new double[Bars];

            //----	Calculating the initial values
            adPSAR[0] = 0;
            dAF       = dAFMin;
            intDirNew = 0;
            if (hfClose[1] > hfOpen[0])
            {
                aiDir[0]  = 1;
                aiDir[1]  = 1;
                dPExtr    = Math.Max(hfHigh[0], hfHigh[1]);
                adPSAR[1] = Math.Min(hfLow[0],  hfLow[1]);
            }
            else
            {
                aiDir[0]  = -1;
                aiDir[1]  = -1;
                dPExtr    = Math.Min(hfLow[0],  hfLow[1]);
                adPSAR[1] = Math.Max(hfHigh[0], hfHigh[1]);
            }

            for (int iBar = 2; iBar < Bars; iBar++)
            {
                //----	PSAR for the current period
                if (intDirNew != 0)
                {	
                    // The direction was changed during the last period
                    aiDir[iBar]  = intDirNew;
                    intDirNew    = 0;
                    adPSAR[iBar] = dPSARNew + dAF * (dPExtr - dPSARNew);
                }
                else
                {
                    aiDir[iBar]  = aiDir[iBar - 1];
                    adPSAR[iBar] = adPSAR[iBar - 1] + dAF * (dPExtr - adPSAR[iBar - 1]);
                }

                // PSAR has to be out of the previous two bars limits
                if (aiDir[iBar] > 0 && adPSAR[iBar] > Math.Min(hfLow[iBar - 1], hfLow[iBar - 2]))
                    adPSAR[iBar] = Math.Min(hfLow[iBar - 1], hfLow[iBar - 2]);
                else if (aiDir[iBar] < 0 && adPSAR[iBar] < Math.Max(hfHigh[iBar - 1], hfHigh[iBar - 2]))
                    adPSAR[iBar] = Math.Max(hfHigh[iBar - 1], hfHigh[iBar - 2]);

                //----	PSAR for the next period

                // Calculation of the new values of flPExtr and flAF
                // if there is a new extreme price in the PSAR direction
                if (aiDir[iBar] > 0 && hfHigh[iBar] > dPExtr)
                {
                    dPExtr = hfHigh[iBar];
                    dAF    = Math.Min(dAF + dAFInc, dAFMax);
                }

                if (aiDir[iBar] < 0 && hfLow[iBar] < dPExtr)
                {
                    dPExtr = hfLow[iBar];
                    dAF    = Math.Min(dAF + dAFInc, dAFMax);
                }

                // Wheather the price reaches PSAR
                if (hfLow[iBar] <= adPSAR[iBar] && adPSAR[iBar] <= hfHigh[iBar])
                {
                    intDirNew = -aiDir[iBar];
                    dPSARNew  = dPExtr;
                    dAF       = dAFMin;
                    if (intDirNew > 0)
                        dPExtr = hfHigh[iBar];
                    else
                        dPExtr = hfLow[iBar];
                }

            }
            int iFirstBar = 8;

			// Convert to Current Time Frame ----------------------------------------------
			//iPrvs = iPrvs*iFrame;
			iFirstBar = iFirstBar*iFrame;
			if (iFirstBar>0.25*Bars) return;
			
			int err3 = CurrentTimeFrame(hIndex, hBars, ref adPSAR);	
			int err4 = CurrentTimeFrame(hIndex, hBars, ref hfOpen);
			int err5 = CurrentTimeFrame(hIndex, hBars, ref hfClose);
			int err6 = CurrentTimeFrame(hIndex, hBars, ref hfHigh);
			int err7 = CurrentTimeFrame(hIndex, hBars, ref hfLow);	
			//-----------------------------------------------------------------------------	

            // Saving the components
            Component = new IndicatorComp[5];

            Component[0] = new IndicatorComp();
            Component[0].CompName = "PSAR value";
            if (slotType == SlotTypes.Close)
                Component[0].DataType = IndComponentType.ClosePrice;
            else
                Component[0].DataType = IndComponentType.IndicatorValue;
            Component[0].ChartType  = IndChartType.Dot;
            Component[0].ChartColor = Color.Violet;
            Component[0].FirstBar   = iFirstBar;
            Component[0].PosPriceDependence = PositionPriceDependence.BuyHigherSellLower;
            Component[0].Value      = adPSAR;
			
			Component[1] = new IndicatorComp();
            Component[1].CompName   = "High";
            Component[1].DataType   = IndComponentType.IndicatorValue;
			Component[1].ChartType	= IndChartType.Level;
			Component[1].ChartColor	= Color.Blue;
			Component[1].FirstBar	= iFirstBar;
			Component[1].Value	    = hfHigh;

			Component[2] = new IndicatorComp();
		    Component[2].CompName   = "Low";
            Component[2].DataType   = IndComponentType.IndicatorValue;
			Component[2].ChartType	= IndChartType.Level;
            Component[2].ChartColor = Color.Blue;
			Component[2].FirstBar	= iFirstBar;
			Component[2].Value	    = hfLow;
		
			Component[3] = new IndicatorComp();
            Component[3].CompName   = "Open";
            Component[3].DataType   = IndComponentType.IndicatorValue;
			Component[3].ChartType	= IndChartType.CloudUp;
			Component[3].ChartColor	= Color.DarkRed;
			Component[3].FirstBar	= iFirstBar;
			Component[3].Value	    = hfOpen;

			Component[4] = new IndicatorComp();
		    Component[4].CompName   = "Close";
            Component[4].DataType   = IndComponentType.IndicatorValue;
			Component[4].ChartType	= IndChartType.CloudDown;
            Component[4].ChartColor = Color.DarkGreen;
			Component[4].FirstBar	= iFirstBar;
			Component[4].Value	    = hfClose;

            return;
        }

        /// <summary>
        /// Sets the indicator logic description
        /// </summary>
        public override void SetDescription(SlotTypes slotType)
        {
            EntryFilterLongDescription  = "the price is higher than the " + ToString();
            EntryFilterShortDescription = "the price is lower than the " + ToString();
            ExitPointLongDescription    = "at the " + ToString() + ". It determines the position direction also";
            ExitPointShortDescription   = "at the " + ToString() + ". It determines the position direction also";

            return;
        }

        /// <summary>
        /// Indicator to string
        /// </summary>
        public override string ToString()
        {
            string sString = IndicatorName + " (" +
                IndParam.NumParam[0].ValueToString + ", " + // Starting AF
                IndParam.NumParam[1].ValueToString + ", " + // Increment
                IndParam.NumParam[2].ValueToString + ")";   // Max AF

            return sString;
        }
		
		/// <summary>
        /// Convert current time frame to higher time frame
        /// </summary>
		protected static int HigherTimeFrame(DataPeriods currentTF, DataPeriods higherTF, out int[] hIndex, out int hBars, out int iFrame, out double[] hfHigh, out double[] hfLow, out double[] hfOpen, out double[] hfClose, out double[] hfVolume)
		{
			hfOpen = new double[Bars];
			hfClose = new double[Bars];
			hfHigh = new double[Bars];
			hfLow = new double[Bars];
			hfVolume = new double[Bars];
			hIndex = new int[Bars];
			hBars =0;
			int iRem =0;
			
			
			// Frame Calculation
			if (higherTF>currentTF)
				iFrame = (int)((int)higherTF/(int)currentTF);
			else
				iFrame = 1;
			if (higherTF==DataPeriods.week && currentTF==DataPeriods.day) iFrame=5;
			if (iFrame>60) return 1;
			
			int hBar=0;
			for (int iBar=1; iBar<Bars; iBar++)
			{			
				
				
				// Time Synchronization
				int iMin  = (int) Time[iBar].Minute;
				int iHour = (int) Time[iBar].Hour;
				int iDay  = (int) Time[iBar].DayOfWeek;
				if ((int)higherTF<=60 && (int)currentTF<=30)	iRem=(int)((iMin%(int)higherTF)/(int)currentTF);
				else if (higherTF==DataPeriods.hour4 && currentTF==DataPeriods.min15) 	iRem=(int)((iHour%4)*4+(iMin/15));
				else if (higherTF==DataPeriods.hour4 && currentTF==DataPeriods.min30) 	iRem=(int)((iHour%4)*2+(iMin/30));
				else if (higherTF==DataPeriods.hour4 && currentTF==DataPeriods.hour1) 	iRem=(int)(iHour%4);
				else if (higherTF==DataPeriods.day && currentTF==DataPeriods.min30)		iRem=(int)((iHour%24)*2+(iMin/30));
				else if (higherTF==DataPeriods.day && currentTF==DataPeriods.hour1)		iRem=(int)(iHour%24);
				else if (higherTF==DataPeriods.day && currentTF==DataPeriods.hour4)		iRem=(int)((iHour%24)/4);
				else if (higherTF==DataPeriods.week && currentTF==DataPeriods.day)		iRem=(int)((iDay-1)%7);
				else	iRem = iBar % iFrame;
				
				// Higher Time Frame Conversion
				if (iRem == 0)
				{
					hBar++;
					hfOpen[hBar]   = Open[iBar];
					hfClose[hBar]  = Close[iBar];
					hfHigh[hBar]   = High[iBar];
					hfLow[hBar]    = Low[iBar];
					hfVolume[hBar] = Volume[iBar];
				}
				else
				{
					hfClose[hBar]  = Close[iBar];
					if (High[iBar] > hfHigh[hBar])	hfHigh[hBar] = High[iBar];
					if (Low[iBar]  < hfLow[hBar])	hfLow[hBar] = Low[iBar];
					hfVolume[hBar] += Volume[iBar];
				}
				hIndex[iBar] = hBar;
			}
			hBars = hBar+1;
			
			return 0;
		}

		/// <summary>
        /// Convert higher time frame to current time frame
        /// </summary>
		protected static int CurrentTimeFrame(int[] hIndex, int hBars, ref double[] hIndicator)
		{
			double[] hBuffer = new double[Bars];
			int hBar;
			
			for (int iBar=0; iBar<Bars; iBar++)
			{	
				hBar = hIndex[iBar];	
				hBuffer[iBar] = hIndicator[hBar];
			}
			
			for (int iBar=0; iBar<Bars; iBar++)
			{
				hIndicator[iBar] = hBuffer[iBar];
			}
			
			return 0;
		}
				
		/// <summary>
        /// Calculate base price for higher time frame data
        /// </summary>
		protected static int HigherBasePrice(BasePrice basePrice, int hBars, double[] hfHigh, double[] hfLow, double[] hfOpen, double[] hfClose, out double[] hfPrice)
		{
			hfPrice = new double[Bars];
			
			switch (basePrice)
			{
				case BasePrice.Open:	hfPrice = hfOpen;	break;
				case BasePrice.Close:	hfPrice = hfClose;	break;
				case BasePrice.High:	hfPrice = hfHigh;	break;
				case BasePrice.Low:		hfPrice = hfLow;	break;
				case BasePrice.Median:
					for (int iBar=0; iBar<hBars; iBar++)
					{
						hfPrice[iBar] = (hfLow[iBar] + hfHigh[iBar]) / 2;
					}
					break;
				case BasePrice.Typical:
					for (int iBar=0; iBar<hBars; iBar++)
					{
						hfPrice[iBar] = (hfLow[iBar] + hfHigh[iBar] + hfClose[iBar]) / 3;
					}
					break;
				case BasePrice.Weighted:
					for (int iBar=0; iBar<hBars; iBar++)
					{
						hfPrice[iBar] = (hfOpen[iBar] + hfLow[iBar] + hfHigh[iBar] + hfClose[iBar]) / 4;
					}
					break;
				default:
					break;
			}
			return 0;
		}


    }
}