// Ross Hook Indicator (Modified to HTF v 4.0.0 by Denny Imanuel)
// Last changed on 2009-05-15
// Part of Forex Strategy Builder v2.9.0.0+
// Website http://forexsb.com/
// This code or any part of it cannot be used in other applications without a permission.
// Copyright (c) 2006 - 2009 Miroslav Popov - All rights reserved.

using System.Drawing;

namespace Forex_Strategy_Builder
{
    /// <summary>
    /// Ross Hook Indicator
    /// </summary>
    public class Ross_Hook_HTF : Indicator
	{
        /// <summary>
        /// Constructor
        /// </summary>
        public Ross_Hook_HTF(SlotTypes slotType)
        {
            // General properties
            IndicatorName = "Ross Hook (HTF v4)";
            PossibleSlots = SlotTypes.Open | SlotTypes.Close;
			CustomIndicator = true;

            // Setting up the indicator parameters
            IndParam = new IndicatorParam();
            IndParam.IndicatorName = IndicatorName;
            IndParam.SlotType      = slotType;

            // The ComboBox parameters
            IndParam.ListParam[0].Caption  = "Logic";
            if (slotType == SlotTypes.Open)
                IndParam.ListParam[0].ItemList = new string[]
                {
                    "Enter long at an Up Ross hook",
                    "Enter long at a Down Ross hook"
                };
            else if (slotType == SlotTypes.Close)
                IndParam.ListParam[0].ItemList = new string[]
                {
                    "Exit long at an Up Ross hook",
                    "Exit long at a Down Ross hook"
                };
            else
                IndParam.ListParam[0].ItemList = new string[]
                {
                    "Not Defined"
                };
            IndParam.ListParam[0].Index = 0;
            IndParam.ListParam[0].Text    = IndParam.ListParam[0].ItemList[IndParam.ListParam[0].Index];
            IndParam.ListParam[0].Enabled = true;
            IndParam.ListParam[0].ToolTip = "Logic of application of the indicator.";

			IndParam.ListParam[4].Caption = "Higher Time Frame Reference";
			IndParam.ListParam[4].ItemList = new string[] { "Automatic", "5 Minutes", "15 Minutes", "30 Minutes", "1 Hour", "4 Hours", "1 Day", "1 Week"};
            if (Period == DataPeriods.min1) 		IndParam.ListParam[4].Index    = 1;
            else if (Period == DataPeriods.min5) 	IndParam.ListParam[4].Index    = 2;
            else if (Period == DataPeriods.min15) 	IndParam.ListParam[4].Index    = 3;
            else if (Period == DataPeriods.min30)	IndParam.ListParam[4].Index    = 4;
            else if (Period == DataPeriods.hour1)	IndParam.ListParam[4].Index    = 5;
            else if (Period == DataPeriods.hour4)	IndParam.ListParam[4].Index    = 6;
            else if (Period == DataPeriods.day)		IndParam.ListParam[4].Index    = 7;
            IndParam.ListParam[4].Text     = IndParam.ListParam[4].ItemList[IndParam.ListParam[4].Index];
            IndParam.ListParam[4].Enabled  = true;
            IndParam.ListParam[4].ToolTip  = "Choose higher time frame as compared to current chart period for this indicator.";		

            return;
        }

        /// <summary>
        /// Calculates the indicator's components
        /// </summary>
        public override void Calculate(SlotTypes slotType)
        {
            double[] adRhUp = new double[Bars];
			double[] adRhDn = new double[Bars];

			// Convert to Higher Time Frame ---------------------------------------------
			DataPeriods htfPeriod = DataPeriods.week;
			double[] hfOpen = new double[Bars];
			double[] hfClose = new double[Bars];
			double[] hfHigh = new double[Bars];
			double[] hfLow = new double[Bars];
			double[] hfVolume = new double[Bars];
			double[] hfPrice = new double[Bars];
			int[] 	 hIndex = new int[Bars];
			int      iFrame;
			int      hBars;
			
			switch (IndParam.ListParam[4].Index)
			{
				case 1: htfPeriod = DataPeriods.min5; break;
				case 2: htfPeriod = DataPeriods.min15; break;
				case 3: htfPeriod = DataPeriods.min30; break;
				case 4: htfPeriod = DataPeriods.hour1; break;
				case 5: htfPeriod = DataPeriods.hour4; break;
				case 6: htfPeriod = DataPeriods.day; break;
				case 7: htfPeriod = DataPeriods.week; break;
			}
			
			int err1 = HigherTimeFrame(Period, htfPeriod, out hIndex, out hBars, out iFrame, out hfHigh, out hfLow, out hfOpen, out hfClose, out hfVolume);
			//int err2 = HigherBasePrice(basePrice, hBars, hfHigh, hfLow, hfOpen, hfClose, out hfPrice);	
			if (err1 ==1) return;
			//------------------------------------------------------------------------

            for (int iBar = 5; iBar < hBars - 1; iBar++)
            {
                if (hfHigh[iBar] < hfHigh[iBar - 1])
                {
                    if (hfHigh[iBar - 3] < hfHigh[iBar - 1] && hfHigh[iBar - 2] < hfHigh[iBar - 1])
                        adRhUp[iBar + 1] = hfHigh[iBar - 1];
                }

                if (hfLow[iBar] > hfLow[iBar - 1])
                {
                    if (hfLow[iBar - 3] > hfLow[iBar - 1] && hfLow[iBar - 2] > hfLow[iBar - 1])
                        adRhDn[iBar + 1] = hfLow[iBar - 1];
                }
            }

            // Is visible
            for (int iBar = 5; iBar < hBars; iBar++)
            {
                if (adRhUp[iBar - 1] > 0 && adRhUp[iBar] == 0 && hfHigh[iBar - 1] < adRhUp[iBar - 1])
                    adRhUp[iBar] = adRhUp[iBar - 1];
                if (adRhDn[iBar - 1] > 0 && adRhDn[iBar] == 0 && hfLow[iBar - 1] > adRhDn[iBar - 1])
                    adRhDn[iBar] = adRhDn[iBar - 1];
            }

			// Convert to Current Time Frame ----------------------------------------------
			//iPrvs = iPrvs*iFrame;
			int iFirstBar = 5*iFrame;
			if (iFirstBar>0.25*Bars) return;
			
			int err3 = CurrentTimeFrame(hIndex, hBars, ref adRhUp);
			int err4 = CurrentTimeFrame(hIndex, hBars, ref adRhDn);		
			//-----------------------------------------------------------------------------	

            // Saving the components
            Component = new IndicatorComp[2];

			Component[0]			= new IndicatorComp();
            Component[0].ChartType  = IndChartType.Level;
			Component[0].ChartColor	= Color.SpringGreen;
			Component[0].FirstBar	= iFirstBar;
			Component[0].Value	    = adRhUp;

			Component[1]			= new IndicatorComp();
            Component[1].ChartType  = IndChartType.Level;
            Component[1].ChartColor = Color.DarkRed;
			Component[1].FirstBar	= iFirstBar;
			Component[1].Value	    = adRhDn;

            // Sets the Component's type
            if (slotType == SlotTypes.Open)
            {
                if (IndParam.ListParam[0].Text == "Enter long at an Up Ross hook")
                {
                    Component[0].DataType = IndComponentType.OpenLongPrice;
                    Component[1].DataType = IndComponentType.OpenShortPrice;
                }
                else
                {
                    Component[0].DataType = IndComponentType.OpenShortPrice;
                    Component[1].DataType = IndComponentType.OpenLongPrice;
                }
                Component[0].CompName = "Up Ross hook";
                Component[1].CompName = "Down Ross hook";
            }
            else if (slotType == SlotTypes.Close)
            {
                if (IndParam.ListParam[0].Text == "Exit long at an Up Ross hook")
                {
                    Component[0].DataType = IndComponentType.CloseLongPrice;
                    Component[1].DataType = IndComponentType.CloseShortPrice;
                }
                else
                {
                    Component[0].DataType = IndComponentType.CloseShortPrice;
                    Component[1].DataType = IndComponentType.CloseLongPrice;
                }
                Component[0].CompName = "Up Ross hook";
                Component[1].CompName = "Down Ross hook";
            }

            return;
		}

        /// <summary>
        /// Sets the indicator logic description
        /// </summary>
        public override void SetDescription(SlotTypes slotType)
        {
            switch (IndParam.ListParam[0].Text)
            {
                case "Enter long at an Up Ross hook":
                    EntryPointLongDescription  = "at the peak of an Up Ross hook";
                    EntryPointShortDescription = "at the bottom of a Down  Ross hook";
                    break;

                case "Enter long at a Down Ross hook":
                    EntryPointLongDescription  = "at the bottom of a Down Ross hook";
                    EntryPointShortDescription = "at the peak of an Up Ross hook";
                    break;

                case "Exit long at an Up Ross hook":
                    ExitPointLongDescription  = "at the peak of an Up Ross hook";
                    ExitPointShortDescription = "at the bottom of a Down Ross hook";
                    break;

                case "Exit long at a Down Ross hook":
                    ExitPointLongDescription  = "at the bottom of a Down Ross hook";
                    ExitPointShortDescription = "at the peak of an Up Fractal";
                    break;

                default:
                    break;
            }

            return;
        }

        /// <summary>
        /// Indicator to string
        /// </summary>
        public override string ToString()
        {
            string sString = IndicatorName;

            return sString;
        }
		
		/// <summary>
        /// Convert current time frame to higher time frame
        /// </summary>
		protected static int HigherTimeFrame(DataPeriods currentTF, DataPeriods higherTF, out int[] hIndex, out int hBars, out int iFrame, out double[] hfHigh, out double[] hfLow, out double[] hfOpen, out double[] hfClose, out double[] hfVolume)
		{
			hfOpen = new double[Bars];
			hfClose = new double[Bars];
			hfHigh = new double[Bars];
			hfLow = new double[Bars];
			hfVolume = new double[Bars];
			hIndex = new int[Bars];
			hBars =0;
			int iRem =0;
			
			
			// Frame Calculation
			if (higherTF>currentTF)
				iFrame = (int)((int)higherTF/(int)currentTF);
			else
				iFrame = 1;
			if (higherTF==DataPeriods.week && currentTF==DataPeriods.day) iFrame=5;
			if (iFrame>60) return 1;
			
			int hBar=0;
			for (int iBar=1; iBar<Bars; iBar++)
			{			
				
				
				// Time Synchronization
				int iMin  = (int) Time[iBar].Minute;
				int iHour = (int) Time[iBar].Hour;
				int iDay  = (int) Time[iBar].DayOfWeek;
				if ((int)higherTF<=60 && (int)currentTF<=30)	iRem=(int)((iMin%(int)higherTF)/(int)currentTF);
				else if (higherTF==DataPeriods.hour4 && currentTF==DataPeriods.min15) 	iRem=(int)((iHour%4)*4+(iMin/15));
				else if (higherTF==DataPeriods.hour4 && currentTF==DataPeriods.min30) 	iRem=(int)((iHour%4)*2+(iMin/30));
				else if (higherTF==DataPeriods.hour4 && currentTF==DataPeriods.hour1) 	iRem=(int)(iHour%4);
				else if (higherTF==DataPeriods.day && currentTF==DataPeriods.min30)		iRem=(int)((iHour%24)*2+(iMin/30));
				else if (higherTF==DataPeriods.day && currentTF==DataPeriods.hour1)		iRem=(int)(iHour%24);
				else if (higherTF==DataPeriods.day && currentTF==DataPeriods.hour4)		iRem=(int)((iHour%24)/4);
				else if (higherTF==DataPeriods.week && currentTF==DataPeriods.day)		iRem=(int)((iDay-1)%7);
				else	iRem = iBar % iFrame;
				
				// Higher Time Frame Conversion
				if (iRem == 0)
				{
					hBar++;
					hfOpen[hBar]   = Open[iBar];
					hfClose[hBar]  = Close[iBar];
					hfHigh[hBar]   = High[iBar];
					hfLow[hBar]    = Low[iBar];
					hfVolume[hBar] = Volume[iBar];
				}
				else
				{
					hfClose[hBar]  = Close[iBar];
					if (High[iBar] > hfHigh[hBar])	hfHigh[hBar] = High[iBar];
					if (Low[iBar]  < hfLow[hBar])	hfLow[hBar] = Low[iBar];
					hfVolume[hBar] += Volume[iBar];
				}
				hIndex[iBar] = hBar;
			}
			hBars = hBar+1;
			
			return 0;
		}

		/// <summary>
        /// Convert higher time frame to current time frame
        /// </summary>
		protected static int CurrentTimeFrame(int[] hIndex, int hBars, ref double[] hIndicator)
		{
			double[] hBuffer = new double[Bars];
			int hBar;
			
			for (int iBar=0; iBar<Bars; iBar++)
			{	
				hBar = hIndex[iBar];	
				hBuffer[iBar] = hIndicator[hBar];
			}
			
			for (int iBar=0; iBar<Bars; iBar++)
			{
				hIndicator[iBar] = hBuffer[iBar];
			}
			
			return 0;
		}
				
		/// <summary>
        /// Calculate base price for higher time frame data
        /// </summary>
		protected static int HigherBasePrice(BasePrice basePrice, int hBars, double[] hfHigh, double[] hfLow, double[] hfOpen, double[] hfClose, out double[] hfPrice)
		{
			hfPrice = new double[Bars];
			
			switch (basePrice)
			{
				case BasePrice.Open:	hfPrice = hfOpen;	break;
				case BasePrice.Close:	hfPrice = hfClose;	break;
				case BasePrice.High:	hfPrice = hfHigh;	break;
				case BasePrice.Low:		hfPrice = hfLow;	break;
				case BasePrice.Median:
					for (int iBar=0; iBar<hBars; iBar++)
					{
						hfPrice[iBar] = (hfLow[iBar] + hfHigh[iBar]) / 2;
					}
					break;
				case BasePrice.Typical:
					for (int iBar=0; iBar<hBars; iBar++)
					{
						hfPrice[iBar] = (hfLow[iBar] + hfHigh[iBar] + hfClose[iBar]) / 3;
					}
					break;
				case BasePrice.Weighted:
					for (int iBar=0; iBar<hBars; iBar++)
					{
						hfPrice[iBar] = (hfOpen[iBar] + hfLow[iBar] + hfHigh[iBar] + hfClose[iBar]) / 4;
					}
					break;
				default:
					break;
			}
			return 0;
		}


    }
}