// HTF Strategy Template (Modified to HTF v 4.0.0 by Denny Imanuel)
// Part of Forex Strategy Builder v2.8.3.7+
// Website http://forexsb.com/
// Copyright (c) 2010 Denny Imanuel - All rights reserved.
// This code or any part of it cannot be used in other applications without a permission.

using System;
using System.Drawing;

namespace Forex_Strategy_Builder
{
    /// <summary>
    /// System_Template Indicator
    /// </summary>
    public class Strategy_Template_HTF : Indicator
	{
		/// <summary>
        /// Sets the default indicator parameters for the designated slot type
        /// </summary>
		public Strategy_Template_HTF(SlotTypes slotType)
        {
            // General properties
            IndicatorName = "Strategy Template (HTF v4)";
            PossibleSlots = SlotTypes.Open | SlotTypes.Close;
			CustomIndicator = true;
			IsDescreteValues = true;

            // Setting up the indicator parameters
            IndParam = new IndicatorParam();
            IndParam.IndicatorName = IndicatorName;
            IndParam.SlotType      = slotType;

            // The ComboBox parameters
            IndParam.ListParam[0].Caption = "Logic";
            if (slotType == SlotTypes.Open)
            {
                IndParam.ListParam[0].ItemList = new string[]
                {
                    "Enter strategy using HTF Visualization",
                };
            }
            else if (slotType == SlotTypes.Close)
            {
                IndParam.ListParam[0].ItemList = new string[]
                {
                    "Exit strategy using HTF Visualization",
                };
            }
            else
                IndParam.ListParam[0].ItemList = new string[]
                {
                    "Not Defined"
                };
            IndParam.ListParam[0].Index = 0;
            IndParam.ListParam[0].Text     = IndParam.ListParam[0].ItemList[IndParam.ListParam[0].Index];
            IndParam.ListParam[0].Enabled  = true;
            IndParam.ListParam[0].ToolTip  = "Logic of application of the indicator.";

            IndParam.ListParam[1].Caption  = "Smoothing method";
            IndParam.ListParam[1].ItemList = Enum.GetNames(typeof(MAMethod));
            IndParam.ListParam[1].Index    = (int)MAMethod.Simple;
            IndParam.ListParam[1].Text     = IndParam.ListParam[1].ItemList[IndParam.ListParam[1].Index];
            IndParam.ListParam[1].Enabled  = true;
            IndParam.ListParam[1].ToolTip  = "The smoothing method of Moving Average.";

            IndParam.ListParam[2].Caption  = "Base price";
            IndParam.ListParam[2].ItemList = Enum.GetNames(typeof(BasePrice));
            IndParam.ListParam[2].Index    = (int)BasePrice.Close;
            IndParam.ListParam[2].Text     = IndParam.ListParam[2].ItemList[IndParam.ListParam[2].Index];
            IndParam.ListParam[2].Enabled  = true;
            IndParam.ListParam[2].ToolTip  = "The price the Moving Average is based on.";			
/*Start Copy1*/			
			IndParam.ListParam[4].Caption = "Higher Time Frame Reference";
			IndParam.ListParam[4].ItemList = new string[] { "Automatic", "5 Minutes", "15 Minutes", "30 Minutes", "1 Hour", "4 Hours", "1 Day", "1 Week"};
            if (Period == DataPeriods.min1) 		IndParam.ListParam[4].Index    = 1;
            else if (Period == DataPeriods.min5) 	IndParam.ListParam[4].Index    = 2;
            else if (Period == DataPeriods.min15) 	IndParam.ListParam[4].Index    = 3;
            else if (Period == DataPeriods.min30)	IndParam.ListParam[4].Index    = 4;
            else if (Period == DataPeriods.hour1)	IndParam.ListParam[4].Index    = 5;
            else if (Period == DataPeriods.hour4)	IndParam.ListParam[4].Index    = 6;
            else if (Period == DataPeriods.day)		IndParam.ListParam[4].Index    = 7;
            IndParam.ListParam[4].Text     = IndParam.ListParam[4].ItemList[IndParam.ListParam[4].Index];
            IndParam.ListParam[4].Enabled  = true;
            IndParam.ListParam[4].ToolTip  = "Choose higher time frame as compared to current chart period for this indicator.";		
/*End Copy1*/		
            IndParam.NumParam[0].Caption = "Price Movement Margin";
            IndParam.NumParam[0].Value   = 10;
            IndParam.NumParam[0].Min     = 0;
            IndParam.NumParam[0].Max     = +1000;
            IndParam.NumParam[0].Enabled = true;
            IndParam.NumParam[0].ToolTip = "Price movement margin to ensure the price direction.";
		
			// The CheckBox parameters
            IndParam.CheckParam[0].Caption = "Use previous bar value.";
            IndParam.CheckParam[0].Checked = PrepareUsePrevBarValueCheckBox(slotType);
            IndParam.CheckParam[0].Enabled = true;
            IndParam.CheckParam[0].ToolTip = "Use the indicator value from the previous bar.";
	
            return;
        }

        /// <summary>
        /// Calculates the indicator's components
        /// </summary>
        public override void Calculate(SlotTypes slotType)
        {
            // Reading the parameters
			MAMethod  maMethod 	= (MAMethod )IndParam.ListParam[1].Index;
			BasePrice basePrice = (BasePrice)IndParam.ListParam[2].Index;
            double dShift     = (double)IndParam.NumParam[0].Value * Point;
			int iPrvs  = IndParam.CheckParam[0].Checked ? 1 : 0;

/*Start Copy2*/ 
			// Convert to Higher Time Frame ---------------------------------------------
			DataPeriods htfPeriod = DataPeriods.week;
			double[] hfOpen = new double[Bars];
			double[] hfClose = new double[Bars];
			double[] hfHigh = new double[Bars];
			double[] hfLow = new double[Bars];
			double[] hfVolume = new double[Bars];
			double[] hfPrice = new double[Bars];
			int[] 	 hIndex = new int[Bars];
			int      iFrame;
			int      hBars;
			
			switch (IndParam.ListParam[4].Index)
			{
				case 1: htfPeriod = DataPeriods.min5; break;
				case 2: htfPeriod = DataPeriods.min15; break;
				case 3: htfPeriod = DataPeriods.min30; break;
				case 4: htfPeriod = DataPeriods.hour1; break;
				case 5: htfPeriod = DataPeriods.hour4; break;
				case 6: htfPeriod = DataPeriods.day; break;
				case 7: htfPeriod = DataPeriods.week; break;
			}
			
			int err1 = HigherTimeFrame(Period, htfPeriod, out hIndex, out hBars, out iFrame, out hfHigh, out hfLow, out hfOpen, out hfClose, out hfVolume);
			int err2 = HigherBasePrice(basePrice, hBars, hfHigh, hfLow, hfOpen, hfClose, out hfPrice);	
			if (err1==1) return;
			//------------------------------------------------------------------------
/*End Copy2*/

			double[] adLELevel = new double[Bars];
			double[] adSELevel = new double[Bars];
			double[] adLXLevel = new double[Bars];
			double[] adSXLevel = new double[Bars];
			
			int iFirstBar = 0;
			// Calculation of the LE Level, SE Level, LX Level, SX Level
			for (int iBar=iFirstBar; iBar<hBars; iBar++)
			{
				////////////////////////////////////////////////////////////////////////////////////////
				//
				//	Add your indicator calculation here
				//	
				//	Current Time Frame use: 
				//		Open[iBar], Close[iBar], High[iBar], Low[iBar], Volume[iBar]
				//  Higher Time Frame use:
				//		hfOpen[iBar], hfClose[iBar], hfHigh[iBar], hfLow[iBar], hfVolume[iBar] 	
				//
				///////////////////////////////////////////////////////////////////////////////////////
				
				adLELevel[iBar] = 0;
				adSELevel[iBar] = 0;
				adLXLevel[iBar] = 0;
				adSXLevel[iBar] = 0;
			}

/*Start Copy3*/ 
			// Convert to Current Time Frame ----------------------------------------------
			iPrvs = iPrvs*iFrame;
			iFirstBar = iFirstBar*iFrame;
			if (iFirstBar>0.25*Bars) return;
			
			int err3 = CurrentTimeFrame(hIndex, hBars, ref hfOpen);
			int err4 = CurrentTimeFrame(hIndex, hBars, ref hfClose);
			int err5 = CurrentTimeFrame(hIndex, hBars, ref hfHigh);
			int err6 = CurrentTimeFrame(hIndex, hBars, ref hfLow);			
			//-----------------------------------------------------------------------------	
/*End Copy3*/ 

            // Saving the components
            Component = new IndicatorComp[6];
			
			Component[0] = new IndicatorComp();
            Component[0].CompName   = "High";
            Component[0].DataType   = IndComponentType.IndicatorValue;
			Component[0].ChartType	= IndChartType.Level;
			Component[0].ChartColor	= Color.Blue;
			Component[0].FirstBar	= iFirstBar;
			Component[0].Value	    = hfHigh;

			Component[1] = new IndicatorComp();
		    Component[1].CompName   = "Low";
            Component[1].DataType   = IndComponentType.IndicatorValue;
			Component[1].ChartType	= IndChartType.Level;
            Component[1].ChartColor = Color.Blue;
			Component[1].FirstBar	= iFirstBar;
			Component[1].Value	    = hfLow;
		
			Component[2] = new IndicatorComp();
            Component[2].CompName   = "Open";
            Component[2].DataType   = IndComponentType.IndicatorValue;
			Component[2].ChartType	= IndChartType.CloudUp;
			Component[2].ChartColor	= Color.DarkRed;
			Component[2].FirstBar	= iFirstBar;
			Component[2].Value	    = hfOpen;

			Component[3] = new IndicatorComp();
		    Component[3].CompName   = "Close";
            Component[3].DataType   = IndComponentType.IndicatorValue;
			Component[3].ChartType	= IndChartType.CloudDown;
            Component[3].ChartColor = Color.DarkGreen;
			Component[3].FirstBar	= iFirstBar;
			Component[3].Value	    = hfClose;

            Component[4] = new IndicatorComp();
            Component[4].ChartType = IndChartType.NoChart;
            Component[4].FirstBar  = iFirstBar;
            Component[4].Value     = new double[Bars];

            Component[5] = new IndicatorComp();
            Component[5].ChartType = IndChartType.NoChart;
            Component[5].FirstBar  = iFirstBar;
            Component[5].Value     = new double[Bars];
			
		    if (slotType == SlotTypes.Open)
            {
                Component[4].CompName = "Long position entry price";
                Component[4].DataType = IndComponentType.OpenLongPrice;
                Component[5].CompName = "Short position entry price";
                Component[5].DataType = IndComponentType.OpenShortPrice;
            }
            else if (slotType == SlotTypes.Close)
            {
                Component[4].CompName = "Long position closing price";
                Component[4].DataType = IndComponentType.CloseLongPrice;
                Component[5].CompName = "Short position closing price";
                Component[5].DataType = IndComponentType.CloseShortPrice;
            }
		
            switch (IndParam.ListParam[0].Text)
            {
                case "Enter strategy using HTF Visualization":
                    for (int iBar = iFirstBar; iBar < Bars; iBar++)
                    {
                        if (adLELevel[iBar]>Point && Close[iBar]>Open[iBar] && Close[iBar]>Open[iBar-iFrame])
                            Component[4].Value[iBar] = adLELevel[iBar];
                        if (adSELevel[iBar]>Point && Open[iBar]>Close[iBar] && Close[iBar]<Open[iBar-iFrame])
                            Component[5].Value[iBar] = adSELevel[iBar];
                    }
                    break;
				case "Exit strategy using HTF Visualization":
                    for (int iBar = iFirstBar; iBar < Bars; iBar++)
                    {
                        if (adSELevel[iBar]>Point)
                            Component[4].Value[iBar] = adLXLevel[iBar];
                        if (adLELevel[iBar]>Point)
                            Component[5].Value[iBar] = adSXLevel[iBar];
                    }
                    break;
  				default:
                    break;
            }
            return;
		}

        /// <summary>
        /// Sets the indicator logic description
        /// </summary>
        public override void SetDescription(SlotTypes slotType)
        {
            int iShift = (int)IndParam.NumParam[0].Value;

            string sUpperTrade;
            string sLowerTrade;
            
            if (iShift > 0)
            {
                sUpperTrade = iShift + " pips above ";
                sLowerTrade = iShift + " pips below ";
            }
            else if (iShift == 0)
            {
                sUpperTrade = "at ";
                sLowerTrade = "at ";
            }
            else
            {
                sUpperTrade = -iShift + " pips below ";
                sLowerTrade = -iShift + " pips above ";
            }

            switch (IndParam.ListParam[0].Text)
            {
                case "Enter strategy using HTF Visualization":
                    EntryPointLongDescription  = sUpperTrade + "the LE Level";
                    EntryPointShortDescription = sLowerTrade + "the SE Level";
                    break;
                case "Exit strategy using HTF Visualization":
                    ExitPointLongDescription  = sLowerTrade + "the LX Level";
                    ExitPointShortDescription = sUpperTrade + "the SX Level";
                    break;
                default:
                    break;
            }
            return;
        }

        /// <summary>
        /// Indicator to string
        /// </summary>
        public override string ToString()
        {
            string sString =  IndicatorName; 
			
			return sString;
        }
    
		
/*Start Copy4*/		
		/// <summary>
        /// Convert current time frame to higher time frame
        /// </summary>
		protected static int HigherTimeFrame(DataPeriods currentTF, DataPeriods higherTF, out int[] hIndex, out int hBars, out int iFrame, out double[] hfHigh, out double[] hfLow, out double[] hfOpen, out double[] hfClose, out double[] hfVolume)
		{
			hfOpen = new double[Bars];
			hfClose = new double[Bars];
			hfHigh = new double[Bars];
			hfLow = new double[Bars];
			hfVolume = new double[Bars];
			hIndex = new int[Bars];
			hBars =0;
			int iRem =10;
			
			
			// Frame Calculation
			if (higherTF>currentTF)
				iFrame = (int)((int)higherTF/(int)currentTF);
			else
				iFrame = 1;
			if (higherTF==DataPeriods.week && currentTF==DataPeriods.day) iFrame=5;
			if (iFrame>60) return 1;
			
			int hBar=0;
			for (int iBar=1; iBar<Bars; iBar++)
			{			
					
				// Time Synchronization
				int iMin  = (int) Time[iBar].Minute;
				int iHour = (int) Time[iBar].Hour;
				int iDay  = (int) Time[iBar].DayOfWeek;
				if ((int)higherTF<=60 && (int)currentTF<=30)	iRem=(int)((iMin%(int)higherTF)/(int)currentTF);
				else if (higherTF==DataPeriods.hour4 && currentTF==DataPeriods.min15) 	iRem=(int)((iHour%4)*4+(iMin/15));
				else if (higherTF==DataPeriods.hour4 && currentTF==DataPeriods.min30) 	iRem=(int)((iHour%4)*2+(iMin/30));
				else if (higherTF==DataPeriods.hour4 && currentTF==DataPeriods.hour1) 	iRem=(int)(iHour%4);
				else if (higherTF==DataPeriods.day && currentTF==DataPeriods.min30)		iRem=(int)((iHour%24)*2+(iMin/30));
				else if (higherTF==DataPeriods.day && currentTF==DataPeriods.hour1)		iRem=(int)(iHour%24);
				else if (higherTF==DataPeriods.day && currentTF==DataPeriods.hour4)		iRem=(int)((iHour%24)/4);
				else if (higherTF==DataPeriods.week && currentTF==DataPeriods.day)		iRem=(int)((iDay-1)%7);
				else	iRem = iBar % iFrame;
				
				// Higher Time Frame Conversion
				if (iRem == 0)
				{
					hBar++;
					hfOpen[hBar]   = Open[iBar];
					hfClose[hBar]  = Close[iBar];
					hfHigh[hBar]   = High[iBar];
					hfLow[hBar]    = Low[iBar];
					hfVolume[hBar] = Volume[iBar];
				}
				else
				{
					hfClose[hBar]  = Close[iBar];
					if (High[iBar] > hfHigh[hBar])	hfHigh[hBar] = High[iBar];
					if (Low[iBar]  < hfLow[hBar])	hfLow[hBar] = Low[iBar];
					hfVolume[hBar] += Volume[iBar];
				}
				hIndex[iBar] = hBar;
			}
			hBars = hBar+1;
			
			return 0;
		}

		/// <summary>
        /// Convert higher time frame to current time frame
        /// </summary>
		protected static int CurrentTimeFrame(int[] hIndex, int hBars, ref double[] hIndicator)
		{
			double[] hBuffer = new double[Bars];
			int hBar;
			
			for (int iBar=0; iBar<Bars; iBar++)
			{	
				hBar = hIndex[iBar];	
				hBuffer[iBar] = hIndicator[hBar];
			}
			
			for (int iBar=0; iBar<Bars; iBar++)
			{
				hIndicator[iBar] = hBuffer[iBar];
			}
			
			return 0;
		}
				
		/// <summary>
        /// Calculate base price for higher time frame data
        /// </summary>
		protected static int HigherBasePrice(BasePrice basePrice, int hBars, double[] hfHigh, double[] hfLow, double[] hfOpen, double[] hfClose, out double[] hfPrice)
		{
			hfPrice = new double[Bars];
			
			switch (basePrice)
			{
				case BasePrice.Open:	hfPrice = hfOpen;	break;
				case BasePrice.Close:	hfPrice = hfClose;	break;
				case BasePrice.High:	hfPrice = hfHigh;	break;
				case BasePrice.Low:		hfPrice = hfLow;	break;
				case BasePrice.Median:
					for (int iBar=0; iBar<hBars; iBar++)
					{
						hfPrice[iBar] = (hfLow[iBar] + hfHigh[iBar]) / 2;
					}
					break;
				case BasePrice.Typical:
					for (int iBar=0; iBar<hBars; iBar++)
					{
						hfPrice[iBar] = (hfLow[iBar] + hfHigh[iBar] + hfClose[iBar]) / 3;
					}
					break;
				case BasePrice.Weighted:
					for (int iBar=0; iBar<hBars; iBar++)
					{
						hfPrice[iBar] = (hfOpen[iBar] + hfLow[iBar] + hfHigh[iBar] + hfClose[iBar]) / 4;
					}
					break;
				default:
					break;
			}
			return 0;
		}

/*End Copy4*/	
	
	}
}