// Vortex Indicator V1.0.0 (Modified to HTF v 4.0.0 by Denny Imanuel)
// Last changed on 20 Feb 2010
// Part of Forex Strategy Builder v2.8.3.8+
// Website http://forexsb.com/
// Copyright (c) 2010 Denny Imanuel - All rights reserved.
// This code or any part of it cannot be used in other applications without a permission.

using System;
using System.Drawing;

namespace Forex_Strategy_Builder
{
    /// <summary>
    /// Vortex Indicators
    /// </summary>
    public class Vortex_Indicators_HTF : Indicator
    {
        /// <summary>
        /// Sets the default indicator parameters for the designated slot type
        /// </summary>
        public Vortex_Indicators_HTF(SlotTypes slotType)
        {
            // General properties
            IndicatorName  = "Vortex Indicators (HTF v4)";
            PossibleSlots  = SlotTypes.OpenFilter | SlotTypes.CloseFilter;
            SeparatedChart = true;
            // SeparatedChartMinValue = 0;
			CustomIndicator = true;

            // Setting up the indicator parameters
            IndParam = new IndicatorParam();
            IndParam.IndicatorName = IndicatorName;
            IndParam.SlotType      = slotType;

            // The ComboBox parameters
            IndParam.ListParam[0].Caption   = "Logic";
            IndParam.ListParam[0].ItemList  = new string[]
            {
                "The VI+ rises",
                "The VI+ falls",
                "The VI- rises",
                "The VI- falls",
                "The VI+ is higher than VI-",
                "The VI+ is lower than VI-",
                "The VI+ crosses the VI- line upward",
                "The VI+ crosses the VI- line downward",
                "The VI+ changes its direction upward",
                "The VI+ changes its direction downward",
                "The VI- changes its direction upward",
                "The VI- changes its direction downward"
            };
            IndParam.ListParam[0].Index   = 0;
            IndParam.ListParam[0].Text    = IndParam.ListParam[0].ItemList[IndParam.ListParam[0].Index];
            IndParam.ListParam[0].Enabled = true;
            IndParam.ListParam[0].ToolTip = "Logic of application of the indicator.";

            IndParam.ListParam[1].Caption  = "Smoothing method";
            IndParam.ListParam[1].ItemList = Enum.GetNames(typeof(MAMethod));
            IndParam.ListParam[1].Index    = (int)MAMethod.Exponential;
            IndParam.ListParam[1].Text     = IndParam.ListParam[1].ItemList[IndParam.ListParam[1].Index];
            IndParam.ListParam[1].Enabled  = true;
            IndParam.ListParam[1].ToolTip  = "The Moving Average method used for VI smoothing.";

            IndParam.ListParam[4].Caption = "Higher Time Frame Reference";
			IndParam.ListParam[4].ItemList = new string[] { "1 Minute", "5 Minutes", "15 Minutes", "30 Minutes", "1 Hour", "4 Hours", "1 Day", "1 Week"};
            if (Period == DataPeriods.min1) 		IndParam.ListParam[4].Index    = 1;
            else if (Period == DataPeriods.min5) 	IndParam.ListParam[4].Index    = 2;
            else if (Period == DataPeriods.min15) 	IndParam.ListParam[4].Index    = 3;
            else if (Period == DataPeriods.min30)	IndParam.ListParam[4].Index    = 4;
            else if (Period == DataPeriods.hour1)	IndParam.ListParam[4].Index    = 5;
            else if (Period == DataPeriods.hour4)	IndParam.ListParam[4].Index    = 6;
            else if (Period == DataPeriods.day)		IndParam.ListParam[4].Index    = 7;
            IndParam.ListParam[4].Text     = IndParam.ListParam[4].ItemList[IndParam.ListParam[4].Index];
            IndParam.ListParam[4].Enabled  = true;
            IndParam.ListParam[4].ToolTip  = "Choose higher time frame as compared to current chart period for this indicator.";		

			// The NumericUpDown parameters
            IndParam.NumParam[0].Caption = "Period";
            IndParam.NumParam[0].Value   = 14;
            IndParam.NumParam[0].Min     = 5;
            IndParam.NumParam[0].Max     = 200;
            IndParam.NumParam[0].Enabled = true;
            IndParam.NumParam[0].ToolTip = "The period of VI.";

            // The CheckBox parameters
            IndParam.CheckParam[0].Caption = "Use previous bar value.";
            IndParam.CheckParam[0].Checked = (slotType == SlotTypes.OpenFilter);
            IndParam.CheckParam[0].Enabled = false;
            IndParam.CheckParam[0].ToolTip = "Use the indicator value from the previous bar.";

            return;
        }

        /// <summary>
        /// Calculates the indicator's components
        /// </summary>
        public override void Calculate(SlotTypes slotType)
        {
            // Reading the parameters
            MAMethod  maMethod  = (MAMethod)IndParam.ListParam[1].Index;
            //BasePrice basePrice = (BasePrice)IndParam.ListParam[2].Index;
            int iNVI = (int)IndParam.NumParam[0].Value;
            int iPrvs = (slotType == SlotTypes.OpenFilter) ? 1 : 0;

            // Convert to Higher Time Frame ---------------------------------------------
			DataPeriods htfPeriod = DataPeriods.week;
			double[] hfOpen = new double[Bars];
			double[] hfClose = new double[Bars];
			double[] hfHigh = new double[Bars];
			double[] hfLow = new double[Bars];
			double[] hfVolume = new double[Bars];
			double[] hfPrice = new double[Bars];
			int[] 	 hIndex = new int[Bars];
			int      iFrame;
			int      hBars;
			
			switch (IndParam.ListParam[4].Index)
			{
				case 1: htfPeriod = DataPeriods.min5; break;
				case 2: htfPeriod = DataPeriods.min15; break;
				case 3: htfPeriod = DataPeriods.min30; break;
				case 4: htfPeriod = DataPeriods.hour1; break;
				case 5: htfPeriod = DataPeriods.hour4; break;
				case 6: htfPeriod = DataPeriods.day; break;
				case 7: htfPeriod = DataPeriods.week; break;
			}
			
			int err1 = HigherTimeFrame(Period, htfPeriod, out hIndex, out hBars, out iFrame, out hfHigh, out hfLow, out hfOpen, out hfClose, out hfVolume);
			//int err2 = HigherBasePrice(basePrice, hBars, hfHigh, hfLow, hfOpen, hfClose, out hfPrice);	
			if (err1==1) return;
			//------------------------------------------------------------------------

			// Calculation
            //double[] adBasePrice = Price(basePrice);
            int iFirstBar = iNVI + 2;

            double[] adVIPos = new double[Bars];
            double[] adVINeg = new double[Bars];
            double[] adVIOsc = new double[Bars];
            double[] adVMPlus  = new double[Bars];
            double[] adVMMinus  = new double[Bars];
			double[] adVMPlusSum  = new double[Bars];
            double[] adVMMinusSum  = new double[Bars];
			double[] adTrueRange  = new double[Bars];
            double[] adTrueHigh  = new double[Bars];
			double[] adTrueLow  = new double[Bars];
			double[] adTRSum  = new double[Bars];
			
			for (int iBar=iFirstBar; iBar<hBars; iBar++)
			{
				adVMPlus[iBar] 	  = Math.Abs(hfHigh[iBar]-hfLow[iBar-1]);
				adVMMinus[iBar]   = Math.Abs(hfLow[iBar] -hfHigh[iBar-1]);
				if (hfClose[iBar-1]>hfHigh[iBar]) adTrueHigh[iBar]=hfClose[iBar-1]; else adTrueHigh[iBar]=hfHigh[iBar];
				if (hfClose[iBar-1]<hfLow[iBar])  adTrueLow[iBar]=hfClose[iBar-1];  else adTrueLow[iBar]=hfLow[iBar];
				adTrueRange[iBar] = adTrueHigh[iBar] - adTrueLow[iBar];
				adVMPlusSum[iBar]  = 0;
				adVMMinusSum[iBar] = 0;
				adTRSum[iBar] = 0;
				for (int iBack=0; iBack<iNVI; iBack++)
				{
					adVMPlusSum[iBar]  += adVMPlus[iBar-iBack];
					adVMMinusSum[iBar] += adVMMinus[iBar-iBack];
					adTRSum[iBar] 	   += adTrueRange[iBar-iBack];					
				}
				if (adTRSum[iBar]!=0)
				adVIPos[iBar] = adVMPlusSum[iBar]/adTRSum[iBar];
				adVINeg[iBar] = adVMMinusSum[iBar]/adTRSum[iBar];
			}
			
            for (int iBar = 0; iBar < hBars; iBar++)
                adVIOsc[iBar] = adVIPos[iBar] - adVINeg[iBar];

			// Convert to Current Time Frame ----------------------------------------------
			iPrvs = iPrvs*iFrame;
			iFirstBar = iFirstBar*iFrame;
			if (iFirstBar>0.25*Bars) return;
			
			int err3 = CurrentTimeFrame(hIndex, hBars, ref adVIPos);
			int err4 = CurrentTimeFrame(hIndex, hBars, ref adVINeg);
			int err5 = CurrentTimeFrame(hIndex, hBars, ref adVIOsc);		
			//-----------------------------------------------------------------------------	

            // Saving the components
            Component = new IndicatorComp[4];

            Component[0] = new IndicatorComp();
            Component[0].CompName   = "The VI+";
            Component[0].DataType   = IndComponentType.IndicatorValue;
            Component[0].ChartType  = IndChartType.Line;
            Component[0].ChartColor = Color.Green;
            Component[0].FirstBar   = iFirstBar;
            Component[0].Value      = adVIPos;

            Component[1] = new IndicatorComp();
            Component[1].CompName   = "The VI-";
            Component[1].DataType   = IndComponentType.IndicatorValue;
            Component[1].ChartType  = IndChartType.Line;
            Component[1].ChartColor = Color.Red;
            Component[1].FirstBar   = iFirstBar;
            Component[1].Value      = adVINeg;

            Component[2] = new IndicatorComp();
            Component[2].ChartType = IndChartType.NoChart;
            Component[2].FirstBar  = iFirstBar;
            Component[2].Value     = new double[Bars];

            Component[3] = new IndicatorComp();
            Component[3].ChartType = IndChartType.NoChart;
            Component[3].FirstBar  = iFirstBar;
            Component[3].Value     = new double[Bars];

            // Sets the Component's type
            if (slotType == SlotTypes.OpenFilter)
            {
                Component[2].DataType = IndComponentType.AllowOpenLong;
                Component[2].CompName = "Is long entry allowed";
                Component[3].DataType = IndComponentType.AllowOpenShort;
                Component[3].CompName = "Is short entry allowed";
            }
            else if (slotType == SlotTypes.CloseFilter)
            {
                Component[2].DataType = IndComponentType.ForceCloseLong;
                Component[2].CompName = "Close out long position";
                Component[3].DataType = IndComponentType.ForceCloseShort;
                Component[3].CompName = "Close out short position";
            }

            switch (IndParam.ListParam[0].Text)
            {
                case "The VI+ rises":
                    OscillatorLogic(iFirstBar, iPrvs, adVIPos, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_rises);
                    break;

                case "The VI+ falls":
                    OscillatorLogic(iFirstBar, iPrvs, adVIPos, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_falls);
                    break;

                case "The VI- rises":
                    OscillatorLogic(iFirstBar, iPrvs, adVINeg, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_rises);
                    break;

                case "The VI- falls":
                    OscillatorLogic(iFirstBar, iPrvs, adVINeg, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_falls);
                    break;

                case "The VI+ is higher than VI-":
                    OscillatorLogic(iFirstBar, iPrvs, adVIOsc, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_is_higher_than_the_level_line);
                    break;

                case "The VI+ is lower than VI-":
                    OscillatorLogic(iFirstBar, iPrvs, adVIOsc, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_is_lower_than_the_level_line);
                    break;

                case "The VI+ crosses the VI- line upward":
                    OscillatorLogic(iFirstBar, iPrvs, adVIOsc, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_crosses_the_level_line_upward);
                    break;

                case "The VI+ crosses the VI- line downward":
                    OscillatorLogic(iFirstBar, iPrvs, adVIOsc, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_crosses_the_level_line_downward);
                    break;

                case "The VI+ changes its direction upward":
                    OscillatorLogic(iFirstBar, iPrvs, adVIPos, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_changes_its_direction_upward);
                    break;

                case "The VI+ changes its direction downward":
                    OscillatorLogic(iFirstBar, iPrvs, adVIPos, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_changes_its_direction_downward);
                    break;

                case "The VI- changes its direction upward":
                    OscillatorLogic(iFirstBar, iPrvs, adVINeg, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_changes_its_direction_upward);
                    break;

                case "The VI- changes its direction downward":
                    OscillatorLogic(iFirstBar, iPrvs, adVINeg, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_changes_its_direction_downward);
                    break;

                default:
                    break;
            }

            return;
        }

        /// <summary>
        /// Sets the indicator logic description
        /// </summary>
        public override void SetDescription(SlotTypes slotType)
        {
            EntryFilterLongDescription  = ToString() + "; ";
            EntryFilterShortDescription = ToString() + "; ";
            ExitFilterLongDescription   = ToString() + "; ";
            ExitFilterShortDescription  = ToString() + "; ";

            switch (IndParam.ListParam[0].Text)
            {
                case "The VI+ rises":
                    EntryFilterLongDescription  += "the VI+ rises";
                    EntryFilterShortDescription += "the VI+ falls";
                    ExitFilterLongDescription   += "the VI+ rises";
                    ExitFilterShortDescription  += "the VI+ falls";
                    break;

                case "The VI+ falls":
                    EntryFilterLongDescription  += "the VI+ falls";
                    EntryFilterShortDescription += "the VI+ rises";
                    ExitFilterLongDescription   += "the VI+ falls";
                    ExitFilterShortDescription  += "the VI+ rises";
                    break;

                case "The VI- rises":
                    EntryFilterLongDescription  += "the VI- rises";
                    EntryFilterShortDescription += "the VI- falls";
                    ExitFilterLongDescription   += "the VI- rises";
                    ExitFilterShortDescription  += "the VI- falls";
                    break;

                case "The VI- falls":
                    EntryFilterLongDescription  += "the VI- falls";
                    EntryFilterShortDescription += "the VI- rises";
                    ExitFilterLongDescription   += "the VI- falls";
                    ExitFilterShortDescription  += "the VI- rises";
                    break;

                case "The VI+ is higher than VI-":
                    EntryFilterLongDescription  += "the VI+ is higher than the VI-";
                    EntryFilterShortDescription += "the VI+ is lower than the VI-";
                    ExitFilterLongDescription   += "the VI+ is higher than the VI-";
                    ExitFilterShortDescription  += "the VI+ is lower than the VI-";
                    break;

                case "The VI+ is lower than VI-":
                    EntryFilterLongDescription  += "the VI+ is lower than the VI-";
                    EntryFilterShortDescription += "the VI+ is higher than the VI-";
                    ExitFilterLongDescription   += "the VI+ is lower than the VI-";
                    ExitFilterShortDescription  += "the VI+ is higher than the VI-";
                    break;

                case "The VI+ crosses the VI- line upward":
                    EntryFilterLongDescription  += "the VI+ crosses the VI- line upward";
                    EntryFilterShortDescription += "the VI+ crosses the VI- line downward";
                    ExitFilterLongDescription   += "the VI+ crosses the VI- line upward";
                    ExitFilterShortDescription  += "the VI+ crosses the VI- line downward";
                    break;

                case "The VI+ crosses the VI- line downward":
                    EntryFilterLongDescription  += "the VI+ crosses the VI- line downward";
                    EntryFilterShortDescription += "the VI+ crosses the VI- line upward";
                    ExitFilterLongDescription   += "the VI+ crosses the VI- line downward";
                    ExitFilterShortDescription  += "the VI+ crosses the VI- line upward";
                    break;

                case "The VI+ changes its direction upward":
                    EntryFilterLongDescription  += "the VI+ changes its direction upward";
                    EntryFilterShortDescription += "the VI+ changes its direction downward";
                    ExitFilterLongDescription   += "the VI+ changes its direction upward";
                    ExitFilterShortDescription  += "the VI+ changes its direction downward";
                    break;

                case "The VI+ changes its direction downward":
                    EntryFilterLongDescription  += "the VI+ changes its direction downward";
                    EntryFilterShortDescription += "the VI+ changes its direction upward";
                    ExitFilterLongDescription   += "the VI+ changes its direction downward";
                    ExitFilterShortDescription  += "the VI+ changes its direction upward";
                    break;

                case "The VI- changes its direction upward":
                    EntryFilterLongDescription  += "the VI- changes its direction upward";
                    EntryFilterShortDescription += "the VI- changes its direction downward";
                    ExitFilterLongDescription   += "the VI- changes its direction upward";
                    ExitFilterShortDescription  += "the VI- changes its direction downward";
                    break;

                case "The VI- changes its direction downward":
                    EntryFilterLongDescription  += "the VI- changes its direction downward";
                    EntryFilterShortDescription += "the VI- changes its direction upward";
                    ExitFilterLongDescription   += "the VI- changes its direction downward";
                    ExitFilterShortDescription  += "the VI- changes its direction upward";
                    break;

                default:
                    break;
            }

            return;
        }

        /// <summary>
        /// Indicator to string
        /// </summary>
        public override string ToString()
        {
            string sString = IndicatorName + 
                (IndParam.SlotType == SlotTypes.OpenFilter ? "* (" : " (") +
                IndParam.ListParam[1].Text         + ", " + // Method
                IndParam.ListParam[2].Text         + ", " + // Base price
                IndParam.NumParam[0].ValueToString + ")";   // Period

            return sString;
        }
				/// <summary>
        /// Convert current time frame to higher time frame
        /// </summary>
		protected static int HigherTimeFrame(DataPeriods currentTF, DataPeriods higherTF, out int[] hIndex, out int hBars, out int iFrame, out double[] hfHigh, out double[] hfLow, out double[] hfOpen, out double[] hfClose, out double[] hfVolume)
		{
			hfOpen = new double[Bars];
			hfClose = new double[Bars];
			hfHigh = new double[Bars];
			hfLow = new double[Bars];
			hfVolume = new double[Bars];
			hIndex = new int[Bars];
			hBars =0;
			int iRem =10;
			
			
			// Frame Calculation
			if (higherTF>currentTF)
				iFrame = (int)((int)higherTF/(int)currentTF);
			else
				iFrame = 1;
			if (higherTF==DataPeriods.week && currentTF==DataPeriods.day) iFrame=5;
			if (iFrame>60) return 1;
			
			int hBar=0;
			for (int iBar=1; iBar<Bars; iBar++)
			{			
					
				// Time Synchronization
				int iMin  = (int) Time[iBar].Minute;
				int iHour = (int) Time[iBar].Hour;
				int iDay  = (int) Time[iBar].DayOfWeek;
				if ((int)higherTF<=60 && (int)currentTF<=30)	iRem=(int)((iMin%(int)higherTF)/(int)currentTF);
				else if (higherTF==DataPeriods.hour4 && currentTF==DataPeriods.min15) 	iRem=(int)((iHour%4)*4+(iMin/15));
				else if (higherTF==DataPeriods.hour4 && currentTF==DataPeriods.min30) 	iRem=(int)((iHour%4)*2+(iMin/30));
				else if (higherTF==DataPeriods.hour4 && currentTF==DataPeriods.hour1) 	iRem=(int)(iHour%4);
				else if (higherTF==DataPeriods.day && currentTF==DataPeriods.min30)		iRem=(int)((iHour%24)*2+(iMin/30));
				else if (higherTF==DataPeriods.day && currentTF==DataPeriods.hour1)		iRem=(int)(iHour%24);
				else if (higherTF==DataPeriods.day && currentTF==DataPeriods.hour4)		iRem=(int)((iHour%24)/4);
				else if (higherTF==DataPeriods.week && currentTF==DataPeriods.day)		iRem=(int)((iDay-1)%7);
				else	iRem = iBar % iFrame;
				
				// Higher Time Frame Conversion
				if (iRem == 0)
				{
					hBar++;
					hfOpen[hBar]   = Open[iBar];
					hfClose[hBar]  = Close[iBar];
					hfHigh[hBar]   = High[iBar];
					hfLow[hBar]    = Low[iBar];
					hfVolume[hBar] = Volume[iBar];
				}
				else
				{
					hfClose[hBar]  = Close[iBar];
					if (High[iBar] > hfHigh[hBar])	hfHigh[hBar] = High[iBar];
					if (Low[iBar]  < hfLow[hBar])	hfLow[hBar] = Low[iBar];
					hfVolume[hBar] += Volume[iBar];
				}
				hIndex[iBar] = hBar;
			}
			hBars = hBar+1;
			
			return 0;
		}

		/// <summary>
        /// Convert higher time frame to current time frame
        /// </summary>
		protected static int CurrentTimeFrame(int[] hIndex, int hBars, ref double[] hIndicator)
		{
			double[] hBuffer = new double[Bars];
			int hBar;
			
			for (int iBar=0; iBar<Bars; iBar++)
			{	
				hBar = hIndex[iBar];	
				hBuffer[iBar] = hIndicator[hBar];
			}
			
			for (int iBar=0; iBar<Bars; iBar++)
			{
				hIndicator[iBar] = hBuffer[iBar];
			}
			
			return 0;
		}
				
		/// <summary>
        /// Calculate base price for higher time frame data
        /// </summary>
		protected static int HigherBasePrice(BasePrice basePrice, int hBars, double[] hfHigh, double[] hfLow, double[] hfOpen, double[] hfClose, out double[] hfPrice)
		{
			hfPrice = new double[Bars];
			
			switch (basePrice)
			{
				case BasePrice.Open:	hfPrice = hfOpen;	break;
				case BasePrice.Close:	hfPrice = hfClose;	break;
				case BasePrice.High:	hfPrice = hfHigh;	break;
				case BasePrice.Low:		hfPrice = hfLow;	break;
				case BasePrice.Median:
					for (int iBar=0; iBar<hBars; iBar++)
					{
						hfPrice[iBar] = (hfLow[iBar] + hfHigh[iBar]) / 2;
					}
					break;
				case BasePrice.Typical:
					for (int iBar=0; iBar<hBars; iBar++)
					{
						hfPrice[iBar] = (hfLow[iBar] + hfHigh[iBar] + hfClose[iBar]) / 3;
					}
					break;
				case BasePrice.Weighted:
					for (int iBar=0; iBar<hBars; iBar++)
					{
						hfPrice[iBar] = (hfOpen[iBar] + hfLow[iBar] + hfHigh[iBar] + hfClose[iBar]) / 4;
					}
					break;
				default:
					break;
			}
			return 0;
		}


	}
}