// R-Squared Indicator (Modified to HTF v 4.0.0 by Denny Imanuel)
// Last changed on 20 Mar 2010
// Part of Forex Strategy Builder v2.8.3.7+
// Website http://forexsb.com/
// Copyright (c) 2010 Denny Imanuel - All rights reserved.
// This code or any part of it cannot be used in other applications without a permission.

using System;
using System.Drawing;

namespace Forex_Strategy_Builder
{
    /// <summary>
    /// R Squared Indicator
    /// </summary>
    public class R_Squared : Indicator
    {
        /// <summary>
        /// Sets the default indicator parameters for the designated slot type
        /// </summary>
        public R_Squared(SlotTypes slotType)
        {
            // General properties
            IndicatorName  = "R-Squared";
            PossibleSlots  = SlotTypes.OpenFilter | SlotTypes.CloseFilter;
            SeparatedChart = true;
			CustomIndicator = true;

            // Setting up the indicator parameters
            IndParam = new IndicatorParam();
            IndParam.IndicatorName = IndicatorName;
            IndParam.SlotType      = slotType;
            IndParam.IndicatorType = TypeOfIndicator.Additional;

            // The ComboBox parameters
            IndParam.ListParam[0].Caption  = "Logic";
            IndParam.ListParam[0].ItemList = new string[]
            {
                "The R-Squared rises",
                "The R-Squared falls",
                "The R-Squared is higher than the Level line",
                "The R-Squared is lower than the Level line",
                "The R-Squared crosses the Level line upward",
                "The R-Squared crosses the Level line downward",
                "The R-Squared changes its direction upward",
                "The R-Squared changes its direction downward"
            };
            IndParam.ListParam[0].Index   = 0;
            IndParam.ListParam[0].Text    = IndParam.ListParam[0].ItemList[IndParam.ListParam[0].Index];
            IndParam.ListParam[0].Enabled = true;
            IndParam.ListParam[0].ToolTip = "Logic of application of the indicator.";

            IndParam.ListParam[1].Caption  = "Smoothing method";
            IndParam.ListParam[1].ItemList = Enum.GetNames(typeof(MAMethod));
            IndParam.ListParam[1].Index    = (int)MAMethod.Simple;
            IndParam.ListParam[1].Text     = IndParam.ListParam[1].ItemList[IndParam.ListParam[1].Index];
            IndParam.ListParam[1].Enabled  = true;
            IndParam.ListParam[1].ToolTip  = "The method of Moving Average used for the calculations.";

            IndParam.ListParam[2].Caption  = "Base price";
            IndParam.ListParam[2].ItemList = Enum.GetNames(typeof(BasePrice));
            IndParam.ListParam[2].Index    = (int)BasePrice.Close;
            IndParam.ListParam[2].Text     = IndParam.ListParam[2].ItemList[IndParam.ListParam[2].Index];
            IndParam.ListParam[2].Enabled  = true;
            IndParam.ListParam[2].ToolTip  = "The price the indicator is based on.";

            // The NumericUpDown parameters
            IndParam.NumParam[0].Caption = "R-Squared Period";
            IndParam.NumParam[0].Value   = 30;
            IndParam.NumParam[0].Min     = 2;
            IndParam.NumParam[0].Max     = 200;
            IndParam.NumParam[0].Enabled = true;
            IndParam.NumParam[0].ToolTip = "The period of R-Squared calculation.";

            IndParam.NumParam[1].Caption = "Smoothing Period";
            IndParam.NumParam[1].Value   = 3;
            IndParam.NumParam[1].Min     = 2;
            IndParam.NumParam[1].Max     = 200;
            IndParam.NumParam[1].Enabled = true;
            IndParam.NumParam[1].ToolTip = "The period of Smoothing calculation.";
			
			IndParam.NumParam[2].Caption = "Level";
            IndParam.NumParam[2].Value   = 0;
            IndParam.NumParam[2].Min     = 0;
            IndParam.NumParam[2].Max     = 100;
            IndParam.NumParam[2].Point   = 4;
            IndParam.NumParam[2].Enabled = true;
            IndParam.NumParam[2].ToolTip = "A critical level (for the appropriate logic).";

            // The CheckBox parameters
            IndParam.CheckParam[0].Caption = "Use previous bar value";
            IndParam.CheckParam[0].Checked = PrepareUsePrevBarValueCheckBox(slotType);
            IndParam.CheckParam[0].Enabled = true;
            IndParam.CheckParam[0].ToolTip = "Use the indicator value from the previous bar.";

            return;
        }

        /// <summary>
        /// Calculates the indicator's components
        /// </summary>
        public override void Calculate(SlotTypes slotType)
        {
            // Reading the parameters
            MAMethod  maMethod 		= (MAMethod)IndParam.ListParam[1].Index;
            BasePrice basePrice     = (BasePrice)IndParam.ListParam[2].Index;
            int       iRSPeriod		= (int)IndParam.NumParam[0].Value;
			int       iMAPeriod		= (int)IndParam.NumParam[1].Value;
            double    dLevel   		=      IndParam.NumParam[2].Value;
            int       iPrvs    		=      IndParam.CheckParam[0].Checked ? 1 : 0;

            // Calculation
            double[] adPrice 	= Price(basePrice);
            double[] adR	    = RCoefficient(adPrice, iRSPeriod);
			double[] adRR  = new double[Bars];
			double[] adAvgRR  = new double[Bars];

            int iFirstBar = iRSPeriod + iMAPeriod + 1;
			
            for (int iBar=iFirstBar; iBar< Bars; iBar++)
            {
				adRR[iBar] = Math.Pow(adR[iBar],2);
			}	
            
			adAvgRR = MovingAverage(iMAPeriod, 0, maMethod, adRR);;

            // Saving the components
            Component = new IndicatorComp[3];

            Component[0] = new IndicatorComp();
            Component[0].CompName   = "R-Squared";
            Component[0].DataType   = IndComponentType.IndicatorValue;
            Component[0].ChartType  = IndChartType.Line;
            Component[0].ChartColor = Color.Blue;
            Component[0].FirstBar   = iFirstBar;
            Component[0].Value      = adAvgRR;

            Component[1] = new IndicatorComp();
            Component[1].ChartType = IndChartType.NoChart;
            Component[1].FirstBar  = iFirstBar;
            Component[1].Value     = new double[Bars];

            Component[2] = new IndicatorComp();
            Component[2].ChartType = IndChartType.NoChart;
            Component[2].FirstBar  = iFirstBar;
            Component[2].Value     = new double[Bars];

            // Sets the Component's type
            if (slotType == SlotTypes.OpenFilter)
            {
                Component[1].DataType = IndComponentType.AllowOpenLong;
                Component[1].CompName = "Is long entry allowed";
                Component[2].DataType = IndComponentType.AllowOpenShort;
                Component[2].CompName = "Is short entry allowed";
            }
            else if (slotType == SlotTypes.CloseFilter)
            {
                Component[1].DataType = IndComponentType.ForceCloseLong;
                Component[1].CompName = "Close out long position";
                Component[2].DataType = IndComponentType.ForceCloseShort;
                Component[2].CompName = "Close out short position";
            }

            // Calculation of the logic
            IndicatorLogic indLogic = IndicatorLogic.It_does_not_act_as_a_filter;

            switch (IndParam.ListParam[0].Text)
            {
                case "The R-Squared rises":
                    indLogic = IndicatorLogic.The_indicator_rises;
                    break;

                case "The R-Squared falls":
                    indLogic = IndicatorLogic.The_indicator_falls;
                    break;

                case "The R-Squared is higher than the Level line":
                    indLogic = IndicatorLogic.The_indicator_is_higher_than_the_level_line;
                    SpecialValues = new double[1] { dLevel };
                    break;

                case "The R-Squared is lower than the Level line":
                    indLogic = IndicatorLogic.The_indicator_is_lower_than_the_level_line;
                    SpecialValues = new double[1] { dLevel };
                    break;

                case "The R-Squared crosses the Level line upward":
                    indLogic = IndicatorLogic.The_indicator_crosses_the_level_line_upward;
                    SpecialValues = new double[1] { dLevel };
                    break;

                case "The R-Squared crosses the Level line downward":
                    indLogic = IndicatorLogic.The_indicator_crosses_the_level_line_downward;
                    SpecialValues = new double[1] { dLevel };
                    break;

                case "The R-Squared changes its direction upward":
                    indLogic = IndicatorLogic.The_indicator_changes_its_direction_upward;
                    break;

                case "The R-Squared changes its direction downward":
                    indLogic = IndicatorLogic.The_indicator_changes_its_direction_downward;
                    break;

                default:
                    break;
            }

            NoDirectionOscillatorLogic(iFirstBar, iPrvs, adAvgRR, dLevel, ref Component[1], indLogic);
            Component[2].Value = Component[1].Value;

            return;
        }

        /// <summary>
        /// Sets the indicator logic description
        /// </summary>
        public override void SetDescription(SlotTypes slotType)
        {
            string sLevelLong  = IndParam.NumParam[1].ValueToString;
            string sLevelShort = sLevelLong;

            EntryFilterLongDescription  = "the " + ToString() + " ";
            EntryFilterShortDescription = "the " + ToString() + " ";
            ExitFilterLongDescription   = "the " + ToString() + " ";
            ExitFilterShortDescription  = "the " + ToString() + " ";

            switch (IndParam.ListParam[0].Text)
            {
                case "The R-Squared rises":
                    EntryFilterLongDescription  += "rises";
                    EntryFilterShortDescription += "rises";
                    ExitFilterLongDescription   += "rises";
                    ExitFilterShortDescription  += "rises";
                    break;

                case "The R-Squared falls":
                    EntryFilterLongDescription  += "falls";
                    EntryFilterShortDescription += "falls";
                    ExitFilterLongDescription   += "falls";
                    ExitFilterShortDescription  += "falls";
                    break;

                case "The R-Squared is higher than the Level line":
                    EntryFilterLongDescription  += "is higher than the Level " + sLevelLong;
                    EntryFilterShortDescription += "is higher than the Level " + sLevelShort;
                    ExitFilterLongDescription   += "is higher than the Level " + sLevelLong;
                    ExitFilterShortDescription  += "is higher than the Level " + sLevelShort;
                    break;

                case "The R-Squared is lower than the Level line":
                    EntryFilterLongDescription  += "is lower than the Level "  + sLevelLong;
                    EntryFilterShortDescription += "is lower than the Level "  + sLevelShort;
                    ExitFilterLongDescription   += "is lower than the Level "  + sLevelLong;
                    ExitFilterShortDescription  += "is lower than the Level "  + sLevelShort;
                    break;

                case "The R-Squared crosses the Level line upward":
                    EntryFilterLongDescription  += "crosses the Level " + sLevelLong  + " upward";
                    EntryFilterShortDescription += "crosses the Level " + sLevelShort + " upward";
                    ExitFilterLongDescription   += "crosses the Level " + sLevelLong  + " upward";
                    ExitFilterShortDescription  += "crosses the Level " + sLevelShort + " upward";
                    break;

                case "The R-Squared crosses the Level line downward":
                    EntryFilterLongDescription  += "crosses the Level " + sLevelLong  + " downward";
                    EntryFilterShortDescription += "crosses the Level " + sLevelShort + " downward";
                    ExitFilterLongDescription   += "crosses the Level " + sLevelLong  + " downward";
                    ExitFilterShortDescription  += "crosses the Level " + sLevelShort + " downward";
                    break;

                case "The R-Squared changes its direction upward":
                    EntryFilterLongDescription  += "changes its direction upward";
                    EntryFilterShortDescription += "changes its direction upward";
                    ExitFilterLongDescription   += "changes its direction upward";
                    ExitFilterShortDescription  += "changes its direction upward";
                    break;

                case "The R-Squared changes its direction downward":
                    EntryFilterLongDescription  += "changes its direction downward";
                    EntryFilterShortDescription += "changes its direction downward";
                    ExitFilterLongDescription   += "changes its direction downward";
                    ExitFilterShortDescription  += "changes its direction downward";
                    break;

                default:
                    break;
            }

            return;
        }

        /// <summary>
        /// Indicator to string
        /// </summary>
        public override string ToString()
        {
            string sString = IndicatorName + 
                (IndParam.CheckParam[0].Checked ? "* (" : " (") +
                IndParam.ListParam[1].Text         + ", " + // Smoothing method
                IndParam.ListParam[2].Text         + ", " + // Base price
                IndParam.NumParam[0].ValueToString + ")";   // Period

            return sString;
        }
		
		protected static double[] RCoefficient(double[] adPrice, int iLength)
		{
			double[] RCoeff	= new double[Bars];
			double[] X		= new double[Bars];
			double[] Y  	= new double[Bars];
			double AvgX;
			double AvgP;
			double SumXP;
			double SumXX;
			double SumPP;
			double LowerEQ1;
			double LowerEQ2;
			double UpperEQ;
			double LowerEQ;
			
			for (int iBar=iLength; iBar<Bars; iBar++)
			{
				X[iBar]	 = iBar;
				Y[iBar]  = adPrice[iBar];
				
				AvgX = 0; AvgP = 0; SumXP=0; SumXX =0; SumPP=0;
				for (int iBack=0; iBack<iLength; iBack++)
				{
					AvgX = AvgX + X[iBar-iBack]/iLength;
					AvgP = AvgP + Y[iBar-iBack]/iLength;
					SumXP = SumXP + X[iBar-iBack]*Y[iBar-iBack];
					SumXX = SumXX + X[iBar-iBack]*X[iBar-iBack];
					SumPP = SumPP + Y[iBar-iBack]*Y[iBar-iBack];
				}
				LowerEQ1 	 = SumXX - (iLength*AvgX*AvgX);
				LowerEQ2	 = SumPP - (iLength*AvgP*AvgP);
				UpperEQ 	 = SumXP - (iLength*AvgX*AvgP);
				LowerEQ		 = Math.Sqrt(Math.Abs(LowerEQ1*LowerEQ2));
				if (LowerEQ!=0)	
					RCoeff[iBar] = UpperEQ/LowerEQ;
			}
			return RCoeff;
		}
		

    }
}
 