// Sine Wave Indicator
// Last changed on 2009-05-05
// Part of Forex Strategy Builder v2.8.3.7+
// Website http://forexsb.com/
// This code or any part of it cannot be used in other applications without a permission.
// Copyright (c) 2010 Denny Imanuel - All rights reserved.

using System;
using System.Drawing;

namespace Forex_Strategy_Builder
{
    /// <summary>
    /// Relative Vigor Index Indicator
    /// </summary>
    public class Sinewave : Indicator
    {
        /// <summary>
        /// Sets the default indicator parameters for the designated slot type
        /// </summary>
        public Sinewave(SlotTypes slotType)
        {
            // General properties
            IndicatorName  = "Sinewave";
            PossibleSlots  = SlotTypes.OpenFilter | SlotTypes.CloseFilter;
            SeparatedChart = true;
			CustomIndicator = true;

            // Setting up the indicator parameters
            IndParam = new IndicatorParam();
            IndParam.IndicatorName = IndicatorName;
            IndParam.SlotType      = slotType;

            // The ComboBox parameters
            IndParam.ListParam[0].Caption  = "Logic";
            IndParam.ListParam[0].ItemList = new string[]
            {
                "The Sinewave rises",
                "The Sinewave falls",
                "The Sinewave is higher than zero",
                "The Sinewave is lower than zero",
                "The Sinewave crosses the zero line upward",
                "The Sinewave crosses the zero line downward",
                "The Sinewave changes its direction upward",
                "The Sinewave changes its direction downward",
                "The Sinewave crosses the Signal line upward",
                "The Sinewave crosses the Signal line downward",
                "The Sinewave is higher than the Signal line",
                "The Sinewave is lower than the Signal line"
            };
            IndParam.ListParam[0].Index    = 0;
            IndParam.ListParam[0].Text     = IndParam.ListParam[0].ItemList[IndParam.ListParam[0].Index];
            IndParam.ListParam[0].Enabled  = true;
            IndParam.ListParam[0].ToolTip  = "Logic of application of the indicator.";

            IndParam.ListParam[1].Caption  = "Base price";
            IndParam.ListParam[1].ItemList = Enum.GetNames(typeof(BasePrice));
            IndParam.ListParam[1].Index    = (int)BasePrice.Median;
            IndParam.ListParam[1].Text     = IndParam.ListParam[1].ItemList[IndParam.ListParam[1].Index];
            IndParam.ListParam[1].Enabled  = true;
            IndParam.ListParam[1].ToolTip  = "The price both Moving Averages are based on.";


            // The CheckBox parameters
            IndParam.CheckParam[0].Caption = "Use previous bar value";
            IndParam.CheckParam[0].Checked = PrepareUsePrevBarValueCheckBox(slotType);
            IndParam.CheckParam[0].Enabled = true;
            IndParam.CheckParam[0].ToolTip = "Use the indicator value from the previous bar.";

            return;
        }

        /// <summary>
        /// Calculates the indicator's components
        /// </summary>
        public override void Calculate(SlotTypes slotType)
        {
            // Reading the parameters
			BasePrice basePrice = (BasePrice)IndParam.ListParam[1].Index;
            int iPrvs = IndParam.CheckParam[0].Checked ? 1 : 0;

            // Calculation
            int iFirstBar =  100;

            double[] adPrice = Price(basePrice);
			double[] adSine = new double[Bars];
			double[] adLeadSine = new double[Bars];
			double[] Value1 = new double[Bars];
			double[] Value2 = new double[Bars];
			double[] Value3 = new double[Bars];
			double[] Value4 = new double[Bars];
			double[] Value5 = new double[Bars];
			double[] InPhase = new double[Bars];
			double[] Quadrature = new double[Bars];
			double[] Phase = new double[Bars];
			double[] DeltaPhase = new double[Bars];
			double[] InstCycle = new double[Bars];

			double DCPhase=0;
			double Cycle;
			double RealPart;
			double ImagPart;
			
            for (int iBar = iFirstBar; iBar < Bars; iBar++)
            {
			
				//Compute InPhase and Quadrature components
				Value1[iBar] = adPrice[iBar] - adPrice[iBar-6];
				Value2[iBar] = Value1[iBar-3];
				Value3[iBar] = .75*(Value1[iBar] - Value1[iBar-6]) + .25*(Value1[iBar-2] - Value1[iBar-4]);
				InPhase[iBar] = .33*Value2[iBar] + .67*InPhase[iBar-1];
				Quadrature[iBar] = .2*Value3[iBar] + .8*Quadrature[iBar-1];

				//Use ArcTangent to compute the current phase
				if (Math.Abs(InPhase[iBar]+InPhase[iBar-1]) > 0) 
					Phase[iBar] = Math.Atan(Math.Abs((Quadrature[iBar]+Quadrature[iBar-1]) / (InPhase[iBar]+InPhase[iBar-1])));

				//Resolve the ArcTangent ambiguity
				if (InPhase[iBar] < 0 && Quadrature[iBar] > 0) Phase[iBar] = Math.PI - Phase[iBar];
				if (InPhase[iBar] < 0 && Quadrature[iBar] < 0) Phase[iBar] = Math.PI + Phase[iBar];
				if (InPhase[iBar] > 0 && Quadrature[iBar] < 0) Phase[iBar] = 2*Math.PI - Phase[iBar];

				//Compute a differential phase, resolve phase wraparound, and limit delta phase errors
				DeltaPhase[iBar] = Phase[iBar-1] - Phase[iBar];
				if (Phase[iBar-1] < 0.5*Math.PI && Phase[iBar] > 1.5*Math.PI) DeltaPhase[iBar] = 2*Math.PI + Phase[iBar-1] - Phase[iBar];
				if (DeltaPhase[iBar] < (Math.PI/180)) DeltaPhase[iBar] = Math.PI/180;
				if (DeltaPhase[iBar] > (Math.PI/3)) DeltaPhase[iBar] = Math.PI/3;

				//Sum DeltaPhases to reach 360 degrees. The sum is the instantaneous period.
				InstCycle[iBar] = 0;
				Value4[iBar] = 0;
				for (int count = 0; count<=40; count++)
				{
					Value4[iBar] = Value4[iBar] + DeltaPhase[iBar-count];
					if (Value4[iBar] > 2*Math.PI && InstCycle[iBar] == 0)
						InstCycle[iBar] = count;
				}
				

				//Resolve Instantaneous Cycle errors and smooth
				if (InstCycle[iBar] == 0) InstCycle[iBar] = InstCycle[iBar-1];
				Value5[iBar] = .25*InstCycle[iBar] + .75*Value5[iBar-1];
				Cycle = (int) Value5[iBar];

				//Compute Dominant Cycle Phase, Sine of the Phase Angle, and Leadsine			
				RealPart = 0;
				ImagPart = 0;
				for (int count = 0; count<Cycle; count++)
				{
					RealPart = RealPart + Math.Sin(2*Math.PI * count / Cycle) * (adPrice[iBar-count]);
					ImagPart = ImagPart + Math.Cos(2*Math.PI * count / Cycle) * (adPrice[iBar-count]);
				}
				if (Math.Abs(ImagPart) > 0.001) DCPhase = Math.Atan(RealPart / ImagPart);
				if (Math.Abs(ImagPart) <= 0.001) DCPhase = (Math.PI/2) * Math.Sign(RealPart);

				DCPhase = DCPhase + (Math.PI/2);
				if (ImagPart < 0) DCPhase = DCPhase + Math.PI;
				if (DCPhase > (1.75*Math.PI)) DCPhase = DCPhase - 2*Math.PI;
	
				adSine[iBar] = Math.Sin(DCPhase);
				adLeadSine[iBar] = Math.Sin(DCPhase + 0.25*Math.PI);
			}


            // Saving the components
            Component = new IndicatorComp[4];

            Component[0] = new IndicatorComp();
            Component[0].CompName   = "Sine Wave";
            Component[0].DataType   = IndComponentType.IndicatorValue;
            Component[0].ChartType  = IndChartType.Line;
            Component[0].ChartColor = Color.Green;
            Component[0].FirstBar   = iFirstBar;
            Component[0].Value      = adSine;

            Component[1] = new IndicatorComp();
            Component[1].CompName   = "Lead Sine";
            Component[1].DataType   = IndComponentType.IndicatorValue;
            Component[1].ChartType  = IndChartType.Line;
            Component[1].ChartColor = Color.Red;
            Component[1].FirstBar   = iFirstBar;
            Component[1].Value      = adLeadSine;

            Component[2] = new IndicatorComp();
            Component[2].ChartType = IndChartType.NoChart;
            Component[2].FirstBar  = iFirstBar;
            Component[2].Value     = new double[Bars];

            Component[3] = new IndicatorComp();
            Component[3].ChartType = IndChartType.NoChart;
            Component[3].FirstBar  = iFirstBar;
            Component[3].Value     = new double[Bars];

            // Sets the Component's type
            if (slotType == SlotTypes.OpenFilter)
            {
                Component[2].DataType = IndComponentType.AllowOpenLong;
                Component[2].CompName = "Is long entry allowed";
                Component[3].DataType = IndComponentType.AllowOpenShort;
                Component[3].CompName = "Is short entry allowed";
            }
            else if (slotType == SlotTypes.CloseFilter)
            {
                Component[2].DataType = IndComponentType.ForceCloseLong;
                Component[2].CompName = "Close out long position";
                Component[3].DataType = IndComponentType.ForceCloseShort;
                Component[3].CompName = "Close out short position";
            }

            switch (IndParam.ListParam[0].Text)
            {
                case "The Sinewave rises":
                    OscillatorLogic(iFirstBar, iPrvs, adSine, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_rises);
                    break;

                case "The Sinewave falls":
                    OscillatorLogic(iFirstBar, iPrvs, adSine, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_falls);
                    break;

                case "The Sinewave is higher than zero":
                    OscillatorLogic(iFirstBar, iPrvs, adSine, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_is_higher_than_the_level_line);
                    break;

                case "The Sinewave is lower than zero":
                    OscillatorLogic(iFirstBar, iPrvs, adSine, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_is_lower_than_the_level_line);
                    break;

                case "The Sinewave crosses the zero line upward":
                    OscillatorLogic(iFirstBar, iPrvs, adSine, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_crosses_the_level_line_upward);
                    break;

                case "The Sinewave crosses the zero line downward":
                    OscillatorLogic(iFirstBar, iPrvs, adSine, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_crosses_the_level_line_downward);
                    break;

                case "The Sinewave changes its direction upward":
                    OscillatorLogic(iFirstBar, iPrvs, adSine, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_changes_its_direction_upward);
                    break;

                case "The Sinewave changes its direction downward":
                    OscillatorLogic(iFirstBar, iPrvs, adSine, 0, 0, ref Component[2], ref Component[3], IndicatorLogic.The_indicator_changes_its_direction_downward);
                    break;

                case "The Sinewave crosses the Signal line upward":
                    IndicatorCrossesAnotherIndicatorUpwardLogic(iFirstBar, iPrvs, adSine, adLeadSine, ref Component[2], ref Component[3]);
                    break;

                case "The Sinewave crosses the Signal line downward":
                    IndicatorCrossesAnotherIndicatorDownwardLogic(iFirstBar, iPrvs, adSine, adLeadSine, ref Component[2], ref Component[3]);
                    break;

                case "The Sinewave is higher than the Signal line":
                    IndicatorIsHigherThanAnotherIndicatorLogic(iFirstBar, iPrvs, adSine, adLeadSine, ref Component[2], ref Component[3]);
                    break;

                case "The Sinewave is lower than the Signal line":
                    IndicatorIsLowerThanAnotherIndicatorLogic(iFirstBar, iPrvs, adSine, adLeadSine, ref Component[2], ref Component[3]);
                    break;

                default:
                    break;
            }

            return;
        }

        /// <summary>
        /// Sets the indicator logic description
        /// </summary>
        public override void SetDescription(SlotTypes slotType)
        {
            EntryFilterLongDescription  = ToString() + "; the Sinewave ";
            EntryFilterShortDescription = ToString() + "; the Sinewave ";
            ExitFilterLongDescription   = ToString() + "; the Sinewave ";
            ExitFilterShortDescription  = ToString() + "; the Sinewave ";

            switch (IndParam.ListParam[0].Text)
            {
                case "The Sinewave rises":
                    EntryFilterLongDescription  += "rises";
                    EntryFilterShortDescription += "falls";
                    ExitFilterLongDescription   += "rises";
                    ExitFilterShortDescription  += "falls";
                    break;

                case "The Sinewave falls":
                    EntryFilterLongDescription  += "falls";
                    EntryFilterShortDescription += "rises";
                    ExitFilterLongDescription   += "falls";
                    ExitFilterShortDescription  += "rises";
                    break;

                case "The Sinewave is higher than zero":
                    EntryFilterLongDescription  += "is higher than the zero line";
                    EntryFilterShortDescription += "is lower than the zero line";
                    ExitFilterLongDescription   += "is higher than the zero line";
                    ExitFilterShortDescription  += "is lower than the zero line";
                    break;

                case "The Sinewave is lower than zero":
                    EntryFilterLongDescription  += "is lower than the zero line";
                    EntryFilterShortDescription += "is higher than the zero line";
                    ExitFilterLongDescription   += "is lower than the zero line";
                    ExitFilterShortDescription  += "is higher than the zero line";
                    break;

                case "The Sinewave crosses the zero line upward":
                    EntryFilterLongDescription  += "crosses the zero line upward";
                    EntryFilterShortDescription += "crosses the zero line downward";
                    ExitFilterLongDescription   += "crosses the zero line upward";
                    ExitFilterShortDescription  += "crosses the zero line downward";
                    break;

                case "The Sinewave crosses the zero line downward":
                    EntryFilterLongDescription  += "crosses the zero line downward";
                    EntryFilterShortDescription += "crosses the zero line upward";
                    ExitFilterLongDescription   += "crosses the zero line downward";
                    ExitFilterShortDescription  += "crosses the zero line upward";
                    break;

                case "The Sinewave changes its direction upward":
                    EntryFilterLongDescription  += "changes its direction upward";
                    EntryFilterShortDescription += "changes its direction downward";
                    ExitFilterLongDescription   += "changes its direction upward";
                    ExitFilterShortDescription  += "changes its direction downward";
                    break;

                case "The Sinewave changes its direction downward":
                    EntryFilterLongDescription  += "changes its direction downward";
                    EntryFilterShortDescription += "changes its direction upward";
                    ExitFilterLongDescription   += "changes its direction downward";
                    ExitFilterShortDescription  += "changes its direction upward";
                    break;

                case "The Sinewave is higher than the Signal line":
                    EntryFilterLongDescription  += "is higher than the Signal line";
                    EntryFilterShortDescription += "is lower than the Signal line";
                    ExitFilterLongDescription   += "is higher than the Signal line";
                    ExitFilterShortDescription  += "is lower than the Signal line";
                    break;

                case "The Sinewave is lower than the Signal line":
                    EntryFilterLongDescription  += "is lower than the Signal line";
                    EntryFilterShortDescription += "is higher than the Signal line";
                    ExitFilterLongDescription   += "is lower than the Signal line";
                    ExitFilterShortDescription  += "is higher than the Signal line";
                    break;

                case "The Sinewave crosses the Signal line upward":
                    EntryFilterLongDescription  += "crosses the Signal line upward";
                    EntryFilterShortDescription += "crosses the Signal line downward";
                    ExitFilterLongDescription   += "crosses the Signal line upward";
                    ExitFilterShortDescription  += "crosses the Signal line downward";
                    break;

                case "The Sinewave crosses the Signal line downward":
                    EntryFilterLongDescription  += "crosses the Signal line downward";
                    EntryFilterShortDescription += "crosses the Signal line upward";
                    ExitFilterLongDescription   += "crosses the Signal line downward";
                    ExitFilterShortDescription  += "crosses the Signal line upward";
                    break;

                default:
                    break;
            }

            return;
        }

        /// <summary>
        /// Indicator to string
        /// </summary>
        public override string ToString()
        {
            string sString = IndicatorName +
                (IndParam.CheckParam[0].Checked ? "* (" : " (") +
                IndParam.NumParam[0].ValueToString + ")";  // Sine period

            return sString;
        }
    }
}
